Package: Rrepest
Title: An Analyzer of International Large Scale Assessments in
        Education
Version: 1.6.14
Authors@R: 
    c(person("Rodolfo", "Ilizaliturri", , "rodolfo.ilizaliturri@oecd.org", role = c("aut", "cre")),
    person("Francesco", "Avvisati", , "francesco.avvisati@oecd.org", role = c("aut")),
    person("Francois", "Keslair", , "francois.keslair@oecd.org", role = c("aut")))
Description: An easy way to analyze international large-scale assessments and surveys in education or any other dataset that includes replicated weights (Balanced Repeated Replication (BRR) weights, Jackknife replicate weights,...) while also allowing for analysis with multiply imputed variables (plausible values). It supports the estimation of univariate statistics (e.g. mean, variance, standard deviation, quantiles), frequencies, correlation, linear regression and any other model already implemented in R that takes a data frame and weights as parameters. It also includes  options to prepare the results for publication, following the table formatting standards of the Organization for Economic Cooperation and Development (OECD).
Depends: R (>= 4.2.0)
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.3.3
Imports: data.table (>= 1.14.8), doParallel (>= 1.0.17), dplyr (>=
        1.1.2), flextable (>= 0.7.2), foreach (>= 1.5.2), labelled (>=
        2.9.1), magrittr (>= 2.0.3), officer (>= 0.6.2), parallel (>=
        4.2.1), purrr (>= 0.3.4), rlang (>= 1.1.6), stringr (>= 1.5.0),
        tibble (>= 3.1.8), tidyr (>= 1.2.0)
LazyData: true
NeedsCompilation: no
Packaged: 2026-06-12 08:52:02 UTC; Ilizaliturri_R
Author: Rodolfo Ilizaliturri [aut, cre],
  Francesco Avvisati [aut],
  Francois Keslair [aut]
Maintainer: Rodolfo Ilizaliturri <rodolfo.ilizaliturri@oecd.org>
Repository: CRAN
Date/Publication: 2026-06-12 13:10:12 UTC
Built: R 4.5.2; ; 2026-06-12 15:34:01 UTC; unix

Package: Rrepest
Title: An Analyzer of International Large Scale Assessments in
        Education
Version: 1.6.14
Authors@R: 
    c(person("Rodolfo", "Ilizaliturri", , "rodolfo.ilizaliturri@oecd.org", role = c("aut", "cre")),
    person("Francesco", "Avvisati", , "francesco.avvisati@oecd.org", role = c("aut")),
    person("Francois", "Keslair", , "francois.keslair@oecd.org", role = c("aut")))
Description: An easy way to analyze international large-scale assessments and surveys in education or any other dataset that includes replicated weights (Balanced Repeated Replication (BRR) weights, Jackknife replicate weights,...) while also allowing for analysis with multiply imputed variables (plausible values). It supports the estimation of univariate statistics (e.g. mean, variance, standard deviation, quantiles), frequencies, correlation, linear regression and any other model already implemented in R that takes a data frame and weights as parameters. It also includes  options to prepare the results for publication, following the table formatting standards of the Organization for Economic Cooperation and Development (OECD).
Depends: R (>= 4.2.0)
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.3.3
Imports: data.table (>= 1.14.8), doParallel (>= 1.0.17), dplyr (>=
        1.1.2), flextable (>= 0.7.2), foreach (>= 1.5.2), labelled (>=
        2.9.1), magrittr (>= 2.0.3), officer (>= 0.6.2), parallel (>=
        4.2.1), purrr (>= 0.3.4), rlang (>= 1.1.6), stringr (>= 1.5.0),
        tibble (>= 3.1.8), tidyr (>= 1.2.0)
LazyData: true
NeedsCompilation: no
Packaged: 2026-06-12 08:52:02 UTC; Ilizaliturri_R
Author: Rodolfo Ilizaliturri [aut, cre],
  Francesco Avvisati [aut],
  Francois Keslair [aut]
Maintainer: Rodolfo Ilizaliturri <rodolfo.ilizaliturri@oecd.org>
Repository: CRAN
Date/Publication: 2026-06-12 13:10:12 UTC

# Generated by roxygen2: do not edit by hand

export(Rrepest)
export(coverage_daggers)
export(coverage_pct)
export(est)
export(format_data_categ_vars)
export(format_data_cont_vars)
export(format_data_repest)
export(grouped_sum_freqs)
export(grp)
export(indep_diff)
export(inv_test)
export(is_prime)
export(n_obs_x)
export(paired_indep_diff)
export(weighted.corr)
export(weighted.corr.cov.n)
export(weighted.cov)
export(weighted.iqr)
export(weighted.mode)
export(weighted.quant)
export(weighted.std)
export(weighted.var)
import(data.table, except = c(first,last,between,transpose))
import(doParallel)
import(dplyr)
import(flextable)
import(foreach)
import(labelled)
import(magrittr, except = c(set_names,extract))
import(officer)
import(parallel)
import(purrr, except = c(when,compose,accumulate))
import(rlang, except = c("flatten_lgl","splice","flatten_chr","flatten_raw","flatten","flatten_dbl","invoke","flatten_int",":=","%@%"))


import(stringr)
import(tibble)
import(tidyr)
importFrom(stats,aggregate)
importFrom(stats,as.formula)
importFrom(stats,coefficients)
importFrom(stats,glm.fit)
importFrom(stats,lm.wfit)
importFrom(stats,na.omit)
importFrom(stats,qnorm)
importFrom(stats,quasibinomial)
importFrom(stats,resid)
importFrom(stats,runif)
importFrom(stats,setNames)
importFrom(stats,weighted.mean)
importFrom(utils,combn)
importFrom(utils,head)

# Rrepest
>A way to run estimations with weighted replicate samples and plausible values

## Table of Contents

* Description
* Installation
* Documentation
* Examples of use cases
* Authors
* Contributing

## Description

It estimates statistics using replicate weights (*Balanced Repeated Replication (BRR) weights*, *Jackknife replicate weights*,...), thus accounting for complex survey designs in the estimation of sampling variances. It is designed specifically for use with the international education datasets produced by the **OECD** (e.g. PIAAC, PISA, SSES, TALIS, etc.), but works for any educational large-scale assessment and survey that uses replicated weights (e.g. ICCS, ICILS, PIRLS, TIMSS - all produced by IEA). It also allows for analyses with multiply imputed variables (plausible values); where plausible values are used, the average estimator across plausible values is reported and the imputation error is added to the variance estimator.

## Installation

### Using CRAN (latest official version)

Run the following code:
``` r
install.packages("Rrepest")
```

### Using tar.gz file (latest development version)

[Download Rrepest](https://gitlab.algobank.oecd.org/edu_data/rrepest/-/blob/main/Development/Rrepest.tar.gz),
then run

Run the following code replacing "Your_Path" with your path to the tar file:
``` r
install.packages("Your_Path/Rrepest.tar.gz",
repos = NULL,
type ="source")
```

Run:
``` r
library(Rrepest)
```

### Using a GitLab token (latest development version)

Run the following code replacing "MY_TOKEN" with your gitlab token:
``` r
remotes::install_gitlab("edu_data/rrepest", host = "https://algobank.oecd.org:4430", upgrade = "never", auth_token = "MY_TOKEN")
```
Note: It will take a few minutes to install.

Run:
``` r
library(Rrepest)
```
Note: Ensure you have the package `data.table` installed. For a complete list of the dependencies used, consult the [Description](https://gitlab.algobank.oecd.org/edu_data/rrepest/-/blob/main/DESCRIPTION) file.

## Documentation

* **Full documentation** of `Rrepest` is available [here](https://CRAN.R-project.org/package=Rrepest/Rrepest.pdf).
* **Cheat sheet** including an overview of the syntax and auxiliaries of `Rrepest` is available [here](https://gitlab.algobank.oecd.org/edu_data/rrepest/-/blob/main/Development/Rrepest_cheat_sheet.pdf).
* Information on **how to incorporate analyses that are not pre-programmed** into `Rrepest` is available in the following [wiki](https://gitlab.algobank.oecd.org/edu_data/rrepest/-/wikis/General-Analysis).

## Examples of use cases
`Rrepest` supports **summary statistics** (i.e. mean, variance, standard deviation, quantiles, inter-quantile range), **frequency count**, **correlation**, **linear regression** and any other statistics that are not pre-programmed into `Rrepest` but take a data frame and weights as parameters (see **General analysis** below). `Rrepest` also has optional features that provide means, among others, to specify the level of analysis, obtain estimates for each level of a given categorical variable, test for differences, flag estimates that are based on fewer observations than required for reporting, compute averages. More detail on the optional features of `Rrepest` can be found [here](https://CRAN.R-project.org/package=Rrepest/Rrepest.pdf).

### Summary statistics
``` r
# PISA 2018 Data
# df.qqq <- readRDS("//oecdmain/asgenedu/EDUCATION_DATALAKE/sources/PISA/PISA 2018/R/STU/CY07_MSU_STU_QQQ.rds")

Rrepest::Rrepest(data = df.qqq,
        svy = "PISA2015",
        est = est(c("mean","var","std","quant",0.5,"iqr",c(.9,.1)),"age"),
        by = c("cnt"))

```
### Frequency count
``` r
# TALIS 2018 Data
# df.t <- readRDS("//oecdmain/asgenedu/EDUCATION_DATALAKE/sources/TALIS/2018/R/International/TTGINTT3.rds")

Rrepest::Rrepest(data = df.t,
                 svy = "TALISTCH",
                 est = est("freq","tt3g01"),
                 by = "cntry")
```
### Correlation
``` r
# PISA 2018 Data
# df.qqq <- readRDS("//oecdmain/asgenedu/EDUCATION_DATALAKE/sources/PISA/PISA 2018/R/STU/CY07_MSU_STU_QQQ.rds")

Rrepest::Rrepest(data = df.qqq,
        svy = "PISA2015",
        est = est("corr",c("pv@math","pv@read")),
        by = c("cnt"))

```

### Linear regression
``` r
# TALIS 2018 Data
# df.t <- readRDS("//oecdmain/asgenedu/EDUCATION_DATALAKE/sources/TALIS/2018/R/International/TTGINTT3.rds")

df.t <- df.t %>% 
        mutate(TT3G01_rec = case_when(TT3G01 == 2 ~ 1,
                                      TT3G01 == 1 ~ 0))

Rrepest::Rrepest(data = df.t,
        svy = "TALISTCH",
        est = est("lm","tt3g01_rec","tt3g39c"),
        by = "cntry")
```

Further examples can be found in the [Examples.R](https://gitlab.algobank.oecd.org/edu_data/rrepest/-/blob/main/Development/Examples.R) file.

### General analysis
To incorporate analyses that are not pre-programmed into Rrepest, you can utilize the **'gen'** option within the `est()` function of Rrepest. Any line of code that takes a data frame and weights as parameters can be used with the **'gen'** option. For more information, please see the following [wiki](https://gitlab.algobank.oecd.org/edu_data/rrepest/-/wikis/General-Analysis).

# Authors

[Francesco Avvisati](mailto:francesco.avvisati@oecd.org), [Rodolfo Ilizaliturri](mailto:Rodolfo.ILIZALITURRI@oecd.org) and  [François Keslair](mailto:francois.keslair@oecd.org). 

Contact us if you want to join!


## Contributing

Do you have suggestions or comments? Please open an [issue](https://gitlab.algobank.oecd.org/edu_data/rrepest/-/issues).

################# Invert Test Column #################
# 15 Sept 2023
# Rodolfo Ilizaliturri
#############################################################

#' inv_test
#' @description Invert test column from Rrepest test = TRUE by name on "b." and "se." in the column name and by sign (*-1) on "b."
#' @param data (dataframe) df to analyze
#' @param name_index (string/numeric) name or index for the estimate (b.) columns containing the data for the test in Rrepest)
#'
#' @return Dataframe cointaining inverted test column names for "b." and "se." according to Rrepest structure and column multiplied by (-1) for "b."
#' @export
#'
#' @examples
#' inv_test(rrepest_pisa_age_gender, name_index = 6) 
inv_test <- function(data, name_index){
  # Goal: Invert test column from Rrepest by name on b. and se. in the test part in the column name and by sign (*-1) on b.
  # ------ INPUTS ------.
  # data : (dataframe) df to analize
  # name_index : (string/numeric) name or index for the estimate (b.) columns containing the data for the test in Rrepest
  
  rev_col_name <- function(input_string){
    # Split the string on the last ".."
    split_parts <- strsplit(input_string, "\\.{2}(?!.*\\.{2})", perl = TRUE) %>% unlist()
    # Grab the second part where the test string is
    test_parts <- substr(split_parts[2],2,nchar(split_parts[2])-1) %>% strsplit(split = "-") %>% unlist
    # Inversed result
    output_string <- paste0(split_parts[1],"..(",test_parts[2],"-",test_parts[1],")")
    return(output_string)
  }
  if (is.numeric(name_index)){ #Index -> Numeric
    # Get name 
    input_string <- data %>% select(name_index, name_index+1) %>% names()
    # Vectorize rev_col_name
    output_string <- sapply(input_string, rev_col_name, USE.NAMES = FALSE)
    # Rename original column
    data <- data %>% rename(!!!setNames(input_string,output_string))
    # Invert sign of column
    data <- data %>% mutate_at(name_index, ~ . * -1) 
  } else if (is.character(name_index)) { #Name -> Characheter
    # Get name 
    input_string <- data %>% select(which(names(data)==name_index), which(names(data)==name_index)+1) %>% names()
    # Vectorize rev_col_name
    output_string <- sapply(input_string, rev_col_name, USE.NAMES = FALSE)
    # Rename original column
    data <- data %>% rename(!!!setNames(input_string,output_string))
    # Invert sign of column
    data <- data %>% mutate_at(which(names(data)==name_index), ~ . * -1) 
  }
  return(data)
}

################# Functionalize General option and PVs #################
# 19 Jun 2023
# Rodolfo Ilizaliturri
#############################################################
# Goal: Obtain a dataframe with any R like input set into Rrepest
# REQUIREMENTS:
#   - Have at least one argument where you input weights as w_i
#   - Have an argument where you input the data as d_i
#   - Must return a named data frame
#     - Extreme case: If no over or by arguments, it would return a 1Xn named df
#   - Optional: A flextable with a column name prefix indication what you are doing deparated by a point (Ex. reg_age.(Intercept))
#   - PVs: intidace with an @ what you would like to be evaluated as a Plausible and indicate what you are doing in pv at begining of column name

pv.do.gen <- function(df, y, by.var, w, ...){
  # Goal: Get coefficients and r2 from linear regression
  # ------ INPUTS ------.
  # df : (dataframe) df to analyze previosly formated
  # by.var : (string) column in which we'll break down results !IOP!: several variables
  # y : (string) Deparsed method to include in normal r script
  # w : (string) weighting variable (just 1)
  
  # Only colnames of the same # of characters as my replicated weight name
  col_names <- subset(names(df), nchar(names(df)) == nchar(w))
  # Grab closest non case sensitive weight name
  w_sub <- col_names[grepl(w, col_names,ignore.case = T)]
  # Add warning for missing weights
  if(length(w_sub) == 0) message(w," does not exist in dataset")
  message(as.character(w_sub)) #indicate wich w you are in
  # Replace what is w_gen with the replicated weight needed
  y <- sub("w_gen",w_sub,y)
  #model only fitting values. Out: Coeff and R2 ONLY
  res.df <- df %>% 
    group_by(across(all_of(by.var))) %>% 
    do({
      #Parsed script model REQUIREMENTS 
      eval(parse(text = y))
    }
    ) %>%
    ungroup()
  return(res.df)
}
pv.do.gen.PAR <- function(df, y, by.var, w, ...){
  # Goal: Get coefficients and r2 from linear regression
  # ------ INPUTS ------.
  # df : (dataframe) df to analyze previosly formated
  # by.var : (string) column in which we'll break down results !IOP!: several variables
  # y : (string) Deparsed method to include in normal r script
  # w : (string) weighting variable (just 1)
  # ...
  
  # get ... arguments
  arg <- list(...)

  # Only colnames of the same # of characters as my replicated weight name
  col_names <- subset(names(df), nchar(names(df)) == nchar(w))
  # Grab closest non case sensitive weight name
  w_sub <- col_names[grepl(w, col_names,ignore.case = T)]
  # Add warning for missing weights
  if(length(w_sub) == 0) message(w," does not exist in dataset")
  
  message(as.character(w_sub)) #indicate which w you are in
  
  # Replace what is w_gen with the replicated weight needed
  y <- sub("w_gen",w_sub,y)
  
  #model only fitting values. Out: Coeff and R2 ONLY
  res.df <- df %>% 
    group_by(across(all_of(by.var))) %>% 
    do({
      #Parsed script model REQUIREMENTS 
      eval(parse(text = y))
    }
    ) %>%
    ungroup()
  
  return(res.df)
}
pv.loop.gen.on.weights <- function (data, x, y, by.var, over, test = F, flag = F,
                                   svy, rep_weights, fast = F, pv = F) {
  # Goal: Regression model results for each weight
  # ------ INPUTS ------.
  # data : (dataframe) df to analize
  # rep_weights : (string vector) names of replicated weight vars
  # reg.model : (expression) linear model to implement
  # by.var : (string vector) variables to break analysis by
  # fast : (bool) TRUE → Only do 6 replicated weights
  # pv : (Bool) TRUE → We are in plausible values and must parallelize
  
  # If fast then less replicated weights
  if (fast) {
    rep_weights <- rep_weights[1:6]
  }
  
  if (!pv) {
    # NON PARALLEL ------------------------------------------------------------.
    
    res.l <- lapply(rep_weights, function(w.i){
      #Do gen, add weight to colnames, append new data of results to list
      res.df <- pv.do.gen(data, 
                          y, 
                          all_of(c(by.var,over)), 
                          w.i) %>% 
        unite("by.var", all_of(c(by.var,over)), sep = "|")
      
      # Change col names to include weight name
      # colnames(res.df)[-1] <- paste0(colnames(res.df)[-1], paste0("|",w.i))
      
      return(res.df)
    })
    # PARALLEL ---------------------------------------------------------------.
  }else{
    res.l <- foreach(w.i = rep_weights,
                     .packages = c("dplyr","tidyr","data.table","tibble"),
                     .export = c("pv.do.gen.PAR","n.obs.x",
                                 "weighted.var")) %dopar% {
                                   
                                   res.df <- pv.do.gen.PAR(df = data.par,
                                                           y = y,
                                                           by.var = all_of(c(by.var,over)),
                                                           w = w.i) %>% 
                                     unite("by.var", all_of(c(by.var,over)), sep = "|")
                                   return(res.df)
                                 }
    
  }
  
  
  # ---------------- FLAGS
  if (flag) {
    # Get n for flags and separate column
    # NOTE!!!!!!: Here x = y since the target variable in regressions is the y
    n.df <- n.obs.x(df = data, by.var = c(by.var, over), x = y, svy = svy)
    
    res.l <- lapply(res.l, function(res.i){
      # Merge both dfs together n and freqs
      res.df <- left_join(res.i, n.df, by = c("by.var"="by.group"))
      
      #Assign NaN when not enough coverage for estimation (flags.nan is deprecated)
      res.df <- flags(data = res.df, svy = svy)
      
      return(res.df)
    })
  }
  # ---------------- .
  
  # ---------------- TEST
  if (test) {
    # Create extra rows with test differences
    res.l <- lapply(res.l, function(res.i){
      # Split by.var into c(by.var, over, x) by |
      res.df <- res.i %>% 
        separate(col = "by.var", into = all_of(c(by.var, over)), sep = "\\|") %>% 
        over.test(over = c(over)) %>% 
        rename("by.var"="by.group")
      
      return(res.df)
    })
  } 
  # ---------------- .
  # ---------------- FORMAT DF to have over on the columns
  if (length(over)>0) {
    res.l <- lapply(res.l, function(res.i){
      res.df <- res.i %>% 
        # Split by.var leaving only the variables in "by.var" united 
        separate(col = "by.var", into = all_of(c(by.var, over)), sep = "\\|") %>% 
        unite(col = "by.var", all_of(by.var), sep = "|") %>% 
        # Pivot to columns over
        pivot_wider(names_from = all_of(c(over)),
                    values_from = colnames(select_if(.,is.numeric)),
                    names_sep = "..")
      
      return(res.df)
    })
  }
  # ---------------- .
  # ---------------- ASSIGN reg_{name.of.y} to each dataframe
  # res.l <- lapply(res.l, function(res.i){
  #   res.i <- res.i
  #   colnames(res.i)[-1] <- paste0("reg_", y,".", colnames(res.i)[-1])
  #   return(res.i)
  # })
  
  return(res.l)
}


# --------- FINAL LINEAR MODEL FUNCTION

pv.rrepest.gen <- function(data,  svy, y, by.var = NULL, over = NULL, test = F, 
                          user_na=F, flag = F, fast = F, pv = F, ...) {
  # Goal: Dataframe with β and SE by Fay's BRR model for linear regression
  # ------ INPUTS ------.
  # data : (dataframe) df to analyze
  # svy : List of possible projects to analyse PIAAC, PISA, TALISSCH and TALISTCH
  # isced : (number) isced level to analyze
  # by.var : (string) column in which we'll break down results
  # y : (string) method to include in normal r script
  # user_na : (Bool) TRUE → show nature of user defined missing values in by.var
  # over : (vector string) columns over which to do analysis
  # test : (bool) If TRUE will calculate the difference between over variables
  # flag : (Bool) TRUE → Show NaN when there is not enough cases (or schools)
  # fast : (bool) TRUE → Only do 6 replicated weights
  # pv : (Bool) TRUE → We are in plausible values and must parallelize
  # ...
  # isced : (number) isced level to analyze
  
  # Get optional arguments
  extra.args <- list(...)
  
  # If there is an OVER variable remove NAs from OVER vars
  # if (length(over) > 0) {
  #   for (i in over) {
  #     df <- df %>% drop_na(i)
  #   }
  # }
  
  # Get weight names
  weight.names <- replicated_w_names(svy, ...)
  
  # Loop over each variable
  brr.res <- pv.loop.gen.on.weights(data = data,
                                   y = y,
                                   svy = svy,
                                   by.var = by.var,
                                   over = over,
                                   test = test,
                                   flag = flag,
                                   rep_weights = weight.names,
                                   fast = fast,
                                   pv = pv)
  
  # Reorder data to have b and se intertwined
  res <- pv.get.se.reorder(brr.res, svy = svy, pv = pv, ... = ...)
  
  return(res)
}

################# Frequencies on categoric variables PVs #################
# 1 April 2023
# Rodolfo Ilizaliturri
#############################################################
# Goal: Obtain frequencies using for replicated weights and/or plausible values


# --------- WORK FUNCTIONS
# ------------------.

pv.get.se.reorder <- function(brr.res, svy, pv = F, ...) {
  # Goal: Dataframe with coefficients and standard error of statistics from model
  # ------ INPUTS ------.
  # brr.res: (list of DFs) A list of dataframes in tidy format with 1st colum by.var
  # and next columns all wanted statistics from model. Each element is statistics for
  # each weight
  # svy : (string) List of possible projects to analyse PIAAC, PISA, TALISSCH and TALISTCH
  # pv : (Bool) TRUE → We are in plausible values and keep calculation as variance, not SE
  
  betas <- brr.res[[1]] 
  #Get indexes of columns minus by.var
  cols.data <- seq_along( colnames(brr.res[[1]][-1]) ) + 1 
  #Get raw column names
  se.c.names <- sapply(colnames(brr.res[[1]]), function (x) strsplit(x,
                                                                     split = "[|]")[[1]][1])
  #Paste colnames and .b on betas
  colnames(betas)[-1] <- paste0("b.",se.c.names[-1])
  
  
  #Empty list > for all columns but by.bar > extract same col on all dfs 
  #> join them and store them . Each element of list is statistic on T and i weights
  brr.beta = list()
  for (i in cols.data) {
    res <- brr.res %>% 
      lapply(function(x) x[c(1,i)]) %>% 
      reduce(full_join, by = "by.var")
    
    brr.beta[[i-1]] <- res
  }
  #1st col to rnames> rrvar on each element > rnames to by.var > join them all
  se.beta <- lapply(brr.beta, function(x) {
    rowname.x <- pull(x[1]) 
    x <- data.frame(x[,-1])
    rownames(x) <- rowname.x
    apply(x,1,rrvar.pv, svy = svy, pv = pv, ... = ...) %>% 
      as.data.frame() %>% 
      rownames_to_column("by.var")
  }) %>% 
    reduce(full_join, by = "by.var")
  
  #rename se.beta colnames to include .se and join
  colnames(se.beta)[-1] <- paste0("se.",se.c.names[-1])
  res <- full_join(betas, se.beta, by="by.var")
  
  #reorder df with intercalating columns
  n.betas <- (ncol(res) - 1)/2
  betas.order <- lapply(1:n.betas,function(x) c(x,x+n.betas)) %>% 
    reduce(function (x,y) c(x,y))
  res <- res[,c(1,betas.order + 1)]
  
  return(res)
}

#' Within group frequencies of categories in a column.
#'
#' @param data (data frame) Data to analyze.
#' @param small.level (string vector) all variables to get grouped sum.
#' @param big.level (string vector) Must be fully contained in variables from small.level
#' @param w (string) Numeric variable from which to get weights (if NULL then 1).
#'
#' @return Data frame with frequencies from the grouped sum of small.level and big.level used for getting percentages.
#' @export
#'
#' @examples grouped_sum_freqs(data = mtcars,small.level = c("cyl","gear"),big.level = c("cyl"))
grouped_sum_freqs <- function(data, small.level, big.level, w = NULL) {
  # Goal: Get a dataframe with frequencies from the grouped sum of small.level and big.level used for getting percentages
  # Example: cnt gender wealth 
  # ------ INPUTS ------.
  # data : (dataframe) df to analyze
  # w : (string) numeric variable from which to get weights (if NULL then 1)
  # small.level : (string vector) all variables to get grouped sum
  # big.level : (string vector) Must be fully contained in variables from small.level 
  
  # If no weights then add 1s
  if(is.null(w)) {
    data$weight <- 1
    w <-"weight"
  }
  
  #Small group
  small.group <- data %>% 
    group_by(across(all_of(small.level))) %>% 
    summarise(small.sum = sum(get(w), na.rm = T)) %>% 
    ungroup() %>% # Complete missing combinations of data
    complete(!!! rlang::syms(small.level), fill = list(small.sum = 0))
  
  #Bigger group with variable list fully contained in small group
  big.group <- data %>% 
    group_by(across(all_of(big.level))) %>% 
    summarise(big.sum = sum(get(w), na.rm = T)) %>% 
    ungroup() %>% # Complete missing combinations of data
    complete(!!! rlang::syms(big.level), fill = list(small.sum = 0))
  
  
  # Do frequencies in 100% format. Resulting variable is "freq"
  res <- full_join(small.group,big.group,by = big.level) %>%
    mutate(freq = small.sum / big.sum * 100) %>% 
    select(-small.sum,-big.sum) %>% 
    
  
  return(res)
}


row.pct.byvar.x <- function(data, x, by.var, over, test = F, flag = F, w, ...) {
  # Goal: Dataframe with coefficients and standard error of statistics from model
  # ------ INPUTS ------.
  # data : (dataframe) df to analize
  # w : (string) name of replicated weight var
  # x : (string) lvariable from which to get frequencies
  # by.var : (string vector) variables to break analysis by
  # over : (vector string) columns over which to do analysis
  # test : (bool) If TRUE will calculate the difference between over variables
  
  res.df <- grouped_sum_freqs(data = data,
                              w = w,
                              small.level = c(by.var, over, x),
                              big.level = c(by.var, over)) %>% 
    ungroup()
  
  # Put together all categorical variables into by.var
  res.df <- res.df %>% 
    unite("by.var", all_of(c(by.var, over, x)), sep = "|")
  
  # Give names with weight to number columns
  #colnames(res.df) <- c(colnames(res.df)[1], 
  #                      paste0(colnames(res.df[-1]),paste0("|",w))) #separator must be | for pretty names
  
  return(res.df)
}

row.pct.byvar.x.PAR <- function(data, x, by.var, over, test = F, flag = F, w, ...) {
  # Goal: Dataframe with coefficients and standard error of statistics from model
  # ------ INPUTS ------.
  # data : (dataframe) df to analize
  # w : (string) name of replicated weight var
  # x : (string) lvariable from which to get frequencies
  # by.var : (string vector) variables to break analysis by
  # over : (vector string) columns over which to do analysis
  # test : (bool) If TRUE will calculate the difference between over variables
  # ...
  #(functions) grouped_sum_freqs
  arg <- list(...)
  
  res.df <- arg$grouped_sum_freqs(data = data,
                              w = w,
                              small.level = c(by.var, over, x),
                              big.level = c(by.var, over)) %>% 
    ungroup()
  
  # Put together all categorical variables into by.var
  res.df <- res.df %>% 
    unite("by.var", all_of(c(by.var, over, x)), sep = "|")
  
  # Give names with weight to number columns
  #colnames(res.df) <- c(colnames(res.df)[1], 
  #                      paste0(colnames(res.df[-1]),paste0("|",w))) #separator must be | for pretty names
  
  return(res.df)
}

pv.loop.freq.on.weights <- function(data, x, by.var, over, test = F, flag = F, svy, rep_weights, pv = F, ...) {
  # Goal: List with all weighted frequencies of x by by.var
  # ------ INPUTS ------.
  # data : (dataframe) df to analize
  # w : (string) name of replicated weight var
  # x : (string) lvariable from which to get frequencies
  # by.var : (string vector) variables to break analysis by
  # over : (vector string) columns over which to do analysis
  # test : (bool) If TRUE will calculate the difference between over variables
  # rep_weights : (string vector) names of replicated weight vars
  # flag : (Bool) TRUE → Show NaN when there is not enough coverage
  # svy : List of possible projects to analyse PIAAC, PISA, TALISSCH and TALISTCH
  # pv : (Bool) TRUE → We are in plausible values and must parallelize
  
  if (!pv) {
  # NON PARALLEL ------------------------------------------------------------.
    res.l <- lapply(rep_weights, function(w.i) {
      # would remove NAs from over variables here
      row.pct.byvar.x(data = data,
                      x =  x,
                      by.var =  by.var,
                      over =  over,
                      test = test,
                      flag = flag,
                      w =  w.i)
    })
  # PARALLEL ---------------------------------------------------------------.
  }else{
    res.l <- foreach(w.i = rep_weights,
                     .packages = c("dplyr","tidyr", "purrr"),
                     .export = c("row.pct.byvar.x.PAR","grouped_sum_freqs")) %dopar% {
                      # would remove NAs from over variables here             
                               
                       # same loop but paralelizing w.i
                       row.pct.byvar.x.PAR(data = data.par,
                                       x =  x,
                                       by.var =  by.var,
                                       over =  over,
                                       test = test,
                                       flag = flag,
                                       w =  w.i,
                                       grouped_sum_freqs = grouped_sum_freqs)
                     }
  }
  
  
  
  # ---------------- FLAGS
  if (flag) {
    # Get n for flags and separate column
    n.df <- n.obs.x(df = data, by.var = c(by.var, over, x), x = x, svy = svy)
    # Fix for frequencies
    n.df <- n_obs_sch_freq_fix(n_df = n.df, by.var = by.var, over = over, x = x)
    
    res.l <- lapply(res.l, function(res.i){
      # Merge both dfs together n and freqs
      res.df <- left_join(res.i, n.df, by = c("by.var"="by.group"))
      
      #Assign NaN when not enough coverage for estimation
      res.df <- flags(data = res.df, svy = svy)
      
      return(res.df)
    })
  }
  # ---------------- .
  
  # ---------------- TEST
  if (test) {
    # Create extra rows with test differences
    res.l <- lapply(res.l, function(res.i){
      # Split by.var into c(by.var, over, x) by |
      res.df <- res.i %>% 
        separate(col = "by.var", into = all_of(c(by.var, over, x)), sep = "\\|") %>% 
        over.test.freq(over = c(over, x)) %>% 
        rename("by.var"="by.group")
      
      return(res.df)
    })
  } 
  # ---------------- .
  # ---------------- FORMAT DF to have over and x on the columns
  res.l <- lapply(res.l, function(res.i){
    res.df <- res.i %>% 
      # Split by.var leaving only the variables in "by.var" united 
      separate(col = "by.var", into = all_of(c(by.var, over, x)), sep = "\\|") %>% 
      unite(col = "by.var", all_of(by.var), sep = "|") %>% 
      # Add variable name to column values
      mutate("{x}" := paste0(x,".",get(x))) %>% 
      # Pivot to columns over and x
      pivot_wider(names_from = all_of(c(x,over)),
                  # grab only numeric columns after by.var, over, and x
                  values_from = colnames(select_if(.,is.numeric)),
                  names_sep = "..")
    
    return(res.df)
  })
  # ---------------- .
  
  return(res.l)
}

# --------- FINAL FREQUENCIES FUNCTION
pv.rrepest.frequencies <- function(data, svy, x, by.var = NULL, over = NULL, test = F,
                                user_na=F, show_na=F, flag = F, fast = F, pv = F, ...) {
  # Goal: Dataframe with means and SE by Fay's BRR model
  # ------ INPUTS ------.
  # data : (dataframe) df to analyze
  # svy : List of possible projects to analyse PIAAC, PISA, TALISSCH and TALISTCH
  # by.var : (string vector) column in which we'll break down results
  # x : (string) variable from where to get frequencies
  # over : (vector string) columns over which to do analysis
  # test : (bool) If TRUE will calculate the difference between over variables
  # user_na : (Bool) TRUE → show nature of user defined missing values for by.var
  # show_na : (Bool) TRUE → include na in frequencies of x
  # flag : (Bool) TRUE → Show NaN when there is not enough cases (or schools)
  # fast : (bool) TRUE → Only do 6 replicated weights
  # pv : (Bool) TRUE → We are in plausible values and must parallelize
  # ...
  # isced : (number) isced level to analyze
  # pvdigits : (string/ numeric vector) pv.digits argument: # of pvs or @ if none
  
  # Get optional arguments
  extra.args <- list(...)
  

  # Remove NA's from tgt and over -------------------------------------------
  
  # If there is an OVER variable remove NAs from OVER vars
  if (length(get.pv.arguments(extra.args$pvdigits,over)) > 0) {
    if (!any(show_na,user_na)) {
      for (i in get.pv.arguments(extra.args$pvdigits,over)) {
        data <- data %>% drop_na(i)
      }
    }
  }
  
  # X is categorical in a frequency, must be formatted accordingly
  data <- format_data_categ_vars(data,
                               get.pv.arguments(extra.args$pvdigits, x),
                               show_na)
  
  # if there are PVs, export this new database to the cluster "cl"
  # if (pv) {
  #   clusterExport(cl, "data.par",envir = environment())
  # }
  
  
  # Get weight names
  weight.names <- replicated_w_names(svy, ...)
  # If fast then less replicated weights
  if (fast) {
    weight.names <- weight.names[1:6]
  }
  
  # Loop over each variable
  brr.res <- pv.loop.freq.on.weights(data = data,
                                  svy = svy,
                                  x = x, 
                                  by.var = by.var, 
                                  over = over,
                                  test = test,
                                  flag = flag,
                                  rep_weights = weight.names,
                                  pv = pv,
                                  ... = ...)
  
  
  
  # Reorder data to have b and se intertwined
  res <- pv.get.se.reorder(brr.res, svy = svy, pv = pv, ...)
  
  
  return(res)
  
}

################# PV B and SE calculation from LISTS #################
# 28 Mar 2023
# Rodolfo Ilizaliturri
#############################################################
# Goal: From a list of size n pvs, get Sampling Var, Imputation Var, SE and Estimate
# Note: Each dataframe must be a 1st column with by.var and all after a sequence of b., se. results


get.common.name <- function(str.vec, pv.inputs) {
  # Goal: Get common part of the string in a vector of strings
  # ------ INPUTS ------.
  # str.vec : (String vector) vector with the almost identical strs
  # pv.inputs : (string vector) All strings containing an @ as an input
  
  for (pv.in.i in pv.inputs){
    # Get regular expression instruction to replace string names
    #Separate by @ but keep the @ as separator
    reg.inst <- unlist(strsplit(pv.in.i, "(?<=@)|(?=@)", perl=TRUE)) %>%
      # Replace the @ with digit for reg exp
      str_replace("@","\\\\d") %>%
      #paste together with +
      paste0(collapse = "+")
    
    # Use the re exp instruction
    str.vec <- gsub(reg.inst,
                    # Replace with separated parts plus the @ symbol in place
                    paste0(unlist(strsplit(pv.in.i, "(?<=@)|(?=@)", perl=TRUE)),collapse = ""),
                    str.vec)
    
  }
  
  return(str.vec[1])
  
}
#Ex. get.common.name(get.pv.names(names(df.qqq), "pv@math"))


b.se.pv <- function(res.l, pv.inputs, statistic){
  # Goal: From a list of size n pvs, get Sampling Var, Imputation Var, SE and Estimate
  # ------ INPUTS ------.
  # res.l : (list of dfs) Each dataframe must have a 1st column named "by.var" and all after a sequence of [b. se.] results
  # pv.inputs : (string vector) All strings containing an @ as an input
  
  # Names from columns
  c.names <- names(res.l[[1]])
  #Start on the second colname
  res.l.b.se <- lapply(2:length(c.names),function(n.i){
    
    # β -----------------------------------------------------------------------.
    # If b. then create a dabase and take the average
    if (startsWith(c.names[n.i],"b.")) {
      # Grab first element and n.i element of each df
      df.b <- lapply(res.l, function(df.i) df.i[c(1,n.i)]) %>% 
        reduce(full_join, by = "by.var") #%>% 
      #Note for later: this should be done only for frequencies!
      # When there are missing values depending on the pv # replace NA with 0 
        #replace(is.na(.),0)
      # Get the common name an all to use as variable name
      name.b <- get.common.name(names(df.b)[-1],pv.inputs)
      
      # If general argument "gen" the common name will be the first name
      if ("gen" %in% statistic) name.b <- c.names[n.i]
      
      # Mean of all rows for estimate of b  
      df.b[name.b] <- rowMeans(df.b[-1])
      
      res <- df.b[c(1,length(df.b))]
      # If se. then create a dabase and take the average
      
      # SE ----------------------------------------------------------------------.
    } else if (startsWith(c.names[n.i],"se.")){
      
      # SAMPLING VARIANCE------------------------------------------------------.
      # Grab first element and n.i element of each df for SE
      df.se <- lapply(res.l, function(df.i) df.i[c(1,n.i)]) %>% 
        reduce(full_join, by = "by.var") #%>% 
        # When there are missing values depending on the pv # replace NA with 0 
        # replace(is.na(.),0)
      # Get the common name an all to use as variable name
      name.se <- get.common.name(names(df.se)[-1],pv.inputs)
      
      # If general argument "gen" the common name will be the first name
      if ("gen" %in% statistic) name.se <- c.names[n.i]
      
      sampling.variance <- rowMeans(df.se[-1], na.rm = T)
      
      # IMPUTATION VARIANCE----------------------------------------------------.
      # Grab first element and n.i - 1 element of each df for B
      df.b <- lapply(res.l, function(df.i) df.i[c(1,n.i-1)]) %>% 
        reduce(full_join, by = "by.var") #%>% 
        # When there are missing values depending on the pv # replace NA with 0 
        # replace(is.na(.),0)
      # Number of PVs
      n.pvs <- length(df.b)-1
      # Mean of estimates
      mean_b <- rowMeans(df.b[-1], na.rm = T)
      imputation.variance <- (1+1/n.pvs)*rowSums((df.b[-1] - mean_b)^2, na.rm = T)/(n.pvs-1)
      
      #Calculate SE and name the variable according to most common strings
      df.se[name.se] <- sqrt(imputation.variance + sampling.variance)
      
      res <- df.se[c(1,length(df.se))]
    }
    return(res)
  })
  return(reduce(res.l.b.se, full_join, by = "by.var"))
}

#' Weighted quantile
#' 
#' @description Computes weighted quantiles of a numeric vector.
#' @param x (numeric vector) Variable of interest for which to compute the quantile
#' @param w (numeric vector) Vector with the weights
#' @param q (numeric vector) A number between 0 and less than 1 indicating the desired quantile
#'
#' @return Scalar containing the quantile
#' @export
#'
#' @examples
#' weighted.quant(x = mtcars$mpg, w = mtcars$wt,  q = seq(.1,.9,.1))
#' 

weighted.quant <- function(x, w =  rep(1, length(x)), q = 0.5) {
  # Goal: Get weighted quantiles
  # ------ INPUTS ------.
  # x: (numeric vector) variable from where to get quantiles
  # w : (numeric vector) vector of weights
  # q : (numeric vector) [0,1] for the quantile to get
  
  # Apply function over vector of quants
  res.v <- sapply(q, function(q_i){
    #Only non NA data
    w <- w[!is.na(x)]
    x <- x[!is.na(x)]
    #Order ascending both
    ord <- order(x)
    w <- w[ord]
    x <- x[ord]
    #Running sum
    w.sum <- cumsum(w)
    
    #Get x_i s.t. max Σw_i <= W*q
    res <- head(x[!(w.sum <= sum(w)*q_i)], n=1)
    
    return(res)
  })
  
  names(res.v) <- q
  
  return(res.v)
  
}

#' Multivariate correlation and covariance
#'
#' @description Computes multivariate correlation and covariance for the variables of interest.
#' @param data (data frame) Data to analyse
#' @param x (string vector) Variables of interest for which to compute the correlation/covariance
#' @param w (string) Name of the numeric variable representing the weights
#' @param corr (bool) if TRUE: Computes correlation; if FALSE: Computes covariance
#' @param na.rm (bool) if TRUE: Excludes missing values before computing the correlation/covariance.
#'
#' @return Data frame containing each pairwise bivariate correlation/covariance
#' @export
#'
#' @examples
#' data(df_talis18)
#' 
#' weighted.corr.cov.n(df_talis18,c("tt3g03","tt3g06a2","tt3g06b1"),"tchwgt")
#' 
weighted.corr.cov.n <- function(data, x, w =  rep(1, length(data[x[1]])), corr = TRUE, na.rm = TRUE) {
  # Goal: Get weighted correlation or covariance of n variables
  # ------ INPUTS ------.
  # data : (dataframe) df to analyze
  # x: (vector string) variables from where to get correlation/covariance
  # w : (string) weight name
  # corr : (bool) T → get correlation, F → get covariance
  
  # Get dataframe of correlation
  vec.res <- apply(as.data.frame(combn(x,2)), 2, function(x.i) {
    if (corr) {
      res <- weighted.corr(x = data[x.i[1]],
                           y = data[x.i[2]], 
                           w = data[w],
                           na.rm = na.rm)
    }else{
      res <- weighted.cov(x = data[x.i[1]],
                          y = data[x.i[2]], 
                          w = data[w],
                          na.rm = na.rm)
    }
    
    return(res)
  })
  # Convert vector to dataframe if 1 by n
  vec.res <- data.frame(t(vec.res))
  
  # Get column names
  vec.names <- apply(as.data.frame(combn(x,2)), 2, function(x.i) {
    if (corr) {
      res <- paste0("corr.",x.i[1],"&",x.i[2])
    }else{
      res <- paste0("cov.",x.i[1],"&",x.i[2])
    }
    return(as.vector(res))
  })
  colnames(vec.res) <- vec.names
  
  return(vec.res)
}

################# Flags #################
# 3 Feb 2026
# Rodolfo Ilizaliturri
#############################################################
# Goal: Replace numeric variables with nans where there is not enough schools and or teachers

# General for all
flags <- function(data, svy) {
  # Goal: Replace numeric variables with nans where there is not enough schools and or teachers
  # ------ INPUTS ------.
  # data : (dataframe) must have last column have "n.sch" or two last columns "n.tch" and "n.sch"
  # svy : (str) survey to analyse
  
  if (svy %in% c("TALISSCH", "TALISEC_LEADER")) {
    #If principals questionnaire the just check for schools to add NaN
    res.df <- data %>% 
      mutate_if(is.numeric, list(~ifelse(
        # If is not NA and less than 10 schools
        !is.na(grep("n.obs", names(data), value=TRUE) %>% get()) &
          grep("n.obs", names(data), value=TRUE) %>% get() < 10, NaN, .)))
    
    
    #remove last columns
    res.df <- res.df[-c(length(data))]
    res.df[ncol(res.df)] <- labelled_spss(res.df[[names(res.df)[ncol(res.df)]]], c(flagged = NaN)) %>% as.data.frame()
  }else if (svy %in% c("TALISTCH","TALISEC_STAFF","TALISTKS")){
    #If teachers questionnaire check for schools and teachers to add NaN
    res.df <- data %>% 
      mutate_if(is.numeric, list(~ifelse(
        # If is not NA and less than 30 teachers and 10 schools
        !is.na(grep("n.sch", names(data), value=TRUE) %>% get()) & (
          grep("n.sch", names(data), value=TRUE) %>% get() < 10 |
            grep("n.obs", names(data), value=TRUE) %>% get() < 30    ), NaN, .)))
    
    
    #remove last 2 columns
    res.df <- res.df[-c(length(data)-1,length(data))]
    #res.df[ncol(res.df)] <- labelled_spss(res.df[[names(res.df)[ncol(res.df)]]], c(flagged = NaN))
  }else if (svy %in% c("PISA2015","PISA","SSES","SSES2023")){
    #If teachers questionnaire check for schools and teachers to add NaN
    res.df <- data %>% 
      mutate_if(is.numeric, list(~ifelse(
        # If is not NA and less than 30 teachers and 5 schools
        !is.na(grep("n.sch", names(data), value=TRUE) %>% get()) & (
          grep("n.sch", names(data), value=TRUE) %>% get() < 5 |
            grep("n.obs", names(data), value=TRUE) %>% get() < 30    ), NaN, .)))
    
    
    #remove last 2 columns
    res.df <- res.df[-c(length(data)-1,length(data))]
    #res.df[ncol(res.df)] <- labelled_spss(res.df[[names(res.df)[ncol(res.df)]]], c(flagged = NaN))
  }else {
    message("Survey not specified, using n < 30 for flag")
    #If all other surveys the just check for schools to add NaN
    res.df <- data %>% 
      mutate_if(is.numeric, list(~ifelse(
        # If is not NA and less than 30 observations
        !is.na(grep("n.obs", names(data), value=TRUE) %>% get()) &
          grep("n.obs", names(data), value=TRUE) %>% get() < 30, NaN, .)))
    
    
    #remove last columns
    res.df <- res.df[-c(length(data))]
    res.df[ncol(res.df)] <- labelled_spss(res.df[[names(res.df)[ncol(res.df)]]], c(flagged = NaN)) %>% as.data.frame()
  }
  return(res.df)
}

# For rrepest.TALIS
flags.nan <- function(data, svy) {
  # Goal: Replace numeric variables with nans where there is not enough schools and or teachers
  # ------ INPUTS ------.
  # data : (dataframe) must have last column have "n.sch" or two last columns "n.tch" and "n.sch"
  # svy : (str) survey to analyse
  
  if (svy == "TALISSCH") {
    #If principals questionnaire the just check for schools to add NaN
    res.df <- data %>% 
      mutate_if(is.numeric, list(~ifelse(
        # If is not NA and less than 10 schools
        !is.na(grep("n.sch", names(data), value=TRUE) %>% get()) &
          grep("n.sch", names(data), value=TRUE) %>% get() < 10, NaN, .)))

    
    #remove last columns
    res.df <- res.df[-c(length(data))]
    res.df[ncol(res.df)] <- labelled_spss(res.df[[names(res.df)[ncol(res.df)]]], c(flagged = NaN)) %>% as.data.frame()
  }else{
    #If teachers questionnaire check for schools and teachers to add NaN
    res.df <- data %>% 
      mutate_if(is.numeric, list(~ifelse(
        # If is not NA and less than 30 teachers and 10 schools
        !is.na(grep("n.sch", names(data), value=TRUE) %>% get()) & (
          grep("n.sch", names(data), value=TRUE) %>% get() < 10 |
            grep("n.tch", names(data), value=TRUE) %>% get() < 30    ), NaN, .)))
      
    
    #remove last 2 columns
    res.df <- res.df[-c(length(data)-1,length(data))]
    #res.df[ncol(res.df)] <- labelled_spss(res.df[[names(res.df)[ncol(res.df)]]], c(flagged = NaN))
  }
  return(res.df)
}

################# Test for over in Frequencies #################
# 1 May 2023
# Rodolfo Ilizaliturri
#############################################################


over.test.freq <- function(data, over) {
  # Goal: Create a dataframe only with by.var and statistic variable but with difference by over variables
  # ------ INPUTS ------.
  # data : (dataframe) df to analize
  # over : (vector string) columns over which to do analysis
  
  # Create by.var so not to interfere with other functions
  by.var <- colnames(data)[1 : which(colnames(data) == over[1])-1]
  
  # Group by all over and by variables and drop nas for over
  res.data <- data %>% 
    drop_na(all_of(over)) 
  
  
  
  # Pivot frequencie column to column names (freq.category1, freq.category2, ...)
  res.data <- res.data %>% 
    pivot_wider(values_from = "freq", names_from = over[length(over)])
  
  # Keep over as over.old
  over.old <- over
  # Create over variable without last entry (frequency variable name)
  over <- over[-length(over)]
  
  # Exctract colnames of numeric variables
  num.var <- colnames(res.data)[(which(colnames(res.data) == over[length(over)])+1) : ncol(res.data)]
  # Create a list of as many data frames as numeric variables with by.var and over in them
  l.dfs <- lapply(num.var, function(n.i){
    df.i <- res.data %>% select(all_of(c(by.var,over,n.i)))
    return(df.i)
  })
  
  
  # Assumption that all the combinations of "over" categories exist in database
  # for multiple variables when doing difference. IOW: That a difference can be made
  # for the last variable in over
  
  #Apply method to show test on list of l.dfs
  l.dfs <- lapply(l.dfs, function(res.d){
    
    if (length(over) > 1) {
      # If there is more than 1 over variable get how many values per group for the last
      # variable are there (Sprint 2.1.2)
      res.d <- res.d %>% 
        unite("over-l", all_of(over[-length(over)]),sep="|")
      
      n.diff <- res.d$"over-l" %>% unique() %>% length() - 1
      
      res.d <- res.d %>% 
        unite("over", 
              all_of(c(over[length(over)],"over-l")),
              sep = "|")
    }else{
      #If less or equal to 1 variable just rename
      n.diff <- 0
      res.d <- res.d %>% 
        rename("over"=over)
    }
    
    #Merge all by.var variables, do wider with over, and sort all of over alphabetically
    res.d <- res.d %>% 
      unite("by.group", all_of(by.var), sep = "|") %>%
      pivot_wider(names_from = "over", values_from = colnames(.)[length(.)]) %>%
      select("by.group",sort(colnames(.)[-1]))
    
    #Get difference for the first and last categories of over
    diffs <- res.d[2:(2+n.diff)]-res.d[(length(res.d)-n.diff):length(res.d)]
    
    #Get colnames for first and last categories of over
    c.names.a <- colnames(res.d)[2:(2+n.diff)]
    c.names.b <- colnames(res.d)[(length(res.d)-n.diff):length(res.d)]
    
    #Create names for difference columns on the format:
    #(1st - last)|rest
    c.names.diffs <- c()
    for (i in seq_along(c.names.a)) {
      if(n.diff > 0) {
        c.n <- paste0("(",sub("\\|.*$","",c.names.a[i]),"-",sub("\\|.*$","",c.names.b[i]),")",
                      "|",sub(".*?\\|","",c.names.a[i]))
        c.names.diffs <- c(c.names.diffs,c.n)
      }else{
        c.n <- paste0("(",sub("\\|.*$","",c.names.a[i]),"-",sub("\\|.*$","",c.names.b[i]),")")
        c.names.diffs <- c(c.names.diffs,c.n)
      }
    } 
    colnames(diffs) <- c.names.diffs
    res.d <- cbind(res.d,diffs)
    
    #Get long data frame with only by.var  column already with differences
    res.d <- res.d %>% 
      pivot_longer(cols = colnames(res.d)[-1] ,names_to = "over") %>% 
      unite("by.group",all_of(c("by.group","over")),sep = "|")
    
    #Send the value corresponding to last variable in over to the end of the name
    res.d[["by.group"]] <- apply(res.d,1, function(x) {
      c(strsplit(x,"\\|")[[1]],
        strsplit(x,"\\|")[[1]][(length(by.var)+1)])[-(length(by.var)+1)] %>% 
        paste(collapse = "|")
    })
    
    return(res.d)
    
  })  %>% 
    # Merge all dataframes by by.group
    reduce(full_join, by = "by.group")
  
  # Rename numeric columns
  colnames(l.dfs)[-1] <- num.var
  
  #Pivot longer each category from the frequency variable
  l.dfs <- l.dfs %>% 
    pivot_longer(cols = num.var,names_to = over.old[length(over.old)],values_to = "freq") %>% 
    unite(col = "by.group", c("by.group",over.old[length(over.old)]), sep = "|")
  
  return(l.dfs)
}

#' Dagger or double dagger according to coverage level
#' 
#' @description
#' Transform coverage columns in Rrepest to dagger and double dagger according
#' to the coverage (cvge) column. Default levels are 75 (dagger) and 50 (double dagger). 
#' If coverage is above both levels, no symbol is produced (empty). Used with the coverage o
#' option in Rrepest set to TRUE to produce columns with coverage percentages.
#' 
#' @param res_df (data frame) Rrepest output with columns for coverage (cvge)
#' @param one_dagger (numeric) Level at which the coverage is transformed into a dagger. 75 by default.
#' @param two_dagger (numeric) Level at which the coverage is transformed into a double dagger. 50 by default
#'
#' @return Dataframe with daggers or double daggers in coverage column
#' @export
#'
#' @examples
#' coverage_daggers(talis18_tt3g23o_freq, one_dagger = 95, two_dagger = 90)

coverage_daggers <- function(res_df, one_dagger = 75, two_dagger = 50){
  # GOAL: Assign dagger and double dagger to coverage column for determined thresholds
  # --- ARGUMENTS ---
  # res_df (data frame) dataframe qith cvge. column for coverage (as in Rrepest)
  # one_dagger (numeric) threshold for one dagger
  # two_dagger (numeric) threshold for two daggers
  
  # Get coverage variables from result
  cge_vars <- names(res_df)[startsWith(names(res_df),"cvge.")]
  
  for (cge_i in cge_vars){
    # Initiate NA column
    cge_dagger <- rep(NA, nrow(res_df))
    # Assign two daggers
    cge_dagger[res_df[[cge_i]] < two_dagger] <- "\U2021"
    # Assign one dagger
    cge_dagger[res_df[[cge_i]] >= two_dagger & res_df[[cge_i]] < one_dagger] <- "\U2020"
    # Replace coverage columns with dagger
    res_df[[cge_i]] <- cge_dagger
  }
  
  return(res_df)
}

################# Multiple statistics on Continuous variables #################
# 22 Mar 2023
# Rodolfo Ilizaliturri
#############################################################
# Goal: Obtain several statistics using Fay's BRR model


# ------------------.
# --------- WORK FUNCTIONS
# ------------------.

pv.brr.stat.by <- function(df, statistic = "mean",x, w, by.var, over, test, n.df = list(), svy,...) {
  # Goal: Statistics from final weight
  # ------ INPUTS ------.
  # df : (dataframe) df to analize
  # x: (string) variable from where to get means
  # w : (string) weighting variable
  # by.var : (string vector) variables to break analysis by
  # over : (vector string) columns over which to do analysis
  
  arg <- list(...)
  options(dplyr.summarise.inform = FALSE)
  
  res.col <- df %>% 
    group_by(across(all_of(by.var))) 
  
  # STATISTICS --------------------------------------------------------------.
  if(statistic == "quant") {res.col <- summarise(res.col, w.quant = weighted.quant(get(x),get(w),arg$q.iqr))}
  else if(statistic == "iqr") {res.col <- summarise(res.col, w.iqr = weighted.iqr(get(x),get(w),arg$q.iqr))}
  else if(statistic %in% c("mean","means")) {res.col <- summarise(res.col, w.mean = weighted.mean(get(x),get(w),na.rm = T))}
  else if(statistic %in% c("meanpct","meanspct")) {res.col <- summarise(res.col, w.mean = weighted.mean(get(x),get(w),na.rm = T)*100)}
  else if(statistic == "var") {res.col <- summarise(res.col, w.var = weighted.var(get(x),get(w),na.rm = T))}
  else if(statistic %in% c("std","sd")) {res.col <- summarise(res.col, w.std = weighted.std(get(x),get(w),na.rm = T))}

  # FLAG --------------------------------------------------------------------.
  #If flag attach n.df a nd remove based on condition
  if (length(n.df) > 0) {
    # make into one column over and by var
    res.col <- res.col %>% 
      unite("by.group", all_of(c(by.var, over)), sep = "|")
    
    # Merge both dfs together n and freqs
    res.col <- left_join(res.col, n.df, by = "by.group")
    
    #Assign NaN when not enough coverage for estimation
    res.col <- flags(data = res.col, svy = svy) %>% 
      separate("by.group",into = all_of(by.var), sep = "\\|" )
  }
  

  
  
  #------ If test for over add tests to res.col
  if (test) {
    res.col <- over.test(res.col, over) %>% 
      column_to_rownames(var= "by.group")
  }else{
    res.col <- res.col %>% 
      remove_rownames %>%
      unite("by.group", all_of(by.var), sep = "|") %>% 
      column_to_rownames(var= "by.group")
  }
  
  
  
  # -- Track weight number
  colnames(res.col) <- paste("w.",
                             sub("RWGT","",w),
                             sep = "")
  
  return(res.col)
}

pv.brr.stat.by.PAR <- function(df, statistic = "mean",x, w, by.var, over, test, n.df = list(), svy,...) {
  # Goal: Statistics from final weight in for each since in packages the info can only go 1 enviroment deep
  # ------ INPUTS ------.
  # df : (dataframe) df to analize
  # x: (string) variable from where to get means
  # w : (string) weighting variable
  # by.var : (string vector) variables to break analysis by
  # over : (vector string) columns over which to do analysis
  # ...
  #(functions) weighted.iqr, weighted.quant, weighted.var, weighted.std
  
  arg <- list(...)
  options(dplyr.summarise.inform = FALSE)
  
  res.col <- df %>% 
    group_by(across(all_of(by.var))) 
  
  # STATISTICS --------------------------------------------------------------.
  if(statistic == "quant") {res.col <- summarise(res.col, w.quant = arg$weighted.quant(get(x),get(w),arg$q.iqr))}
  else if(statistic == "iqr") {res.col <- summarise(res.col, w.iqr = arg$weighted.iqr(get(x),get(w),arg$q.iqr))}
  else if(statistic %in% c("mean","means")) {res.col <- summarise(res.col, w.mean = weighted.mean(get(x),get(w),na.rm = T))}
  else if(statistic %in% c("meanpct","meanspct")) {res.col <- summarise(res.col, w.mean = weighted.mean(get(x),get(w),na.rm = T)*100)}
  else if(statistic == "var") {res.col <- summarise(res.col, w.var = arg$weighted.var(get(x),get(w),na.rm = T))}
  else if(statistic %in% c("std","sd")) {res.col <- summarise(res.col, w.std = arg$weighted.std(get(x),get(w),na.rm = T))}
  
  # FLAG --------------------------------------------------------------------.
  #If flag attach n.df a nd remove based on condition
  if (length(n.df) > 0) {
    # make into one column over and by var
    res.col <- res.col %>% 
      unite("by.group", all_of(c(by.var, over)), sep = "|")
    
    # Merge both dfs together n and freqs
    res.col <- left_join(res.col, n.df, by = "by.group")
    
    #Assign NaN when not enough coverage for estimation
    res.col <- arg$flags(data = res.col, svy = svy) %>% 
      separate("by.group",into = all_of(by.var), sep = "\\|" )
  }
  
  
  #------ If test for over add tests to res.col
  if (test) {
    res.col <- arg$over.test(res.col, over) %>% 
      column_to_rownames(var= "by.group")
  }else{
    res.col <- res.col %>% 
      remove_rownames %>%
      unite("by.group", all_of(by.var), sep = "|") %>% 
      column_to_rownames(var= "by.group")
  }
  
  
  
  # -- Track weight number
  colnames(res.col) <- paste("w.",
                             sub("RWGT","",w),
                             sep = "")
  
  return(res.col)
}

pv.loop.stat.on.weights <- function (data, statistic = "mean", rep_weights,
                                     x, by.var, over, test, n.df = list(),
                                     svy, pv = F, ...) {
  # Goal: Means from final weight
  # ------ INPUTS ------.
  # data : (dataframe) df to analize
  # rep_weights : (string vector) names of replicated weight vars
  # over : (vector string) columns over which to do analysis
  # x: (string) variable from where to get means
  # pv : (Bool) TRUE → We are in plausible values and must parallelize

  
  if (!pv) {
    # NON PARALLEL ------------------------------------------------------------.
    #Filter on only non NA values move here due to performance
    df <- data %>% filter(!is.na(get(x)))
    
    rep.w.stat <- lapply(rep_weights, function(i) pv.brr.stat.by(df = df,
                                                              statistic = statistic,
                                                              x = x,
                                                              w = i,
                                                              by.var = by.var,
                                                              over = over,
                                                              test = test,
                                                              n.df = n.df,
                                                              svy = svy,
                                                              ...))
  } else {
    # PARALLEL ---------------------------------------------------------------.
    rep.w.stat <- foreach(w.i = rep_weights,
                          .packages = c("dplyr","tidyr","purrr","tibble", "data.table"),
                          .export = c("pv.brr.stat.by.PAR",
                                      "weighted.std","weighted.var",
                                      "weighted.quant","weighted.iqr",
                                      "flags","over.test","...")) %dopar% {
                                        
                                        #Filter on only non NA values move here due to performance
                                        data.par <- na.omit(data.par, cols = x)
                                        #NOTE: na.omit uses a data.table since it is 7 times faster
                                        pv.brr.stat.by.PAR(df = data.par,
                                                    statistic = statistic,
                                                    x = x,
                                                    w = w.i,
                                                    by.var = by.var,
                                                    over = over,
                                                    test = test,
                                                    n.df = n.df,
                                                    svy = svy,
                                                    weighted.std = weighted.std,
                                                    weighted.var = weighted.var,
                                                    weighted.quant = weighted.quant,
                                                    weighted.iqr = weighted.iqr,
                                                    flags = flags,
                                                    over.test = over.test,
                                                    ... = ...)
                                      }
    
    
  }
  
  
  
  return(reduce(rep.w.stat, cbind))
  }
  

pv.brr.gstats <- function(data, svy, x, by.var, statistic = "mean",
                          over, test, flag, fast, pv = F, ...) {
  # Note: Same as rrepest.gstats without the number of schools and teachers
  # Goal: Dataframe with statistic and SE by Fay's BRR model
  # ------ INPUTS ------.
  # data : (dataframe) df to analyze
  # statistic : (string) accepts "mean","var","std"
  # q.iqr : (vector numeric) of length 1 if quantile, of length 2 if interquantile range 
  # svy : List of possible projects to analyse PIAAC, PISA, TALISSCH and TALISTCH
  # by.var : (string) column in which we'll break down results !IOP!: several variables
  # x : (string) variable from where to get statistics
  # pv : (Bool) TRUE → We are in plausible values and must parallelize
  # user_na : (Bool) TRUE → show nature of user defined missing values in by.var
  
  eclass <- list()
  arg <- list(...)
  
  
  # Get weight names
  weight.names <- replicated_w_names(svy, ...)
  # If fast then less replicated weights
  if (fast) {
    weight.names <- weight.names[1:6]
  }
  
  # Get n data if flag needed
  if (flag) {
    n.df <-  n.obs.x(df = data, by.var = c(by.var, over), x = x, svy = svy)
  } else {
    n.df <- list()
  }
  
  
  # Loop over weights
  brr.res <- pv.loop.stat.on.weights(data = data,
                                  statistic = statistic,
                                  rep_weights = weight.names,
                                  x = x,
                                  by.var = by.var, 
                                  over = over,
                                  test = test,
                                  n.df = n.df,
                                  svy = svy,
                                  pv = pv,
                                  ...)
  
  #----------statistic
  m <- brr.res[1] %>% 
    set_names(nm = paste0("b.",statistic,
                          if_else(statistic %in% c("quant","iqr"),
                                  str_replace_all(
                                    paste0(arg$q.iqr,collapse = "-"),
                                    "0.","0"),
                                  ""),
                          ".", tolower(x) )) %>% 
    rownames_to_column(var = "by.group")
  
  eclass <- append(eclass,list("m"=m))
  
  #----------se
  se <- apply(brr.res, 1, rrvar.pv, svy=svy, pv=pv, ... = ...) %>% 
    as.data.frame() %>% 
    set_names(nm = paste0("se.",statistic,
                          if_else(statistic %in% c("quant","iqr"),
                                  str_replace_all(
                                    paste0(arg$q.iqr,collapse = "-"),
                                    "0.","0"),
                                  ""),
                          ".", tolower(x) )) %>% 
    rownames_to_column(var = "by.group")
  
  eclass <- append(eclass,list("se"=se))
  
  
  return(reduce(eclass, full_join, by = "by.group"))
}


# --------- FINAL FUNCTION



pv.rrepest.statistics <- function(data, svy, statistic = "mean", x, by.var = NULL,
                               over = NULL, test = FALSE, flag = FALSE, user_na=FALSE, 
                               fast = FALSE, pv = FALSE, ...) {
  # Goal: Dataframe with statistic and SE by Fay's BRR model
  # Note: All formatting depends on higher function
  # ------ INPUTS ------.
  # data : (dataframe) df to analyze
  # svy : List of possible projects to analyse PIAAC, PISA, TALISSCH and TALISTCH
  # statistic : (string vector) accepts "mean","var","std", "quant", "iqr"
  # q.iqr : (vector numeric) of length 1 if quantile, of length 2 if interquantile range 
  # by.var : (vector string) column in which we'll break down results
  # over : (vector string) columns over which to do analysis
  # test : (bool) If TRUE will calculate the difference between over variables
  # x : (string) variable from where to get statistics
  # user_na : (Bool) TRUE → show nature of user defined missing values in by.var
  # flag : (Bool) TRUE → Show NaN when there is not enough observations (or schools)
  # pv : (Bool) TRUE → We are in plausible values and must parallelize
  # ...
  # isced : (number) isced level to analyze
  
  #Store resulting DFs
  results <- list()
  
  stats_rr <- statistic
  
  # Data formatting
  by.over <- c(by.var,over)
  
  # In multiple statistics vector, skipping iterations if "quant" or "iqr"
  skip_iter <- 0
  for (i in seq_along(stats_rr)) {
    if (skip_iter > 0) {
      skip_iter <- skip_iter-1
      next}
    
    if (stats_rr[i] == "quant") {
      skip_iter <- 1
      
      #add brr statistics to original results
      results <- append(results, list(
        pv.brr.gstats(data = data, svy = svy, x = x,by.var = by.over, 
                   statistic = "quant", test = test, over = over, flag = flag,
                   fast = fast, pv = pv, 
                   q.iqr = stats_rr[i+1] %>% as.numeric(), ...) )
      )
      
    }else if (stats_rr[i] == "iqr"){
      skip_iter <- 2
      
      #add brr statistics to original results
      results <- append(results, list(
        pv.brr.gstats(data = data, svy = svy, x = x, by.var = by.over, 
                   statistic = "iqr", test = test, over = over, flag = flag,
                   fast = fast, pv = pv, 
                   q.iqr = c(stats_rr[i+1] %>% as.numeric(),
                             stats_rr[i+2] %>% as.numeric()),...) )
      )
      
    }else{
      #add brr statistics to original results
      results <- append(results, list(
        pv.brr.gstats(data = data, svy = svy, x = x, by.var = by.over,
                   test = test, over = over, flag = flag,
                   fast = fast, pv = pv, 
                   statistic = stats_rr[i],...))
      )
    }
    
  }
  
  # Merge all results together
  df.results <- reduce(results, full_join, by = "by.group")
  
  if (length(over) > 0) {
    #Separate all of by.group by | and unite only by by.var, leaving columns from over
    df.results <- df.results %>% 
      separate(by.group, into = by.over, sep = "\\|") %>% 
      unite("by.group", all_of(by.var), sep = "|") 
    
    #Pivot on the columns from over
    df.results <- df.results %>% 
      pivot_wider(names_from = all_of(over),
                  values_from = colnames(df.results)[! colnames(df.results) %in% c("by.group",over)],
                  names_sep = "..")
  }
  
  #If there is an over reorder results to have se following b
  if (length(over) > 0) {
    
    #reorder df with intercalating columns
    
    # Get columns that start with "b."
    b_only_cols <- names(df.results)[startsWith(names(df.results),"b.")]
    # Start name from 3rd character and add b. & se. to each name to intercalate them
    bse_col_order <- as.vector(sapply( substring(b_only_cols,3), function(name_i){
      paste0(c("b.","se."),name_i)
    }))
    # Set in intercalated order
    df.results <- df.results[,c("by.group",bse_col_order)]
  }
  
  
  return(df.results)
}

#' Group Averages
#'
#' @param data (dataframe) Data frame containing replicated weights.
#' @param res (dataframe) Data frame of results, including estimates and standard errors to be averaged.
#' @param group (grp function) Function used to compute grouped averages; takes arguments group.name, column, and cases, and appends results at the end of the data frame.
#' @param by (string vector) Variable(s) used to break down the results.
#' @param over (vector string) Variable(s) over which the analysis is performed.
#' @param est (est function) Estimation function; takes arguments what = estimate, tgt = target, and rgr = regressor.
#' @param svy (string) Name of the survey/project to analyse (e.g. TALISSCH, TALISTCH).
#' @param user_na (bool) If TRUE, shows the nature of user-defined missing values for the grouping variable(s).
#' @param ... Additional arguments such as na_to_zero: (Bool) TRUE → treats NA values as zero for simple average calculations.
#'
#' @return Dataframe with averages or weighted averages (totals) appended in the last rows for the selected cases.
#'
average_groups <- function(res, data = NULL, group, by = NULL, over = NULL, est = NULL, svy = NULL, user_na = FALSE, ...) {
  # Goal: Average across sub samples of results assuming same size
  # ------ INPUTS ------.
  # data : (dataframe) df from which to get replicated weights
  # res : (dataframe) df of results with b. and se. to average
  # group: (grp function) that takes arguments group.name, column, cases to create averages at the end of dataframe
  # by: (string vector) column in which we'll break down results
  # est : (est function) that takes arguments what = estimate, tgt = target, rgr = regressor
      # what : (string vector) accepts "mean", "mean pct","var","std", "quant", "iqr", "freq", "lm", "corr", "cov", "gen"
      # tgt : (string vector) variable from where to get frequencies or R script if selected "gen"
      # rgr : (string vector) independant variable for regression (1+)
  # over : (vector string) columns over which to do analysis
  # user_na : (Bool) TRUE: show nature of user defined missing values for by.var
  # ...
  # na_to_zero : (Bool) TRUE → will take NA as zero for the simple average calculation
  
  # Example:
  #grp("ARE-AUS","CNTRY",c("ARE","AUS"))
  #grp("OECD Average","CNTRY",c("HUN","MEX"))
  
  #Get na_to_zero from ...
  arguments <- list(...)
  
  if (("." %in% by) | (is.null(by))){
    warning("Average without a 'by' variable is redundant")
    return(res)
  }
  
  #Iterate over the names of each group
  res.avgs <- lapply(names(group), function(g_i){
    # Replacing all NAs with zeros to consider for simple average
    if (!is.null(is.there(arguments$na_to_zero))) if(arguments$na_to_zero) res <- res %>% replace(is.na(.),0)
    
    
    

    # SIMPLE AVERAGE ----------------------------------------------------------
    # If not weighted average
    if(!group[[g_i]][["full_weight"]]){
      # Create a df with only the "cases" in group and renamed by the group "name"
      res.df <- res %>% 
        mutate(!!group[[g_i]][["column"]] := 
                 ifelse(get(group[[g_i]][["column"]]) %in%
                          group[[g_i]][["cases"]], 
                        g_i, # if YES
                        get(group[[g_i]][["column"]]))) %>% # if NO
        # filter from selected variable the group needed
        filter(!!as.name(group[[g_i]][["column"]]) == g_i) 
      
      # Average for b -----------------------------------------------------------
      res.b <- res.df %>% 
        #get by and b.
        select(by, starts_with("b.")) %>% 
        group_by(across(all_of(by))) %>% 
        summarise(across(starts_with("b."),\(x) mean(x, na.rm = TRUE)))
      
      # Average for cvge (if exists) -----------------------------------------------------------
      res.cvge <- res.df %>% 
        #get by and b.
        select(by, starts_with("cvge.")) %>% 
        group_by(across(all_of(by))) %>% 
        summarise(across(starts_with("cvge."),\(x) mean(x, na.rm = TRUE)))
      
      # calculate √(Σ(se))/n for se. --------------------------------------------
      res.se <- res.df %>%
        select(by,starts_with("se.")) %>% 
        group_by(across(all_of(by))) %>% 
        summarise(across(starts_with("se."),\(x) (x^2 %>% sum(na.rm = TRUE) %>% sqrt())/
                           sum(!is.na(x))))
    } else {
    # -------------------------------------------------------------------------
    # WEIGHTED AVERAGE --------------------------------------------------------
    # -------------------------------------------------------------------------
      
      # Same as initial Repest formating
      # Argument renaming to be adapted in v2 -----------------------------------
      what <- est$what
      tgt <- est$tgt
      rgr <- est$rgr
      
      # Plausible Values DIGITS--------------------------------------------------
      # If tgt or rgr has an @ then get digits of PVs
      if (any(grepl("@", c(tgt,rgr,by,over)))) {
        # Get all PV digits
        pv.digits <- digits.pvs(svy = svy, ...)
      } else {
        # if no @ the just iterate over 1 value
        pv.digits <- "@"
      }
      
      # Get optional arguments
      extra.args <- list(...)
      
      # Formatting depending on continuous or categorical tgt (assuming categorical target for all)
      df_weights <- format_data_repest(df = data,
                               svy = svy,
                               x = NULL, 
                               # all arguments with an @ get converted into pvs
                               by.over = get.pv.arguments(pv.digits, tolower(c(by,over,tgt))),
                               user_na = user_na, 
                               isced = is.there(extra.args$isced),
                               ...)
      

      # Group by BY and OVER but not FREQ the sum of the final weight (1)
      res_weights <- df_weights %>% 
        group_by(across(tolower(by))) %>% 
        summarise(weights = sum(get(replicated_w_names(svy = svy, ...)[1])))
      
      # Rename variables to match upper and lower case
      names(res_weights) <- c(by,"weights")
      
      #Join results and weights and select only relevant cases
      # Create a df with only the "cases" in group and renamed by the group "name"
      res.df <- left_join(res, res_weights, by = by) %>% 
        mutate(!!group[[g_i]][["column"]] := 
                 ifelse(get(group[[g_i]][["column"]]) %in%
                          group[[g_i]][["cases"]], 
                        g_i, # if YES
                        get(group[[g_i]][["column"]]))) %>% # if NO
        # filter from selected variable the group needed
        filter(!!as.name(group[[g_i]][["column"]]) == g_i) 
      
      # Average for WEIGHTED b -----------------------------------------------------------
      res.b <- res.df %>% 
        #get by and b.
        select(by, starts_with("b."), "weights") %>% 
        group_by(across(all_of(by))) %>% 
        summarise(across(starts_with("b."),\(x) weighted.mean(x = x, w = .data$weights, na.rm = TRUE)))
      
      # Average for WEIGHTED cvge -----------------------------------------------------------
      res.cvge <- res.df %>% 
        #get by and b.
        select(by, starts_with("cvge."), "weights") %>% 
        group_by(across(all_of(by))) %>% 
        summarise(across(starts_with("cvge."),\(x) weighted.mean(x = x, w = .data$weights, na.rm = TRUE)))
      
      # calculate √(Σ(N^2 * se^2))/ ΣN. --------------------------------------------
      res.se <- res.df %>%
        select(by,starts_with("se."), "weights") %>% 
        group_by(across(all_of(by))) %>% 
        summarise(across(starts_with("se."),\(x) ((.data$weights^2 * x^2) %>% sum(na.rm = TRUE) %>% sqrt())/
                           sum(.data$weights * x/x, na.rm = TRUE)))
      
    }
    
    # Join b. and se. together
    res.df <- inner_join(x = res.b, y = res.se, by = by) 
    
    # If there are cvge columns in the names of res merge them to the res.df
    if(any(grepl("^cvge\\.",names(res)))){
      res.df <- left_join(x = res.df, y = res.cvge, by = by) 
    }
      #reorder according to original database
    res.df <- res.df %>% select(names(res))

    
    
    return(res.df)
  })
  
  #Join all averages to the original results
  res.avgs <- append(list(res), res.avgs) %>% reduce(rbind)
  
  return(res.avgs)
}

ft_significance <- function(ft_i, z = qnorm(1 - .05/2), statistic){
  # ft_i (flextable) : flextable with b and se
  # z (numeric) : z-score for significance level
  # statistic (string) : statistic required from Rrepest (if "lm" do sig. diffs.)
  
  col_n <- ft_i$col_keys
  
  # Iterate over the indexes of col_n
  for (n_i in seq_along(col_n)) {
    if(any(c("gen","lm") %in% statistic)) { #If "lm" everything gets sig. diff. except R^2 and Intercept
      if (!grepl("Rsqr", col_n[n_i]) & # No R^2
          !grepl("Intercept", col_n[n_i]) & # No Intercept
          grepl(".*SE$", col_n[n_i])) {
        # Evaluate Statistically Significant rows
        sig_rows <- abs(ft_i$body$dataset[n_i-1]/ft_i$body$dataset[n_i])
        # Change flextable according to Statistical Significance
        ft_i <- ft_i %>% 
          color( i = sig_rows > z,
                 color = "blue",
                 j = n_i-1) %>% 
          bold ( i = sig_rows > z,
                 bold = TRUE,
                 j = n_i-1)
      }
    } else { #If no "lm" only tests get sig. diff.
      # If have structure _(_-_)_ and start with se (NOTE: the middle hyphen common hyphen)
      if (grepl(".*\\(.*\\-.*\\).*", col_n[n_i]) &
          grepl(".*SE$", col_n[n_i])) {
        # Evaluate Statistically Significant rows
        sig_rows <- abs(ft_i$body$dataset[n_i-1]/ft_i$body$dataset[n_i])
        # Change flextable according to Statistical Significance
        ft_i <- ft_i %>% 
          color( i = sig_rows > z,
                 color = "blue",
                 j = n_i-1) %>% 
          bold ( i = sig_rows > z,
                 bold = TRUE,
                 j = n_i-1)
      } 
    }
  }
  return(ft_i)
}

################# Functionalize Quantile Table #################
# 8 March 2025
# Rodolfo Ilizaliturri
#############################################################
# Goal: Get table of the mean at each quantile for n quantiles


pv.do.quantiletable <- function(df, quant_n, y, by.var, w, ...){
  # Goal: Get table of the mean at each quantile for n quantiles of equal length according to the weight
  # ------ INPUTS ------.
  # df : (dataframe) df to analyze previosly formated
  # by.var : (string) column in which we'll break down results !IOP!: several variables
  # quant_n : (numeric) number of quantiles to do. If not integer, will get difference of last quantile minus first
  # y : (string) Target variable (just 1). If two variables inputed in the format "var1|var2", 
  #     it will order and get same length quantiles according to the first and get the mean of the second
  # w : (string) weighting variable (just 1)
  
  # Search pattern "var1__var2" in y
  if(grepl("__",y)){
    # If found separete element into "y for order" and "y for mean""
    y_vars <- strsplit(y,"__")[[1]]
    y_4order <- y_vars[1]
    y_4mean <- y_vars[2]
    # Get the name of y for columns
    y_name <- paste0(y_4order,"__",y_4mean)
  } else {
    # If not found they are both the same
    y_4order <- y
    y_4mean <- y
    # Get the name of y for columns
    y_name <- y_4order
  }
  
  diff_quant <- FALSE
  # If quant_n has decimals floor the number
  if(quant_n %% 1 != 0){
    quant_n <- floor(quant_n)
    diff_quant <- TRUE
  }
  
  if (is.data.table(df)) {
    # Faster drop of NAs if it is a data.table
    df.qtable <- na.omit(df, cols = c(y_4order))
  } else {
    # Selecting data and preparation
    df.qtable <- df[c(unique(c(y_4order,y_4mean)),by.var,w)]
    
    df.qtable <- df.qtable %>% drop_na(all_of(c(y_4order)))
  }
  

  # Get quantiles -----------------------------------------------------------

  # Get average by each quantile
  res.df <- df.qtable %>% 
    group_by(across(all_of(c(by.var)))) %>% 
    do({
      # Set seed
      set.seed(5094) #For random order of values
      
      # Create dataframe for organization and noise
      df_quantiletable <- .data[c(unique(c(y_4order,y_4mean)),w)]
      df_quantiletable[["noise"]] <- runif(nrow(df_quantiletable))
      
      # Weight accounting for NA in y
      df_quantiletable[[w]] <- ifelse(is.na(df_quantiletable[[y_4order]]),NA,1) * df_quantiletable[[w]]
      
      # Value of quantiles
      quantile_breaks <- seq(0,sum(df_quantiletable[[w]],na.rm = TRUE),length.out = quant_n + 1)
      # Force last element to be avobe breaks to make them all fall into a bin
      quantile_breaks[length(quantile_breaks)] <- quantile_breaks[length(quantile_breaks)] + 1
      
      # Add noise, arrange and do cumsum
      df_quantiletable <- df_quantiletable %>% 
        mutate("y_order" := !!rlang::sym(y_4order) + 0.0001*.$noise) %>% 
        arrange(.$y_order) %>% 
        mutate("w_cumsum" := cumsum(!!rlang::sym(w)))
      
      # Assign value of y to each bin of quants
      df_quantiletable[["quantile_table_cuts"]] <- cut(
        x = df_quantiletable[["w_cumsum"]], 
        breaks = quantile_breaks, 
        include.lowest = TRUE, right = TRUE,
        labels = paste0("q",1:quant_n,".",y_name))
      
      # Weighted mean grouped by cut
      res_quintiletable <- df_quantiletable %>% 
        group_by("quantile_table_cuts" = .$quantile_table_cuts) %>% #groupby done like that to avoid  pv.do.quantiletable: no visible binding for global variable 'quantile_table_cuts'
        summarise(q_table = weighted.mean(x = .data[[y_4mean]], w = .data[[w]], na.rm = TRUE)) %>% 
        pivot_wider(values_from = "q_table", names_from = "quantile_table_cuts")
      
      # If quant_n has decimals subtract last from first
      if(diff_quant){
        res_quintiletable <- res_quintiletable %>% 
          mutate(!!rlang::sym(paste0("q",quant_n,"_1.",y_name)) := .[[quant_n]] - .[[1]])
      }
      
      res_quintiletable
    }) %>% 
    ungroup()
  
  return(res.df)
}
pv.do.quantiletable.PAR <- function(df, quant_n, y, by.var, w, ...){
  # Goal: Get table of the mean at each quantile for n quantiles
  # ------ INPUTS ------.
  # df : (dataframe) df to analyze previosly formated
  # by.var : (string) column in which we'll break down results !IOP!: several variables
  # quant_n : (numeric) number of quantiles to do.
  # y : (string) target variable (just 1).
  # w : (string) weighting variable (just 1)
  # ...
  #(functions) weighted.var
  
  # get ... arguments
  arg <- list(...)
  
  # Search pattern "var1|var2" in y
  if(grepl("__",y)){
    # If found separete element into "y for order" and "y for mean""
    y_vars <- strsplit(y,"__")[[1]]
    y_4order <- y_vars[1]
    y_4mean <- y_vars[2]
    # Get the name of y for columns
    y_name <- paste0(y_4order,"__",y_4mean)
  } else {
    # If not found they are both the same
    y_4order <- y
    y_4mean <- y
    # Get the name of y for columns
    y_name <- y_4order
  }
  
  diff_quant <- FALSE
  # If quant_n has decimals floor the number
  if(quant_n %% 1 != 0){
    quant_n <- floor(quant_n)
    diff_quant <- TRUE
  }
  
  if (is.data.table(df)) {
    # Faster drop of NAs if it is a data.table
    df.qtable <- na.omit(df, cols = c(y_4order,y_4mean))
  } else {
    # Selecting data and preparation
    df.qtable <- df[c(y_4order,y_4mean,by.var,w)]
    
    df.qtable <- df.qtable %>% drop_na(all_of(c(y_4order,y_4mean)))
  }
  
  
  # Get quantiles -----------------------------------------------------------
  
  # Get average by each quantile
  res.df <- df.qtable %>% 
    group_by(across(all_of(c(by.var)))) %>% 
    do({
      # Set seed
      set.seed(5094) #For random order of values
      
      # Create dataframe for organization and noise
      df_quantiletable <- .data[c(y_4order,y_4mean,w)]
      df_quantiletable[["noise"]] <- runif(nrow(df_quantiletable))
      
      # Weight accounting for NA in y
      df_quantiletable[[w]] <- ifelse(is.na(df_quantiletable[[y_4order]]),NA,1) * df_quantiletable[[w]]
      
      # Value of quantiles
      quantile_breaks <- seq(0,sum(df_quantiletable[[w]],na.rm = TRUE),length.out = quant_n + 1)
      # Force last element to be avobe breaks to make them all fall into a bin
      quantile_breaks[length(quantile_breaks)] <- quantile_breaks[length(quantile_breaks)] + 1
      
      # Add noise, arrange and do cumsum
      df_quantiletable <- df_quantiletable %>% 
        mutate("y_order" := !!rlang::sym(y_4order) + 0.0001*.$noise) %>% 
        arrange(.$y_order) %>% 
        mutate("w_cumsum" := cumsum(!!rlang::sym(w)))
      
      # Assign value of y to each bin of quants
      df_quantiletable[["quantile_table_cuts"]] <- cut(
        x = df_quantiletable[["w_cumsum"]], 
        breaks = quantile_breaks, 
        include.lowest = TRUE, right = TRUE,
        labels = paste0("q",1:quant_n,".",y_name))
      
      # Weighted mean grouped by cut
      res_quintiletable <- df_quantiletable %>% 
        group_by("quantile_table_cuts" = .$quantile_table_cuts) %>% 
        summarise(q_table = weighted.mean(x = .data[[y_4mean]], w = .data[[w]], na.rm = TRUE)) %>% 
        pivot_wider(values_from = "q_table", names_from = "quantile_table_cuts")
      
      # If quant_n has decimals subtract last from first
      if(diff_quant){
        res_quintiletable <- res_quintiletable %>% 
          mutate(!!rlang::sym(paste0("q",quant_n,"_1.",y_name)) := .[[quant_n]] - .[[1]])
      }
      
      res_quintiletable
    }) %>% 
    ungroup()
  
  return(res.df)
}
pv.loop.quantiletable.on.weights <- function (data, quant_n, y, by.var, over, test = F, flag = F,
                                       svy, rep_weights, fast = F, pv = F, ...) {
  # Goal: Regression model results for each weight
  # ------ INPUTS ------.
  # data : (dataframe) df to analize
  # rep_weights : (string vector) names of replicated weight vars
  # reg.model : (expression) linear model to implement
  # by.var : (string vector) variables to break analysis by
  # fast : (bool) TRUE → Only do 6 replicated weights
  # pv : (Bool) TRUE → We are in plausible values and must parallelize
  
  # If fast then less replicated weights
  if (fast) {
    rep_weights <- rep_weights[1:6]
  }
  
  if (!pv) {
    # NON PARALLEL ------------------------------------------------------------.
    
    res.l <- lapply(rep_weights, function(w.i){
      #Do lm, add weight to colnames, append new data of results to list
      res.df <- pv.do.quantiletable(data, quant_n, y, all_of(c(by.var,over)), w.i) %>% 
        unite("by.var", all_of(c(by.var,over)), sep = "|")
      
      # Change col names to include weight name
      # colnames(res.df)[-1] <- paste0(colnames(res.df)[-1], paste0("|",w.i))
      
      return(res.df)
    })
    # PARALLEL ---------------------------------------------------------------.
  }else{
    res.l <- foreach(w.i = rep_weights,
                     .packages = c("dplyr","tidyr","data.table","tibble"),
                     .export = c("pv.do.quantiletable.PAR","n.obs.x",
                                 "weighted.quant","...")) %dopar% {
                                   
                                   res.df <- pv.do.quantiletable.PAR(data.par,
                                                                     quant_n,
                                                              y,
                                                              all_of(c(by.var,over)),
                                                              w.i,
                                                              weighted.quant = weighted.quant,
                                                              ...=...) %>% 
                                     unite("by.var", all_of(c(by.var,over)), sep = "|")
                                   return(res.df)
                                 }
    
  }
  
  
  # ---------------- FLAGS
  if (flag) {
    # Get n for flags and separate column
    # NOTE!!!!!!: Here x = y since the target variable in regressions is the y
    n.df <- n.obs.x(df = data, by.var = c(by.var, over), x = y, svy = svy)
    
    res.l <- lapply(res.l, function(res.i){
      # Merge both dfs together n and freqs
      res.df <- left_join(res.i, n.df, by = c("by.var"="by.group"))
      
      #Assign NaN when not enough coverage for estimation (flags.nan is deprecated)
      res.df <- flags(data = res.df, svy = svy)
      
      return(res.df)
    })
  }
  # ---------------- .
  
  # ---------------- TEST
  if (test) {
    # Create extra rows with test differences
    res.l <- lapply(res.l, function(res.i){
      # Split by.var into c(by.var, over, x) by |
      res.df <- res.i %>% 
        separate(col = "by.var", into = all_of(c(by.var, over)), sep = "\\|") %>% 
        over.test(over = c(over)) %>% 
        rename("by.var"="by.group")
      
      return(res.df)
    })
  } 
  # ---------------- .
  # ---------------- FORMAT DF to have over on the columns
  if (length(over)>0) {
    res.l <- lapply(res.l, function(res.i){
      res.df <- res.i %>% 
        # Split by.var leaving only the variables in "by.var" united 
        separate(col = "by.var", into = all_of(c(by.var, over)), sep = "\\|") %>% 
        unite(col = "by.var", all_of(by.var), sep = "|") %>% 
        # Pivot to columns over
        pivot_wider(names_from = all_of(c(over)),
                    values_from = colnames(select_if(.,is.numeric)),
                    names_sep = "..")
      
      return(res.df)
    })
  }
  # ---------------- .
  # ---------------- ASSIGN qtableq#.{name.of.y} to each dataframe
  res.l <- lapply(res.l, function(res.i){
    res.i <- res.i
    colnames(res.i)[-1] <- paste0("qtable", colnames(res.i)[-1])
    return(res.i)
  })
  
  return(res.l)
}


# --------- FINAL LOGISTIC MODEL FUNCTION

pv.rrepest.quantiletable <- function(data,  svy, quant_n, y, by.var = NULL, over = NULL, test = F, 
                              user_na=F, flag = F, fast = F, pv = F, ...) {
  # Goal: Dataframe with β and SE by Fay's BRR model for logistic regression
  # ------ INPUTS ------.
  # data : (dataframe) df to analyze
  # svy : List of possible projects to analyse PIAAC, PISA, TALISSCH and TALISTCH
  # isced : (number) isced level to analyze
  # by.var : (string) column in which we'll break down results
  # quant_n : (numeric) number of quantiles to do.
  # y : (string) Target variable (just 1).
  # user_na : (Bool) TRUE → show nature of user defined missing values in by.var
  # over : (vector string) columns over which to do analysis
  # test : (bool) If TRUE will calculate the difference between over variables
  # flag : (Bool) TRUE → Show NaN when there is not enough cases (or schools)
  # fast : (bool) TRUE → Only do 6 replicated weights
  # pv : (Bool) TRUE → We are in plausible values and must parallelize
  # ...
  # isced : (number) isced level to analyze
  
  # Get optional arguments
  extra.args <- list(...)
  
  # If there is an OVER variable remove NAs from OVER vars
  # if (length(over) > 0) {
  #   for (i in over) {
  #     df <- df %>% drop_na(i)
  #   }
  # }
  
  # Get weight names
  weight.names <- replicated_w_names(svy, ...)
  
  # Loop over each variable
  brr.res <- pv.loop.quantiletable.on.weights(data = data,
                                              quant_n = quant_n,
                                              y = y,
                                              svy = svy,
                                              by.var = by.var,
                                              over = over,
                                              test = test,
                                              flag = flag,
                                              rep_weights = weight.names,
                                              fast = fast,
                                              pv = pv,
                                              ... = ...)
  
  # Reorder data to have b and se intertwined
  res <- pv.get.se.reorder(brr.res, svy = svy, pv = pv, ... = ...)
  
  return(res)
}

#' Subset of PISA 2018 student questionnaire database
#'
#' This dataset is a subset of the PISA 2018 database produced by the OECD for the countries of France, Italy, and Mexico.
#'
#' @docType data
#' @keywords datasets
#' @name df_pisa18
#' @usage data(df_pisa18)
#' @format A data frame with 1269 rows/observations and 1120 columns/variables.
NULL

#' Subset of TALIS 2018 teacher database
#'
#' This dataset is a subset of the TALIS 2018 database produced by the OECD for the countries of France, Italy, and Mexico.
#'
#' @docType data
#' @keywords datasets
#' @name df_talis18
#' @usage data(df_talis18)
#' @format A data frame with 548 rows/observations and 496 columns/variables.
NULL

#' Number of valid (i.e. non-missing) observations for column/variable x
#'
#' @description Computes the number of valid (i.e. non-missing) observations for the column/variable of interest.
#' @param df (data frame) Data to analyse with lowercase column names.
#' @param by (string vector) Variable(s) used to tabulate the variable of interest.
#' @param x (string) Variable of interest for which to compute the number of valid (i.e. non-missing) observations.
#' @param svy (string) Survey setting, must be one of the following: ALL, IALS, ICCS, ICILS, IELS, PBTS, PIAAC, PIRLS, PISA, PISAOOS, PISA2015, SSES, SSES2023, SVY, TALISSCH, TALISTCH, TALISEC_LEADER, TALISEC_STAFF, TIMSS.
#' @return Data frame containing the number of valid (i.e. non-missing) observations for the variable of interest.
#' @export
#'
#' @examples
#' n_obs_x(df = df_talis18, by = "cntry",x = "tt3g01", svy = "TALISTCH")
n_obs_x <- function(df, by, x, svy = NULL) {
  # Goal: Number of schools and teachers calculation is performed on
  # ------ INPUTS ------.
  # df : (dataframe) df to analyze
  # by : (string) column in which we'll break down results
  # x : (string) variable from where to get means
  # svy : List of possible projects to analyse PIAAC, PISA, TALISSCH and TALISTCH
  
  #All lower caps column names
  colnames(df) <- colnames(df) %>% tolower()
  by <- by %>% tolower()
  x <- x %>% tolower()
  
  tot.n <- df %>%
    group_by(across(all_of(by))) %>%
    {if (svy == "TALISSCH") summarise(., n.obs = sum(!is.na(get(x)))) 
      else if (svy %in% c("TALISTCH","TALISTKS")) {mutate(.,school.n = ifelse(is.na(get(x)) == T, NA, .data$idschool)) %>% 
          summarise(., n.obs = sum(!is.na(get(x))),
                    n.sch = n_distinct(school.n, na.rm = T))}
      else if (svy == "TALISEC_STAFF") {mutate(.,school.n = ifelse(is.na(get(x)) == T, NA, .data$idcentre)) %>% 
          summarise(., n.obs = sum(!is.na(get(x))),
                    n.sch = n_distinct(school.n, na.rm = T))}
      else if (svy == "PISA2015") {mutate(.,school.n = ifelse(is.na(get(x)) == T, NA, .data$cntschid)) %>% 
          summarise(., n.obs = sum(!is.na(get(x))),
                    n.sch = n_distinct(school.n, na.rm = T))}
      else if (svy == "PISA") {mutate(.,school.n = ifelse(is.na(get(x)) == T, NA, .data$schoolid)) %>% 
          summarise(., n.obs = sum(!is.na(get(x))),
                    n.sch = n_distinct(school.n, na.rm = T))}
      else if (svy %in% c("SSES","SSES2023")) {mutate(.,school.n = ifelse(is.na(get(x)) == T, NA, .data$schid)) %>% 
          summarise(., n.obs = sum(!is.na(get(x))),
                    n.sch = n_distinct(school.n, na.rm = T))}
      else summarise(., n.obs = sum(!is.na(get(x))))
    } %>%
    unite("by.group", all_of(by), sep = "|")
  
  #add x colanames
  colnames(tot.n)[-1] <- paste(colnames(tot.n)[-1], tolower(x), sep = ".")
  
  return(tot.n)
}

#' Estimation using replicate weights
#' 
#' @description 
#' Estimates statistics using replicate weights (Balanced Repeated Replication (BRR) weights,
#' Jackknife replicate weights,...), thus accounting for complex survey designs in the estimation
#' of sampling variances. It is designed specifically to be used with the data sets produced by
#' the Organization for Economic Cooperation and Development (OECD), some of which include the
#' Programme for the International Assessment of Adult Competencies (PIAAC), Programme for
#' International Student Assessment (PISA) and Teaching and Learning International Survey (TALIS)
#' data sets, but works for any educational large-scale assessment and survey that uses replicated
#' weights. It also allows for analyses with multiply imputed variables (plausible values);
#' where plausible values are used, average estimator across plausible values is reported and
#' the imputation error is added to the variance estimator.
#' 
#' @param data (data frame) Data to analyse.
#' @param svy (string) Declares the survey settings. It must be equal to one of the following: ALL, IALS, ICCS, ICILS, IELS, PBTS, PIAAC, PIRLS, PISA, PISAOOS, PISA2015, SSES, SSES2023, SVY, TALISSCH, TALISTCH, TALISEC_LEADER, TALISEC_STAFF, TIMSS.
#' @param est (est function) Specifies the estimates of interest. It has three arguments: statistics type, target variable, and an (optional) regressor list in case of a linear regression.
#' @param by (string vector) Produces separate estimates by levels of the variable(s) specified.
#' @param over (string vector) Requests estimates to be obtained separately for each level of the categorical variable(s) specified.
#' @param test (bool) If TRUE: computes the difference between estimates obtained for the lowest and highest values of the 'over' variable(s) (see 'over' option above). Useful for testing differences between dependent samples (e.g. female-male).
#' @param invert_tests (bool) Inverts test columns from Rrepest when test = TRUE, based on "b." and "se." in the column names, and by multiplying "b." by -1.
#' @param user_na (bool) If TRUE: shows the nature of user-defined missing values.
#' @param show_na (bool) If TRUE: includes missing values (i.e. NAs) when estimating frequencies for the variable of interest.
#' @param flag (bool) If TRUE: replaces estimation results that are based on fewer observations than required for reporting with NaN.
#' When used with the PIAAC survey settings, checks if each estimation result is based on at least 30 observations.
#' When used with the PISA, PISAOOS, PISA2015 survey settings, checks if each estimation result is based on at least 30 observations and 5 schools.  
#' When used with the TALISSCH survey settings, checks if each estimation result is based on at least 10 schools.  
#' When used with the TALISTCH survey settings, checks if each estimation result is based on at least 30 observations and 10 schools.
#' @param fast (bool) If TRUE: computes estimates using only 6 replicated weights.
#' @param average (grp function) Computes an arithmetic (or weighted) average. It has three arguments: name of the average, column/variable used for computing the average, and rows/observations included in the average.
#' @param total (grp function) Computes an average weighted by the estimated size of the target population covered.
#' @param tabl (bool) If TRUE: creates customisable and transferable tables using the flextable R package.
#' @param coverage (bool/numeric) TRUE: shows a column next to se. Numeric: shows NaN if below the specified coverage threshold.
#' @param se_zero2na (bool) TRUE: masks SE of 0 as NA for mean, meanpct, or freq equal to 1, 100, or 0.
#' @param save_arg (bool) TRUE: returns a named list with the estimation data frame and all arguments used in Rrepest.
#' @param cores (numeric) NULL: recruits max-1 cores when doing PVs; otherwise, uses the specified number of cores for PVs.
#' @param ... Other optional parameters include:
#' isced = Filters the data used for analysis by ISCED level (e.g. isced = 2), 
#' n.pvs = Customizes the number of plausible values used in the estimation (e.g. n.pvs = 5),
#' cm.weights = Customizes the weights used in the estimation (e.g. cm.weights = c("finw",paste0("repw",1:22))),
#' var.factor = Customizes the variance factor used in the estimation (e.g. var.factor = 1/(0.5^2)),
#' z.score = qnorm(1-0.05/2)
#' 
#' @import dplyr
#' @import tidyr
#' @import tibble
#' @import labelled
#' @import parallel
#' @import foreach
#' @import doParallel
#' @import flextable
#' @import stringr
#' @import officer
#' @importFrom stats as.formula coefficients lm.wfit glm.fit na.omit qnorm resid weighted.mean setNames aggregate quasibinomial runif
#' @importFrom utils combn head
#' @rawNamespace import(data.table, except = c(first,last,between,transpose))
#' @rawNamespace import(purrr, except = c(when,compose,accumulate))
#' @rawNamespace import(magrittr, except = c(set_names,extract))
#' @rawNamespace import(rlang, except = c("flatten_lgl","splice","flatten_chr","flatten_raw","flatten","flatten_dbl","invoke","flatten_int",":=","%@%"))
#' 
#'
#' @return Data frame containing estimation "b." and standard error "se.".
#' @export
#'
#' @examples
#' data(df_pisa18)
#' 
#' Rrepest(data = df_pisa18,
#' svy = "PISA2015",
#' est = est("mean","AGE"),
#' by = c("CNT"))
#' 
#' 
#' 
Rrepest <- function(data, svy, est, by = NULL, over = NULL,
                    test = FALSE, user_na= FALSE, show_na = FALSE, flag = FALSE, fast = FALSE,
                    tabl = FALSE, average = NULL, total = NULL, coverage = FALSE, invert_tests = FALSE,
                    save_arg = FALSE, cores = NULL, se_zero2na = FALSE,...) { #######group to total
  # Goal: Dataframe with statistics for TALIS
  # ------ INPUTS ------.
  ######### WHO ######### 
  # data : (dataframe) df to analyze
  # svy : List of possible projects to analyse TALISSCH and TALISTCH
  ######### WHAT ######### 
  # est : (est function) that takes arguments what = estimate, tgt = target, rgr = regressor
      # what : (string vector) accepts "mean", "mean pct","var","std", "quant", "iqr", "freq", "lm", "corr", "cov", "gen"
      # tgt : (string vector) variable from where to get frequencies or R script if selected "gen"
      # rgr : (string vector) independant variable for regression (1+)
  ######### WHERE ######### 
  # by.var : (string vector) column in which we'll break down results
  # over : (vector string) columns over which to do analysis
  ######### HOW ######### 
  # test : (bool) If TRUE will calculate the difference between over variables
  # user_na : (Bool) TRUE: show nature of user defined missing values for by.var
  # show_na : (Bool) TRUE: include na in frequencies of x 
  # flag : (Bool) TRUE: Show NaN when there is not enough cases (or schools)
  # fast : (Bool) TRUE: Only do 6 replicated weights\
  # table : (Bool) TRUE: Creates a flextable with all examples
  # coverage : (Numeric / Bool) Numeric [0,100]: If less than this percentage it will be marked as not having enough coverage
  #                             TRUE: Shows an extra cvg column with the coverage for that estimate
  # ...
  # isced : (number) isced level to analyze
  # na_to_zero : (Bool) TRUE: will take NA as zero for the simple average calculation
  
  # Turn data into tibble
  data <- data %>% as_tibble()
  
  # Save all arguments to list
  if (save_arg){
    argg <- c(as.list(environment()), list(...))
    argg[["data"]] <- NULL
  }
  
  # NAs in over variables are uninteresting as they will create a column of NAs, so must be removed
  for(over_i in over){
    if(!grepl("@",over_i)){ # Skip of @ in over
      data <- data %>% 
        filter(!is.na(get(over_i)))
    }
  }
  
  # Patch for TALISSCH: At least 50 that start with crwgt (2018 was srwgt)
  if(svy == "TALISSCH" & sum(grepl("^crwgt",names(data))) > 50){
    names(data)[names(data) == "schwgtc"] <- "schwgt"
    names(data)[names(data) %>% startsWith("crwgt")] <- paste0("s",substr(names(data)[names(data) %>% startsWith("crwgt")],2,10))
  }
  # Patch for TALISEC_LEADER: At least 50 that start with lrwgt (2018 at. crwgt)
  if(svy == "TALISEC_LEADER" & sum(grepl("^lrwgt",names(data))) > 50){
    names(data)[names(data) == "leaderwgt"] <- "cntrwgt"
    names(data)[names(data) %>% startsWith("lrwgt")] <- paste0("c",substr(names(data)[names(data) %>% startsWith("lrwgt")],2,10))
  }
  if(svy == "TALISSCH" & sum(grepl("^CRWGT",names(data))) > 50){
    names(data)[names(data) == "SCHWGTC"] <- "SCHWGT"
    names(data)[names(data) %>% startsWith("CRWGT")] <- paste0("S",substr(names(data)[names(data) %>% startsWith("CRWGT")],2,10))
  }
  # Patch for TALISEC_LEADER: At least 50 that start with lrwgt (2018 at. crwgt)
  if(svy == "TALISEC_LEADER" & sum(grepl("^LRWGT",names(data))) > 50){
    names(data)[names(data) == "LEADERWGT"] <- "CNTRWGT"
    names(data)[names(data) %>% startsWith("LRWGT")] <- paste0("C",substr(names(data)[names(data) %>% startsWith("LRWGT")],2,10))
  }
  
  # Get na_to_zero from ...
  arguments <- list(...)

  # Create a list of dfs for each concerning group
  if (!is.null(total)) {
    res.l <- lapply(names(total), function(g) {
      # Assign name of group to all the column value
      data %>% 
        # Filter only the desired cases
        filter(get(total[[g]][["column"]]) %in% total[[g]][["cases"]]) %>% 
        # Rename column values to that of the desired group
        mutate(!!total[[g]][['column']] := g) %>% 
        return()
    })
  } else 
  {res.l <- NULL}
  
  # All dataframes together (original + groups)
  res.l <- append(list(data),res.l)
  
  
  # Apply Rrepest_core to each of those dataframes --------------------------
  res.l <- lapply(res.l, function(df.i){
    # Original Rrepest
    rrepest_base(data = df.i, 
                 svy = svy,
                 est = est,
                 by = by ,
                 over = over,
                 test = test,
                 user_na= user_na,
                 show_na= show_na, 
                 flag = flag,
                 fast = fast,
                 tabl = tabl,
                 cores = cores,
                 ...)
  }) %>% reduce(rbind)
  
  
  
  # If there is no by.var have an empty string in columns
  if (is.null(by)) by <-  "."
  
  # establish by.var as the corresponding the variable names in final results table
  res.l <- res.l %>% 
    separate(col = "by.var",into = by ,sep = "\\|") 
  


  # Force NaN ---------------------------------------------------------------
  # Force NaNs from se. generated by the flag onto b.
  if(flag){
    nans_from_se2b(res.l)
  }
  

  # Coverage ----------------------------------------------------------------
  
  if (coverage != FALSE){
    # Get estimation specs
    what <- est$what
    tgt <- est$tgt
    rgr <- est$rgr
    
    # Patch for quantile table
    if ("quantiletable" %in% what) {
      # Create variable for flags and coverage "var1__var2"
      if(grepl("__",tgt)){
        # If found separete element into "y for order" and "y for mean""
        y_vars <- strsplit(tgt,"__")[[1]]
        y_4order <- y_vars[1]
        y_4mean <- y_vars[2]
        # Get the name of y for columns
        y_name <- paste0(y_4order,"__",y_4mean)
        tgt.i <- y_name
        # y_name will be multiplication of both to get the proper ammount of NAs
        data[[y_name]] <- data[[y_4order]] * data[[y_4mean]]
      }
    }
    # Plausible Values DIGITS--------------------------------------------------
    # If tgt or rgr has an @ then get digits of PVs
    if (any(grepl("@", c(tgt,rgr,by,over)))) {
      # Get all PV digits
      pv.digits <- digits.pvs(svy = svy, ...)
    } else {
      # if no @ the just iterate over 1 value
      pv.digits <- "@"
    }
    
    # Formatting depending on continuous or categorical tgt (assuming categorical target for all)
    df_covg <- format_data_repest(df = data,
                                  svy = svy,
                                  x = NULL, 
                                  # all arguments with an @ get converted into pvs
                                  by.over = get.pv.arguments(pv.digits, tolower(c(by,over, tgt))),
                                  user_na = user_na, 
                                  isced = is.there(arguments$isced),
                                  ...)
  }
  
  
  # If there is a number, show NaNs for where there is coverage problem
  if (is.numeric(coverage)){
    res.l <- coverage_column(data = df_covg, res = res.l, by = by, over = over,
                    est = est, svy = svy, pct = coverage)
  } else if (coverage){
    res.l <- coverage_column(data = df_covg, res = res.l, by = by, over = over,
                    est = est, svy = svy)
  }

  # AVERAGES of results -----------------------------------------------------
  if (!is.null(average)) {
    res.l <- average_groups(data = data, res = res.l, group = average, by = by,
                            over = over, est = est, svy = svy, user_na = user_na, ...)
  }

  # INVERT ALL TESTS --------------------------------------------------------
  if (invert_tests){
    # Test must exist to apply invert_tests
    if(test){
      # Iterate through all accurances of "b.est.(cat1-cat2)" name structure
      for(t_i in grep("^b\\..*\\(.*\\-.*\\)",names(res.l))){
        res.l <- inv_test(data = res.l, name_index = t_i)
      }
    }
  }

  # 0 SE for mean and freq --------------------------------------------------
  # SE of 0 get marked as an NA if meanpct/freq is 0 or 100 but zero is still used for averages
  if(se_zero2na){
    if("freq" %in% est$what){
      res.l <- se_to_na(res.l, mean_stat = FALSE)
    } else if (any(c("mean","means","meanpct","meanspct") %in% est$what)){
      res.l <- se_to_na(res.l, mean_stat = TRUE)
    }
  }
  
  #-------------- PRETTY TABLES --------------.
  if (tabl) {
    # Invert column names
    col_n <- names(res.l)
    col_n <- sapply(USE.NAMES = F, col_n, function(n_i) {
      # Replace first two instances of . with ..
      n_i <- n_i %>% 
        # First dot
        str_replace(pattern = "(?<!\\.)\\.(?!\\.)",
                    replacement =  "..") %>% 
        # Second dot
        str_replace(pattern = "(?<!\\.)\\.(?!\\.)",
                    replacement =  "..")
      
      # Split by .. 
      n_i <- n_i %>% 
        str_split(pattern = "\\..") %>% 
        unlist()
      
      # Replace b with B and se with SE
      if (n_i[1] == "b") {n_i[1] <- "B"}
      else if (n_i[1] == "se") {n_i[1] <- "SE"}
      
      # Invert order and paste back by sep = ..
      n_i <- n_i %>% 
        rev() %>% 
        paste0(collapse = "..")
      
      return(n_i)
    })
    names(res.l) <- col_n
    
    # Create flextable
    res.l <- res.l %>% 
      flextable()
    
    # Significant differences in tests -----------------------------------------
    # Check for z.score in arguments ...
    if (!is.null(list(...)$z.score)) {z <- list(...)$z.score}
    else {z <- qnorm(1 - .05/2)}
    
    res.l <- ft_significance(ft_i = res.l, z = z, statistic = est$what)

    # Combine columns of by vars ----------------------------------------------
    res.l <- res.l %>% 
      theme_vanilla() %>% 
      # Merge all adjacent except for the last one in by.var
      merge_v(j = as.formula(paste0(" ~ ",
                                    paste0(by[1:max(1,length(by)-1)], collapse = "+") ))) %>% 
      colformat_double(na_str = "..", nan_str = "a",
                       digits = 1) %>% 
      hline_bottom(part = "body")
    

    # Separate column names into multiple lines of header--------------------------------------------------
    res.l <- res.l %>% 
      separate_header(split = "\\..") %>%  
      align(align = "center", part = "header") %>% 
      autofit() %>% 
      bold(bold = TRUE, part = "header") %>% 
      vline(border = fp_border(color="gray", width=.5),
            part = "header") %>% 
      bg(bg = "#D9E1F2", part = "header")
    
  }
  
  #---------------------------------------.
  # Remove transitory Data Bases
  gc()
  # If wanted to return arguments all will be put into a list with the resulting dataframe first
  if(save_arg){
    return(c(result=list(res.l),argg))
  } else {
    return(res.l)
  }
}

#' Coverage percentage  (1 - mean(is.na)) * 100
#'
#' @description Computes the coverage percentage for the column/variable of interest.
#' @param df (data frame) Data to analyse.
#' @param by (string vector) Variable(s) used to tabulate the variable of interest.
#' @param x (string) Variable of interest for which to compute the number of valid (i.e. non-missing) observations.
#' @param w (string) Vector of weights.
#' @param limit (numeric) Threshold below which the value will be TRUE.
#' @return Data frame containing the number of valid (i.e. non-missing) observations for the variable of interest.
#' @export
#'
#' @examples
#' data(df_pisa18)
#' data(df_talis18) 
#' 
#' coverage_pct(df = df_pisa18, by = "cnt",x = "st003d03t")
#' coverage_pct(df = df_talis18, by = "cntry",x = "tt3g01", w = "TCHWGT")
coverage_pct <- function(df, by, x, w = NULL, limit = NULL) {
  # Goal: Number of schools and teachers calculation is performed on
  # ------ INPUTS ------.
  # df : (dataframe) df to analyze
  # by : (string) column in which we'll break down results
  # x : (string) variable from where to get means
  # w : (string) vector of weights
  # limit : (numeric) threshold at which if lower, value will be Inf
  
  #All lower caps column names
  colnames(df) <- colnames(df) %>% tolower()
  by <- by %>% tolower()
  x <- x %>% tolower()
  
  if(!is.null(w)) {
    w <- tolower(w)
  }else{
    df <- df %>% mutate(weight..column = 1)
    w <- "weight..column"
  }
  
  #Convert desired variable to is.na
  df[x] <- is.na(df[x])
  
  tot.n <- df %>% 
    group_by(across(all_of(by))) %>%
    summarise(cvge = 100 - weighted.mean(get(x),as.vector(get(w))) * 100) %>%
    unite("by.group", all_of(by), sep = "|")
  
  #add x colanames
  colnames(tot.n)[-1] <- paste(colnames(tot.n)[-1], tolower(x), sep = ".")
  
  if (!is.null(limit)) {
    tot.n[ncol(tot.n)] <- tot.n[ncol(tot.n)] <= limit
  }
  
  return(tot.n)
}

#' Weighted standard deviation
#' 
#' @description Computes the weighted standard deviation of a numeric vector.
#' @param x (numeric vector) Variable of interest for which to compute the standard deviation.
#' @param w (numeric vector) Vector with the weights.
#' @param na.rm (bool) if TRUE: Excludes missing values before computing the standard deviation
#'
#' @return Scalar containing the standard deviation
#' @export
#'
#' @examples 
#' data(df_talis18)
#' 
#' weighted.std(df_talis18$tt3g06d1, df_talis18$tt3g06e1)

weighted.std <- function(x, w, na.rm = TRUE) {
  # Goal: Obtained weighted standard deviation of a vector
  # ------ INPUTS ------.
  # x : (vector) variable to analyze
  # w : (vector) vector of weights
  # na.rm : (bool) if TRUE (default) remove missing values.
  
  w.std <- (sum(w*(x-weighted.mean(x,w,na.rm = na.rm))^2, na.rm = na.rm)/sum(w,na.rm = na.rm))^(1/2)
  return(w.std)
}

################# Functionalize Logistic Regression an PVs #################
# 27 December 2024
# Rodolfo Ilizaliturri
#############################################################
# Goal: Obtain coefficients from logistic regression model in the form of Odds Ratio with greater speed


pv.do.odr <- function(df, x, y, by.var, w){
  # Goal: Get coefficients from logistic regression odds ratio
  # ------ INPUTS ------.
  # df : (dataframe) df to analyze previosly formated
  # by.var : (string) column in which we'll break down results !IOP!: several variables
  # x : (string vector) independant variable (1+)
  # y : (string) dependant variable (just 1). Must have valus betweeen 0 and 1.
  # w : (string) weighting variable (just 1)
  
  
  
  if (is.data.table(df)) {
    # Faster drop of NAs if it is a data.table
    df.reg <- na.omit(df, cols = c(x,y)) %>% 
      mutate(intercept = 1)
  } else {
    # Selecting data and preparation
    df.reg <- df[c(x,y,by.var,w)] %>% 
      mutate(intercept = 1)
    
    df.reg <- df.reg %>% drop_na(all_of(c(x,y)))
  }
  
  
  
  
  #model only fitting values. Out: Coeff ONLY
  res.df <- df.reg %>% 
    group_by(across(all_of(by.var))) %>% 
    do({
      # Converting each weight to a proportion
      weight_vec <- .[w] %>% as.matrix() %>% as.vector()
      # Weights parameter represents the proportion of the total for each row (i.e. when the wight column sums up to one). 
      weight_proportion <- weight_vec / sum(weight_vec,na.rm = TRUE)
      # Logistic regression from family binomial
      model <- glm.fit(x = .[c('intercept', x)] %>% as.matrix(),
                       y = .[y] %>% as.matrix(),
                       weights = weight_proportion, 
                       family = quasibinomial())
      # Get coefficients and covert to data.frame
      model %>% 
        coefficients() %>% 
        t() %>% 
        # exp() %>% #Odds Ratio changed for Delta Method
        as.data.frame()
    }
    ) %>%
    ungroup()
  
  return(res.df)
}
pv.do.odr.PAR <- function(df, x, y, by.var, w, weighted.var){
  # Goal: Get coefficients from logistic regression Odds Ratio
  # ------ INPUTS ------.
  # df : (dataframe) df to analyze previosly formated
  # by.var : (string) column in which we'll break down results !IOP!: several variables
  # x : (string vector) independant variable (1+)
  # y : (string) dependant variable (just 1). Must have valus betweeen 0 and 1.
  # w : (string) weighting variable (just 1)
  # ...
  #(functions) weighted.var
  
  # get ... arguments
  # arg <- list(...)
  
  
  if (is.data.table(df)) {
    # Faster drop of NAs if it is a data.table
    df.reg <- na.omit(df, cols = c(x,y)) %>% 
      mutate(intercept = 1)
  } else {
    # Selecting data and preparation
    df.reg <- df[c(x,y,by.var,w)] %>% 
      mutate(intercept = 1)
    
    df.reg <- df.reg %>% drop_na(all_of(c(x,y)))
  }
  
  
  
  
  #model only fitting values. Out: Coeff ONLY
  res.df <- df.reg %>% 
    group_by(across(all_of(by.var))) %>% 
    do({
      # Converting each weight to a proportion
      weight_vec <- .[w] %>% as.matrix() %>% as.vector()
      # Weights parameter represents the proportion of the total for each row (i.e. when the wight column sums up to one). 
      weight_proportion <- weight_vec / sum(weight_vec,na.rm = TRUE)
      # Logistic regression from family binomial
      model <- glm.fit(x = .[c('intercept', x)] %>% as.matrix(),
                       y = .[y] %>% as.matrix(),
                       weights = weight_proportion, 
                       family = quasibinomial())
      # Get coefficients and covert to data.frame
      model %>% 
        coefficients() %>% 
        t() %>% 
        # exp() %>% #Odds Ratio changed for Delta Method
        as.data.frame()
    }
    ) %>%
    ungroup()
  
  return(res.df)
}
pv.loop.odr.on.weights <- function (data, x, y, by.var, over, test = F, flag = F,
                                       svy, rep_weights, fast = F, pv = F, ...) {
  # Goal: Regression model results for each weight
  # ------ INPUTS ------.
  # data : (dataframe) df to analize
  # rep_weights : (string vector) names of replicated weight vars
  # reg.model : (expression) linear model to implement
  # by.var : (string vector) variables to break analysis by
  # fast : (bool) TRUE → Only do 6 replicated weights
  # pv : (Bool) TRUE → We are in plausible values and must parallelize
  
  # If fast then less replicated weights
  if (fast) {
    rep_weights <- rep_weights[1:6]
  }
  
  if (!pv) {
    # NON PARALLEL ------------------------------------------------------------.
    
    res.l <- lapply(rep_weights, function(w.i){
      #Do lm, add weight to colnames, append new data of results to list
      res.df <- pv.do.odr(data, x, y, all_of(c(by.var,over)), w.i) %>% 
        unite("by.var", all_of(c(by.var,over)), sep = "|")
      
      # Change col names to include weight name
      # colnames(res.df)[-1] <- paste0(colnames(res.df)[-1], paste0("|",w.i))
      
      return(res.df)
    })
    # PARALLEL ---------------------------------------------------------------.
  }else{
    res.l <- foreach(w.i = rep_weights,
                     .packages = c("dplyr","tidyr","data.table","tibble"),
                     .export = c("pv.do.odr.PAR","n.obs.x",
                                 "weighted.var")) %dopar% {
                                   
                                   res.df <- pv.do.odr.PAR(data.par,
                                                              x,
                                                              y,
                                                              all_of(c(by.var,over)),
                                                              w.i,
                                                              weighted.var = weighted.var) %>% 
                                     unite("by.var", all_of(c(by.var,over)), sep = "|")
                                   return(res.df)
                                 }
    
  }
  
  
  # ---------------- FLAGS
  if (flag) {
    # Get n for flags and separate column
    # NOTE!!!!!!: Here x = y since the target variable in regressions is the y
    n.df <- n.obs.x(df = data, by.var = c(by.var, over), x = y, svy = svy)
    
    res.l <- lapply(res.l, function(res.i){
      # Merge both dfs together n and freqs
      res.df <- left_join(res.i, n.df, by = c("by.var"="by.group"))
      
      #Assign NaN when not enough coverage for estimation (flags.nan is deprecated)
      res.df <- flags(data = res.df, svy = svy)
      
      return(res.df)
    })
  }
  # ---------------- .
  
  # ---------------- TEST
  if (test) {
    # Create extra rows with test differences
    res.l <- lapply(res.l, function(res.i){
      # Split by.var into c(by.var, over, x) by |
      res.df <- res.i %>% 
        separate(col = "by.var", into = all_of(c(by.var, over)), sep = "\\|") %>% 
        over.test(over = c(over)) %>% 
        rename("by.var"="by.group")
      
      return(res.df)
    })
  } 
  # ---------------- .
  # ---------------- FORMAT DF to have over on the columns
  if (length(over)>0) {
    res.l <- lapply(res.l, function(res.i){
      res.df <- res.i %>% 
        # Split by.var leaving only the variables in "by.var" united 
        separate(col = "by.var", into = all_of(c(by.var, over)), sep = "\\|") %>% 
        unite(col = "by.var", all_of(by.var), sep = "|") %>% 
        # Pivot to columns over
        pivot_wider(names_from = all_of(c(over)),
                    values_from = colnames(select_if(.,is.numeric)),
                    names_sep = "..")
      
      return(res.df)
    })
  }
  # ---------------- .
  # ---------------- ASSIGN odds_{name.of.y} to each dataframe
  res.l <- lapply(res.l, function(res.i){
    res.i <- res.i
    colnames(res.i)[-1] <- paste0("odr_", y,".", colnames(res.i)[-1])
    return(res.i)
  })
  
  return(res.l)
}


# --------- FINAL LOGISTIC MODEL ODDS RATIO FUNCTION

pv.rrepest.odr <- function(data,  svy, x, y, by.var = NULL, over = NULL, test = F, 
                              user_na=F, flag = F, fast = F, pv = F, ...) {
  # Goal: Dataframe with β and SE by Fay's BRR model for logistic regression Odds Ratios
  # ------ INPUTS ------.
  # data : (dataframe) df to analyze
  # svy : List of possible projects to analyse PIAAC, PISA, TALISSCH and TALISTCH
  # isced : (number) isced level to analyze
  # by.var : (string) column in which we'll break down results
  # x : (string vector) independant variable (1+)
  # y : (string) dependant variable (just 1). Must have values between 0 and 1.
  # user_na : (Bool) TRUE → show nature of user defined missing values in by.var
  # over : (vector string) columns over which to do analysis
  # test : (bool) If TRUE will calculate the difference between over variables
  # flag : (Bool) TRUE → Show NaN when there is not enough cases (or schools)
  # fast : (bool) TRUE → Only do 6 replicated weights
  # pv : (Bool) TRUE → We are in plausible values and must parallelize
  # ...
  # isced : (number) isced level to analyze
  
  # Get optional arguments
  extra.args <- list(...)
  
  # If there is an OVER variable remove NAs from OVER vars
  # if (length(over) > 0) {
  #   for (i in over) {
  #     df <- df %>% drop_na(i)
  #   }
  # }
  
  # Get weight names
  weight.names <- replicated_w_names(svy, ...)
  
  # Loop over each variable 
  brr.res <- pv.loop.odr.on.weights(data = data,
                                       x = x,
                                       y = y,
                                       svy = svy,
                                       by.var = by.var,
                                       over = over,
                                       test = test,
                                       flag = flag,
                                       rep_weights = weight.names,
                                       fast = fast,
                                       pv = pv)
  
  # Reorder data to have b and se intertwined
  res <- pv.get.se.reorder(brr.res, svy = svy, pv = pv, ... = ...)
  
  return(res)
}

#' Group list
#' 
#' @description Obtains a list with the arguments for the grp() function used within Rrepest() function's average() and group() options.
#' @param group.name (string) Name of the group to be displayed
#' @param column (string) Column/variable of interest to be grouped
#' @param cases  (string vector) Rows/values to be included in the group
#' @param full_weight  (bool) TRUE: average of the group will be weighted average
#' @export
#' @return List containing the arguments for the grp() function used within Rrepest() function's average() and group() options.
#'
#' @examples
#' append(grp("OECD Average","CNTRY",c("HUN","MEX")), grp("Europe","CNTRY",c("ITA","FRA")))
#' 
grp <- function(group.name, column, cases, full_weight = FALSE) {
  res <- list()
  res[[group.name]] <- list(column = column, cases = cases, full_weight = full_weight)
  return(res)
}
  

#' Weighted variance
#' 
#' @description Computes the weighted variance of a numeric vector.
#' @param x (numeric vector) Variable of interest for which to compute the variance
#' @param w (numeric vector) Vector with weights
#' @param na.rm (bool) if TRUE: Excludes missing values before computing the variance
#'
#' @return Scalar containing the variance
#' @export
#'
#' @examples
#' data(df_talis18) 
#' 
#' weighted.var(df_talis18$tt3g06d1, df_talis18$tt3g06e1)
weighted.var <- function(x, w, na.rm = TRUE) {
  # Goal: Obtained weighted variance de4of a vector
  # ------ INPUTS ------.
  # x : (vector) variable to analyze
  # w : (vector) vector of weights
  # na.rm : (bool) if TRUE (default) remove missing values.
  
  w.var <- sum(w*(x-weighted.mean(x,w,na.rm = na.rm))^2, na.rm = na.rm)/sum(w,na.rm = na.rm)
  return(w.var)
}

#' Grouped frequency counts
#' 
#' @description Computes a data frame with frequency counts.
#' @param data (data frame) Data to analyse
#' @param small.level (string vector) All variables to get grouped sum that will sum up to 100
#' @param big.level (string vector) Variables... Must be fully contained in variables from small.level.
#' @param w (string) Name of the numeric variable representing the weights (optional)
#'
#' @return Data frame containing the frequency counts
#' @export
#'
#' @examples grouped_sum_freqs(data = mtcars,small.level = c("cyl","am"),big.level = c("cyl"))
#' 

grouped_sum_freqs <- function(data, small.level, big.level, w = NULL) {
  # Goal: Get a dataframe with frequencies from the grouped sum of small.level and big.level used for getting percentages
  # Example: cnt gender wealth 
  # ------ INPUTS ------.
  # data : (dataframe) df to analize
  # w : (string) numeric variable from which to get weights (if NULL then 1)
  # small.level : (string vector) all variables to get grouped sum
  # big.level : (string vector) Must be fully contained in variables from small.level 
  
  # If no weights then add 1s
  if(is.null(w)) {
    data$weight <- 1
    w <-"weight"
  }
  
  #Small group
  small.group <- data %>% 
    group_by(across(all_of(small.level))) %>% 
    summarise(small.sum = sum(get(w), na.rm = T))
  
  #Bigger group with variable list fully contained in small group
  big.group <- data %>% 
    group_by(across(all_of(big.level))) %>% 
    summarise(big.sum = sum(get(w), na.rm = T))
  
  # Do frequencies in 100% format. Resulting variable is "freq"
  res <- full_join(small.group,big.group,by = big.level) %>%
    mutate(freq = small.sum / big.sum * 100) %>% 
    select(-small.sum,-big.sum)
  
  return(res)
}

#' Paired independent differences
#' @description
#' Get differences and standard errors from two columns starting with b. and se. assuming independence.
#' The second column will be subtracted from the first.
#' 
#' @param df (dataframe) Dataframe that contains the columns with b. and se. on their name to get the difference from
#' @param col1 (numeric vector) Column name, not including "b.". Must have a column in dataframe also including "se.".
#' @param col2 (numeric vector) Column name, not including "b.". Must have a column in dataframe also including "se.".
#'
#' @return Dataframe with extra columns containing the difference of the columns along with their standard error.
#' @export
#'
#' @examples
#' paired_indep_diff(rrepest_pisa_age_gender,"mean.age..Male","mean.age..Female")
#' 
paired_indep_diff <- function(df, col1, col2){
  # GOAL: Get differences from two columns starting with b. and se. assuming independence
  # Will do the difference of col2 - col1
  # Parameters col1 and col2 are without b. and se.
  
  
  
  b_col1 <- paste0("b.",col1)
  b_col2 <- paste0("b.",col2)
  b_col2_col1 <- paste0("b.(",col2,")-(",col1,")")
  
  se_col1 <- paste0("se.",col1)
  se_col2 <- paste0("se.",col2)
  se_col2_col1 <- paste0("se.(",col2,")-(",col1,")")
  
  res <- df %>% 
    as_tibble() %>% 
    mutate(!!b_col2_col1:=get(b_col2) - get(b_col1),
           !!se_col2_col1:= sqrt(get(se_col2)^2 + get(se_col1)^2))
  
  return(res)
}

#' Independent Differences of columns
#' @description
#' Get in a dataframe all bivariate combinations of differences and standard errors from columns starting with b. and se. assuming independence.
#' For a time series the vector must be order from oldest to newest.
#' 
#' @param df (dataframe) Dataframe that contains the columns with b. and se. on their name to get the difference from
#' @param vec_series (numeric vector) Column names to get difference from, not including "b." at the start. Must have a column in dataframe also including "se.".
#'
#' @return Dataframe with extra columns containing the difference of the columns along with their standard error.
#' @export
#'
#' @examples
#' indep_diff(rrepest_pisa_age_isced, paste0("mean.age..ISCED level ",c("1","3A","2")))
#' 
indep_diff <- function(df, vec_series){
  # GOAL: Get in a dataframe all bivariate combinations of differences and standard errors
  # of the vector vec_series
  # - ARGUMENTS -
  # vec_series (string vector): names of all time period variables without b. and se. from the oldest to the newest
  
  vec_series <- rev(vec_series)
  
  bivariate_time_periods <- lapply(1:choose(length(vec_series),2), function(x_i){
    all_combs <- combn(vec_series,2,simplify = TRUE)
    return(all_combs[,x_i])
  })
  
  res_df <- df
  for(time_periods_i in bivariate_time_periods){
    res_df <- paired_indep_diff(res_df,time_periods_i[1],time_periods_i[2])
  }
  
  return(res_df)
}

################# General statistics on Continuous variables #################
# 12 Feb 2023
# Rodolfo Ilizaliturri
#############################################################
# Goal: Obtain several statistics using Fay's BRR model

# ------------------.
# --------- WORK FUNCTIONS
# ------------------.

# Variance factors for SE
rrvar.pv <- function(df.rr, svy, pv = F, ...) {
  # Goal: Dispersion (variance) from total mean to get SE of statistic
  # ------ INPUTS ------.
  # df.rr : (dataframe) df containing total (1st columns) and replicate weighted statistic ONLY
  # svy : (string) List of possible projects to analyse PIAAC, PISA, TALISSCH and TALISTCH
  # pv : (Bool) TRUE → We are in plausible values and keep calculation as variance, not SE
  
  #Get args ...
  arguments <- list(...)
  
  # f : (number) variance factor (depends on replication method: BRR, jackknife-1, jk-2,…) (Fay = 1.5)
  if (svy %in% c("TALISTCH","TALISSCH","PISA","PISA2015","PBTS","TALISTKS")) f = 1/(0.5^2)
  else if (svy == "SSES") f = 76/2
  else if (svy == "SSES2023") f = 1/(0.5^2) #Changed from Jk1 method to BBR method 
  else if (svy %in% c("ALL","IALS")) f = 30
  else if (svy == "IELS") f = 92/23
  ### for the case of PIAAC, f depends on countries. The correct variance factor will be applied in rrepest_base
  ### before b.se.pv is called.
  else if (svy == "PIAAC") f = 80
  else if (svy == "PISAOOS") f = 29
  else if (svy %in% c("TIMSS","PIRLS")) f = 1/2*150
  else if (svy %in% c("ICCS","ICILS")) f = 1*75
  else if (svy %in% c("ICCS_T","ICILS_T")) f = 1*75
  else if (svy %in% c("ICCS_C","ICILS_C")) f = 1*75
  else if (svy %in% c("LEADER_TALISEC","STAFF_TALISEC","TALISEC_LEADER","TALISEC_STAFF")) f = 92/23

  
  #if there is custom variance factor grab it from the arguments
  if (!is.null(arguments$var.factor)) {
    f = arguments$var.factor
  }
  
  # Calculate disperion from the mean squared time variance factor according to survey
  m <- df.rr[1]
  vec <- df.rr[-1]
  rr.var <- mean((m - vec)^2)*f
  
  if (!pv) {
    # Only one estimation (n.PV = 1), then can take sqrt of Var to get SE
    rr.se <- sqrt(rr.var)
    return(rr.se)
  } else {
    # Return variance for calculation of Sampling and Imputation variance
    return(rr.var)
  }
}

#Number of observations valid for x
n.obs.x <- function(df, by.var, x, svy) {
  # Goal: Number of schools and teachers calculation is performed on
  # ------ INPUTS ------.
  # df : (dataframe) df to analyze
  # by.var : (string) column in which we'll break down results
  # x : (string) variable from where to get means
  # svy : List of possible projects to analyse PIAAC, PISA, TALISSCH and TALISTCH
  
  #All lower caps column names
  colnames(df) <- colnames(df) %>% tolower()
  by.var <- by.var %>% tolower()
  x <- x %>% tolower()
  
  tot.n <- df %>%
    group_by(across(all_of(by.var))) %>%
    {if (svy == "TALISSCH") summarise(., n.obs = sum(!is.na(get(x)))) 
      else if (svy %in% c("TALISTCH","TALISTKS")) {mutate(.,school.n = ifelse(is.na(get(x)) == T, NA, .data$idschool)) %>% 
          summarise(., n.obs = sum(!is.na(get(x))),
                    n.sch = n_distinct(school.n, na.rm = T))}
      else if (svy == "TALISEC_STAFF") {mutate(.,school.n = ifelse(is.na(get(x)) == T, NA, .data$idcentre)) %>% 
          summarise(., n.obs = sum(!is.na(get(x))),
                    n.sch = n_distinct(school.n, na.rm = T))}
      else if (svy == "PISA2015") {mutate(.,school.n = ifelse(is.na(get(x)) == T, NA, .data$cntschid)) %>% 
          summarise(., n.obs = sum(!is.na(get(x))),
                    n.sch = n_distinct(school.n, na.rm = T))}
      else if (svy == "PISA") {mutate(.,school.n = ifelse(is.na(get(x)) == T, NA, .data$schoolid)) %>% 
          summarise(., n.obs = sum(!is.na(get(x))),
                    n.sch = n_distinct(school.n, na.rm = T))}
      else if (svy %in% c("SSES","SSES2023")) {mutate(.,school.n = ifelse(is.na(get(x)) == T, NA, .data$schid)) %>% 
          summarise(., n.obs = sum(!is.na(get(x))),
                    n.sch = n_distinct(school.n, na.rm = T))}
      else summarise(., n.obs = sum(!is.na(get(x))))
    } %>%
    unite("by.group", all_of(by.var), sep = "|")
  
  #add x colanames
  colnames(tot.n)[-1] <- paste(colnames(tot.n)[-1], tolower(x), sep = ".")
  
  return(tot.n)
}
# n.obs.x(df = df.qqq,by.var = "cnt",x = "wb173q03ha",svy = "PISA2015")
# n.obs.x(df = df.p,by.var = "cntry",x = "tc3g06b",svy = "TALISSCH")
# n.obs.x(df = df.t,by.var = "cntry",x = "tt3g01",svy = "TALISTCH")


#' Formatting target, by, and over variables for Rrepest.
#'
#' @param df (data frame) Data to analyze.
#' @param svy (string) Possible projects to analyse: PIAAC, PISA, TALISSCH, TALISTCH, etc.
#' @param x (string vector) Target variables.
#' @param by.over (string vector) Variables to break analysis by.
#' @param user_na (bool) TRUE → show nature of user defined missing values
#' @param ... Optional arguments such as custom weights (cm.weights) 
#'
#' @return Data frame with variables in numeric format for analysis.
#' @export
#'
#' @examples
#' df1 <- format_data_repest(df_pisa18, "PISA", "pv1math", "cnt")
#' df2 <- format_data_repest(df_pisa18, "PISA", "pv1math", c("cnt","st004d01t"))
format_data_repest <- function(df, svy, x, by.over, user_na = F, ...) {
  # Goal: Get database into appropriate format
  # ------ INPUTS ------.
  # df : (dataframe) df to analyze
  # svy : List of possible projects to analyse PIAAC, PISA, TALISSCH and TALISTCH
  # by.over : (string vector) variables to break analysis by
  # user_na : (Bool) TRUE → show nature of user defined missing values
  
  #Get isced from ...
  arguments <- list(...)
  
  # Lowercase variables
  names(df) <- names(df) %>% tolower()
  
  # If isced specified for TALIS filter
  if (svy %in% c("TALISSCH","TALISTCH") & !is.null(arguments$isced)) {
    df <- df %>% 
      filter(idpop %in% arguments$isced)
  }
  
  # Weight names depending on project (TALIS, PISA, etc.)
  if (svy %in% c("TALISTCH","TALISTKS")) weight.variables <- c("tchwgt",paste0("trwgt",1:100))
  else if (svy == "TALISSCH") weight.variables <- c("schwgt",paste0("srwgt",1:100))
  else if (svy %in% c("PISA","PISA2015")) weight.variables <- c("w_fstuwt",paste0("w_fsturwt",1:80))
  else if (svy == "SSES") weight.variables <- c("wt2019",paste0("rwgt",1:76))
  else if (svy == "SSES2023") weight.variables <- c("wt2023",paste0("rwgt",1:80))
  else if (svy == "PISAOOS") weight.variables <- c("spfwt0",paste0("spfwt",1:30))
  else if (svy == "TALISEC_STAFF") weight.variables <- c("staffwgt",paste0("srwgt",1:92))
  else if (svy == "TALISEC_LEADER") weight.variables <- c("cntrwgt",paste0("crwgt",1:92))
  else if (svy == "PIAAC") weight.variables <- c("spfwt0",paste0("spfwt",1:80))
  else if (svy %in% c("ALL","IALS")) weight.variables <- c("popwt",paste0("replic",1:30))
  else if (svy == "IELS") weight.variables <- c("childwgt",paste0("srwgt",1:92))
  else if (svy =="PBTS") weight.variables <- c("w_stu",paste0("rwgt",1:80))
  else if (svy %in% c("TIMSS","PIRLS")) weight.variables <- c("wgr",paste0("jr",1:150))
  else if (svy %in% c("ICCS","ICILS")) weight.variables <- c("totwgts",paste0("srwgt",1:75))
  else if (svy %in% c("ICCS_T","ICILS_T")) weight.variables <- c("totwgtt",paste0("trwgt",1:75))
  else if (svy %in% c("ICCS_C","ICILS_C")) weight.variables <- c("totwgtc",paste0("crwgt",1:75))
  else if (svy == "SVY") weight.variables <- arguments$cm.weights #custom optional weights that must exist if "SVY"
  
  #if there is custom name of weights grab it from the arguments
  if (!is.null(arguments$cm.weights)) {
    weight.variables <- arguments$cm.weights #custom optional weights
  }
  
  #if PIAAC, compute variance factor at the respondent level
  if (svy=="PIAAC") {
    df <- df %>% mutate(var_fac=
                          case_when(vemethodn==1 ~ (venreps-1)/venreps,
                                    vemethodn==2 ~ 1,
                                    vemethodn==3 ~ 1/venreps,
                                    vemethodn==4 ~ 1/(venreps*(1-vefayfac)^2)
                          ))
  }
  
  
  #If x has an @ for PVs
  if (grep("@", x) %>% length() != 0) {
    x <- get.pv.names(df, x)
  }
  
  num.vars <- c(weight.variables, x)
  
  #Convert x and weights to NUMERIC variables removing user defined na and just leave NA
  df.res <- df %>% 
    mutate(across(contains(num.vars) & ( where(is.labelled)), remove_user_na, user_na_to_na = T)) %>%
    mutate(across(contains(num.vars) & ( where(is.labelled)), as.numeric))
  
  #If we want to know NA identity or not (eg. Not reached, Absent)
  if (!user_na) {
    df.res <- df.res %>% 
      mutate_at(by.over,to_character, user_na_to_na = TRUE)
  }else{
    df.res <- df.res %>% 
      mutate_at(by.over,to_character)
  }
  
  #Show variables that create over categories in column names of results
  if (!is.null(arguments$show_over_vars)) {
    for (i in tolower(arguments$show_over_vars)) {
      df.res[[i]] <- paste(i, df.res[[i]], sep = "..")
    }
  }
  
  
  return(df.res)
}

do_group <- function(data, grp.l, user_na = F, ...){
  # Goal: Create dataframe with the groups on the variables and values defined
  # ------ INPUTS ------.
  # data : (dataframe) Data to analyze
  # grp.l : (list) list of groups to redifine {group_name = {column, values_in_group},...}
  res.l <- lapply(names(grp.l), function(g) {
    # Assign name of group to all the column value
    data %>% 
      filter(get(grp.l[[g]][["column"]]) %in% grp.l[[g]][["cases"]]) %>% 
      mutate(!!grp.l[[g]][['column']] := g) %>% 
      return()
  })
  
  
  for (g.i in sapply(names(grp.l),function(g) grp.l[[g]][["column"]]) %>% unique()) {
    #If we want to know NA identity or not (eg. Not reached, Absent) to get correct format in column
    if (!user_na) {
      data <- data %>% 
        mutate_at(g.i,to_character, user_na_to_na = TRUE)
    }else{
      data <- data %>% 
        mutate_at(g.i,to_character)
    }
  }
  

  return(append(res.l,data))
  
}

#Check existance of argument and use it if so
is.there <- function(argument) {
  if(!is.null(argument)) return(argument)
  else return(NULL)
}


#Plausible Values names from unknown digits of PVs
get.pv.names <- function(df, pv.var) {
  # Goal: Do means, variance or standard deviation
  # ------ INPUTS ------.
  # df : (dataframe) df to analize
  # pv.var : variable that contains an @ symbol to interpret as PVs
  
  pv.split <- strsplit(pv.var,"@")[[1]]
  pv.gex <- paste0("^",pv.split[1],".*",pv.split[2],"$")
  pv.names <- grep(pv.gex,colnames(df) %>% tolower(), value = T)
  
  return(pv.names)
}
#EX. x <- get.pv.names(df,"pv@math")

#Weight names
replicated_w_names <- function(svy, ...) {
  # Goal: Names of weighting variables corresponding to survey
  # ------ INPUTS ------.
  # svy : List of possible projects to analyse PIAAC, PISA, TALISSCH and TALISTCH
  
  #Get custom arguments...
  arguments <- list(...)
  
  # Weight names depending on project (TALIS, PISA, etc.)
  if (svy %in% c("TALISTCH","TALISTKS")) weight.variables <- c("tchwgt",paste0("trwgt",1:100))
  else if (svy == "TALISSCH") weight.variables <- c("schwgt",paste0("srwgt",1:100))
  else if (svy %in% c("PISA","PISA2015")) weight.variables <- c("w_fstuwt",paste0("w_fsturwt",1:80))
  else if (svy == "PISAOOS") weight.variables <- c("spfwt0",paste0("spfwt",1:30))
  else if (svy == "SSES") weight.variables <- c("wt2019",paste0("rwgt",1:76))
  else if (svy == "SSES2023") weight.variables <- c("wt2023",paste0("rwgt",1:80))
  else if (svy == "TALISEC_STAFF") weight.variables <- c("staffwgt",paste0("srwgt",1:92))
  else if (svy == "TALISEC_LEADER") weight.variables <- c("cntrwgt",paste0("crwgt",1:92))
  else if (svy == "PIAAC") weight.variables <- c("spfwt0",paste0("spfwt",1:80))
  else if (svy %in% c("ALL","IALS")) weight.variables <- c("popwt",paste0("replic",1:30))
  else if (svy == "IELS") weight.variables <- c("childwgt",paste0("srwgt",1:92))
  else if (svy =="PBTS") weight.variables <- c("w_stu",paste0("rwgt",1:80))
  else if (svy %in% c("TIMSS","PIRLS")) weight.variables <- c("wgr",paste0("jr",1:150))
  else if (svy %in% c("ICCS","ICILS")) weight.variables <- c("totwgts",paste0("srwgt",1:75))
  else if (svy %in% c("ICCS_T","ICILS_T")) weight.variables <- c("totwgtt",paste0("trwgt",1:75))
  else if (svy %in% c("ICCS_C","ICILS_C")) weight.variables <- c("totwgtc",paste0("crwgt",1:75))
  else if (svy == "SVY") weight.variables <- arguments$cm.weights #custom optional weights that must exist if "SVY"
  
  #if there is custom name of weights grab it from the arguments
  if (!is.null(arguments$cm.weights)) {
    weight.variables <- arguments$cm.weights #custom optional weights
  }
  
  return(c(weight.variables[1], weight.variables[-1]))
}
#EX. wts <- replicated_w_names("PISA")
#EX. wts <- replicated_w_names("TALISTCH")

add_PIAAC_variance_factors <- function(pv.l, pv, df,by.var){
  # GOAL: multiply each se in pv.l by the variance factor specific to the by.var
  # ------ INPUTS ------
  # pv.l (list of df) for each pv, a df with bs and ses
  # pv (boolean) is there any pv
  # df (df) respondent level data
  # by.var (string) value of the by.var
  
  
  #in case the value of the variance factor is not constant within the estimation sample:
  # take the average, and then print a message
  # can happens when the sample contains several countries 
  var_facs <- df %>%  group_by(all_of(across(by.var))) %>%  
    distinct(across("var_fac")) %>% 
    rename(`by.var`=by.var)
  
  check <- var_facs %>%  count %>% 
    ungroup %>% summarize(max(n)) %>%  pull
  
  if (check>1) {
    var_facs <- var_facs %>%  group_by(all_of(across(by.var))) %>% 
      summarise(var_fac=mean(.data$var_fac, na.rm=TRUE))
    message("Variance factors are not constant within by.var")
  } 
  
  # if there is no pv the 'ses' in pv.l are multiplied by the sqrt of the variance factor
  if (!pv) {
    pv.l[[1]] <-  pv.l[[1]] %>% full_join(var_facs, by= "by.var") %>% 
      mutate(across(starts_with("se"),
                    ~ .x *sqrt(.data$var_fac))) %>% 
      select(-"var_fac")
  } else {
    # if there is a pv the 'ses' in pv.l are multiplied by the variance factor 
    # (in such a case, the 'ses' are actually the square of the true ses)!!!
    pv.l <- lapply(pv.l, FUN = function(pv.i) {
      pv.i %>% full_join(var_facs, by= "by.var") %>% 
        mutate(across(starts_with("se"),
                      ~ .x *.data$var_fac)) %>% 
        select(-"var_fac")
    })
    
  }
  
  
  return(pv.l)
}





get.pv.arguments <- function(digit.pvs, argument) {
  # Goal: Replace @ with all preset PV digits from arguments
  # ------ INPUTS ------.
  # digit.pvs (Numeric vector) : 1:x List of possible digits (comes from digits.pv)
  # argument (String vector) : names of variables to analyse, at least one with a @
  
  res <- lapply(digit.pvs, function(x.i){
    gsub("@", as.character(x.i), argument)
  }) %>%
    # Merge all with only unique elements
    reduce(union)
  
  return(res)
}

#Digits in PVs
digits.pvs <- function(svy, ...) {
  # Goal: Number of plausible values corresponding to survey
  # ------ INPUTS ------.
  # svy : List of possible projects to analyse PIAAC, PISA, TALISSCH and TALISTCH
  
  # Extract number of pvs if svy = "SVY"
  arguments <- list(...)
  
  # Number of plausible values (TALIS, PISA, etc.)
  if (svy %in% c("PISA2015","PISAOOS","PIAAC","ALL","IALS")) n.d.pvs <- 1:10
  else if (svy %in% c("PISA","IELS","PBTS","ICCS", "TALISTKS",
                      "ICILS","ICCS_T","ICILS_T",
                      "ICCS_C","ICILS_C","TIMSS","PIRLS")) n.d.pvs <- 1:5
  
  #if there is custom number of pvs grab it from the arguments
  if (!is.null(arguments$n.pvs)) {
    n.d.pvs <- 1:arguments$n.pvs
  }
  
  
  return(n.d.pvs)
}

#Statistics
weighted.var <- function(x, w, na.rm = T) {
  # Goal: Obtained weighted variance of a vector
  # ------ INPUTS ------.
  # x : (vector) variable to analyze
  # w : (vector) vector of weights
  # na.rm : (bool) if TRUE (default) remove missing values.
  
  w.var <- sum(w*(x-weighted.mean(x,w,na.rm = na.rm))^2, na.rm = na.rm)/sum(w,na.rm = na.rm)
  return(w.var)
}
weighted.std <- function(x, w, na.rm = T) {
  # Goal: Obtained weighted standard deviation of a vector
  # ------ INPUTS ------.
  # x : (vector) variable to analyze
  # w : (vector) vector of weights
  # na.rm : (bool) if TRUE (default) remove missing values.
  
  w.std <- (sum(w*(x-weighted.mean(x,w,na.rm = na.rm))^2, na.rm = na.rm)/sum(w,na.rm = na.rm))^(1/2)
  return(w.std)
}

#Core loop
doing_statistic_string <- function(statistic, over.by){
  # Goal: Have string to be evaluated on loop and grouped by
  # ------ INPUTS ------.
  # over.by : (string vector) variables to break analysis by
  # statistic : (string) accepts "mean","var","std", "quant", "iqr"
  
  #After, need to fo eval(parse(text= "..."))
  
  #Create string for over.by in data.table
  over.by.str <- paste0("c('",paste0(over.by,collapse = "','"),"')")
  
  #Do string for data.table and statistic
  if (statistic %in% c("quant","iqr")) {
    fun.str <- paste0("dt[, .(w.",
                      statistic," = weighted.",
                      statistic,"(get(x.i),get(w.i),q.iqr)), by = ",
                      over.by.str,"]")

  }else{
    fun.str <- paste0("dt[, .(w.",
                      statistic," = weighted.",
                      statistic,"(get(x.i),get(w.i),na.rm = T)), by = ",
                      over.by.str,"]")
  }
  return(fun.str)
}

#Sampling error
# TALIS
brr.se <- function(v, Fay = 0.5) {
  # Goal: Dispersion (like variance) from total mean to get SE of statistic
  # ------ INPUTS ------.
  # v : (dataframe) df containing total and replicate weighted statistic
  # Fay : (number) Number corresponding to adjustment by method (Fay = 1.5)
  m <- v[1]
  vec <- v[-1]
  rr.var <- mean((m - vec)^2)*1/Fay^2
  rr.se <- sqrt(rr.var)
  
  return(rr.se)
  
}
# PISA

vectorize.stat.on.weights <- function (df, statistic = "mean", rep_weights, x, by.var, over = NULL, test = F, ...) {
  # Goal: Get list of dataframes with total and replicated weight estimates for x (either x or pvs)
  # ------ INPUTS ------.
  # df : (dataframe) df to analize
  # rep_weights : (string vector) names of replicated weight vars
  # over : (vector string) columns over which to do analysis
  # x: (string vector) variable or pvs from wich to extract statistics
  # by.var : (string vector) variables to break analysis by
  # statistic : (string) accepts "mean","var","std", "quant", "iqr"
  # ... Dots
  # q.iqr : (vector numeric) of length 1 if quantile, of length 2 if interquantile range
  
  arguments <- list(...)
  
  dt <- df %>%
    select(all_of(c(rep_weights, x, by.var, over))) %>% #Select only data to analyze
    filter_at(vars(all_of(x)), all_vars(!is.na(.))) %>%  #Filter on only non NA values move here due to performance
    as.data.table() # Convert to data.table
  
  
  # Create string to be evaluated
  # If quantile or iqr add q.iqr to string
  if (!is.null(arguments$q.iqr)) {
    stat.str <- doing_statistic_string(statistic, c(by.var, over))
    stat.str <- sub("q.iqr",
                    paste0("c(",paste0(arguments$q.iqr,collapse = ","),")"),
                    stat.str)
  } else {
    stat.str <- doing_statistic_string(statistic, c(by.var, over))
  }
  
  #----- Loop over x's
  res.x.par <- lapply(x, function(x.i) {
    #----- Loop over weights
    res.w.par <- lapply(rep_weights, function(w.i) {
      #----- Calculate statistic from string stat.str
      res.col <- eval(parse(text = stat.str))
    
      #------ If test for over add tests to res.col
      if (test) {
        res.col <- over.test(res.col, over) %>% 
          column_to_rownames(var= "by.group")
      }else{
        res.col <- res.col %>% 
          remove_rownames %>%
          unite("by.group", all_of(c(by.var,over)), sep = "|") %>% 
          column_to_rownames(var= "by.group")
      }
      
      # -- Track weight number
      names(res.col)[ncol(res.col)] <- paste0("w.",w.i)
      return(res.col)
    })
    res.w.par <- res.w.par %>% reduce(cbind) %>% as.data.frame()
  })
  
  return(res.x.par)
}
#EX. res.list <- vectorize.stat.on.weights(df,"mean",wts,"pv1math","cnt")
#EX. res.list <- vectorize.stat.on.weights(df,"mean",wts, x,"cnt")
#EX. res.list <- vectorize.stat.on.weights(df,"mean",wts,"pv1math","cnt","st004d01t")
#EX. res.list <- vectorize.stat.on.weights(df,"mean",wts,"pv1math","cnt","st004d01t",T)
#EX. res.list <- vectorize.stat.on.weights(df,"mean",wts,"pv1math","cnt",c("st004d01t","iscedl"),T)
#EX. res.list <- vectorize.stat.on.weights(df,"mean",wts,"pv1math","cnt",c("iscedl","st004d01t"),T)
#EX. res.list <- vectorize.stat.on.weights(df,"mean",wts, "tt3g02","cntry")
#EX. res.list <- vectorize.stat.on.weights(df,"quant",wts, "tt3g02","cntry", q.iqr = .5)
#EX. res.list <- vectorize.stat.on.weights(df,"iqr",wts, "tt3g02","cntry", q.iqr = c(.5,.9))


rrepest.stats <- function(df, svy, statistic = "mean", x, by.var, over, test, show.n = "n", user_na=F, ...){
  # Note: Same as rrepest.gstats without the number of schools and teachers
  # Goal: Dataframe with statistic and SE by Fay's BRR model
  # ------ INPUTS ------.
  # df : (dataframe) df to analyze
  # svy : (string) List of possible projects to analyse PIAAC, PISA, TALISSCH and TALISTCH
  # statistic : (string) accepts "mean","var","std"
  # by.var : (string) column in which we'll break down results !IOP!: several variables
  # x : (string) variable from where to get statistics
  # over : (vector string) columns over which to do analysis
  # test : (Bool) make differences across over categories
  # user_na : (Bool) TRUE → show nature of user defined missing values in by.var
  # show.n : (string options) "n" shows n values and keeps all data, "flag"" removes n values and places NaN in rows
  # ... Dot arguments
  # isced : (number) isced level to analyze
  
  #grab optional arguments
  extra <- list(...)
  
  # If there is an OVER variable the data must be flagged and remove NAs from OVER vars
  if (length(over) > 0) {
    show.n <- "flag"
    
    #remove NAs on over
    for (i in over) {
      data <- data %>% drop_na(i)
    }
  }
  # If there is an @ in x grab PVs
  if (grep("@", x) %>% length() != 0) {
    x <- get.pv.names(df.qqq, x)
  }
  # Get weight names
  wts <- replicated_w_names(svy)
  # Format data according to survey
  df <- format_data_repest(df, svy, x, c(by.var,over), user_na, isced = is.there(extra$isced))
  
  # Loop over weights
  res.list <- vectorize.stat.on.weights(df = df,
                                        statistic = statistic,
                                        rep_weights = wts,
                                        x = x,
                                        by.var = by.var,
                                        over = over,
                                        test = test)
  
  
  
}



separate_test_name <- function(test_name){
  # GOAL: Create two strings from the test column name for coverage
  # ------ INPUTS ------.
  # test_name : (string) string containing column name from test with structure nn.(x-y) 
  name_parts <- strsplit(test_name,"..",fixed = TRUE)[[1]]
  test_parts <- strsplit(name_parts[length(name_parts)],"-")[[1]]
  
  string_1 <- paste(c(name_parts[1:(length(name_parts)-1)],substr(test_parts[1],2,nchar(test_parts[1]))),collapse = "..")
  string_2 <- paste(c(name_parts[1:(length(name_parts)-1)],substr(test_parts[2],1,nchar(test_parts[2])-1)),collapse = "..")
  return(c(string_1,string_2))
}

nans_from_se2b <- function(df){
  # GOAL: Force NaNs that appear in se. columns to b. columns from coverage and flags
  # ------ INPUTS ------.
  # df : (dataframe) Resulting df from analysis with se. and b. columns
  for (col_i in names(df)){
    # Get se. columns
    if(startsWith(col_i,"se")){
      # Find the NaN in the column
      nan_s <- which(is.nan(df[col_i][[1]]))
      # If there are, replace the same values with NaN in b.
      if (length(nan_s)>0){
        common_name <- substr(col_i,4,nchar(col_i))
        df[paste0("b.",common_name)][[1]][nan_s] <- NaN
      }
    }
  }
  return(df)
}

n_obs_sch_freq_fix <- function(n_df, by.var, over, x){
  # Goal: Adjust n for observations ans school for frequencies to use in flags.
  # The rule followed by PISA for flagging simple breakdowns/frequencies is to apply the flag at the QUESTION level.
  # ------ INPUTS ------.
  # n_df : (dataframe) Data with n of observations and schools
  # by.var : (string) column in which we'll break down results !IOP!: several variables
  # x : (string) variable from where to get statistics
  # over : (vector string) columns over which to do analysis
  n_df %>% 
    # Separete x from all else
    separate(col = by.group,into = c(by.var, over, x), sep = "\\|") %>%
    unite(col= "by.var.nox",c(by.var, over),sep = "|") %>% 
    # Sum by breakdown excluding x
    group_by(.data$by.var.nox) %>% 
    mutate(across(where(is.numeric), ~ sum(.,na.rm = TRUE))) %>% 
    ungroup() %>% 
    # Reunite into by.group
    unite(col = "by.group",c("by.var.nox",x),sep = "|")
}

se_to_na <- function(res_df, mean_stat = FALSE){
  # GOAL: Turn se with 0 to NA from a mean, meanpct, or freq with 1, 100, or 0 
  # ------ INPUTS ------.
  # res_df: (dataframe) Results from Rrepest with b. and se. columns
  # mean_stat: (bool) TRUE mean used in statistics, add it to the columns to search for
  
  # if mean used in statistics add it to the columns to search for
  if(mean_stat) mean_string <- "mean" else mean_string <- ""
  # Iterate through all names and find b.
  for(col_i in names(res_df)){
    if(grepl(paste0("^b\\.",mean_string), col_i)){
      # Replace in corresponding se. the 0 with an NA if value for b. is 0. 1 or 100 (only mean and frequency)
      se_i <- paste0("se.",substring(col_i,3), collapse = "")
      # Which are 0 se. and 1, 100, or 0 b.
      se_to_remove <- intersect(which(res_df[[col_i]] %in% c(0,100)), # b. is 0, 1 or 100
                                which(res_df[[se_i]] %in% c(0))) # se. the 0 
      # Set those as NaN
      res_df[[se_i]][se_to_remove] <- NaN
    }
  }
  return(res_df)
}

################# Weighted correlation with PVs #################
# 7 April 2023
# Rodolfo Ilizaliturri
#############################################################
# Goal: Do weighted correlation

# Bivariate correlation and covariance ------------------------------------
################# Weighted correlation #################
# 11 March 2023
# Rodolfo Ilizaliturri
#############################################################
# Goal: Do weighted correlation

# Bivariate correlation and covariance ------------------------------------

weighted.corr <- function(x, y, w, na.rm = T) {
  # Goal: Get weighted correlation
  # ------ INPUTS ------.
  # x: (vector) variable from where to get correlation
  # y: (vector) variable from where to get correlation
  # w : (vector) vector of weights
  
  # Use non NAs in both x and y
  use <- !is.na(x) & !is.na(y)
  x <- x[use]
  y <- y[use]
  w <- w[use]
  
  # Calculate variance and covariance of x and y
  w.mean.x <- weighted.mean(x, w, na.rm = na.rm)
  w.mean.y <- weighted.mean(y, w, na.rm = na.rm)
  cov.w <- sum(w*(x-w.mean.x)*(y-w.mean.y), na.rm = na.rm)
  var.w.x <- sum(w*(x-w.mean.x)^2, na.rm = na.rm)
  var.w.y <- sum(w*(y-w.mean.y)^2, na.rm = na.rm)
  
  return(cov.w / sqrt(var.w.x * var.w.y))
}
weighted.cov <- function(x, y, w, na.rm = T) {
  # Goal: Get weighted covariance
  # ------ INPUTS ------.
  # x: (vector) variable from where to get correlation
  # y: (vector) variable from where to get correlation
  # w : (vector) vector of weights
  
  # Use non NAs in both x and y
  use <- !is.na(x) & !is.na(y)
  x <- x[use]
  y <- y[use]
  w <- w[use]
  
  # Covariance
  w.mean <- weighted.mean(x, w, na.rm = na.rm)
  cov.w <- sum(w * (x - w.mean) * (y - w.mean), na.rm = na.rm)/sum(w, na.rm = na.rm)
  return(cov.w)
}

# Ex. ----
# df.tests <- df.t[c("CNTRY","T3STAKE","T3TEAM","T3STUD","IDPOP","TCHWGT")] %>% 
#   filter(IDPOP == 2,
#          CNTRY == "ABA") %>% 
#   user_na_to_na()
# 
# weighted.corr(df.tests$T3STAKE,df.tests$T3TEAM,df.tests$TCHWGT)

# ----


# Multivariate correlation and covariance ---------------------------------
# Does all 2 Choose length(vector) correlations
weighted.corr.cov.n <- function(data, x, w =  rep(1, length(data[x[1]])), corr = T, na.rm = T) {
  # Goal: Get weighted correlation or covariance of n variables
  # ------ INPUTS ------.
  # data : (dataframe) df to analyze
  # x: (vector string) variables from where to get correlation/covariance
  # w : (string) weight name
  # corr : (bool) T → get correlation, F → get covariance
  
  # Get dataframe of correlation
  vec.res <- apply(as.data.frame(combn(x,2)), 2, function(x.i) {
    if (corr) {
      res <- weighted.corr(x = data[x.i[1]],
                           y = data[x.i[2]], 
                           w = data[w],
                           na.rm = na.rm)
    }else{
      res <- weighted.cov(x = data[x.i[1]],
                          y = data[x.i[2]], 
                          w = data[w],
                          na.rm = na.rm)
    }
    
    return(res)
  })
  # Convert vector to dataframe if 1 by n
  vec.res <- data.frame(t(vec.res))
  
  # Get column names
  vec.names <- apply(as.data.frame(combn(x,2)), 2, function(x.i) {
    if (corr) {
      res <- paste0("corr.",x.i[1],"&",x.i[2])
    }else{
      res <- paste0("cov.",x.i[1],"&",x.i[2])
    }
    return(as.vector(res))
  })
  colnames(vec.res) <- vec.names
  
  return(vec.res)
}
# Ex. weighted.corr.cov.n(df.tests,c("T3STAKE","T3TEAM","T3STUD"),"TCHWGT")

# Special to be faster in the loop
weighted.corr.cov.n.gby <- function(x, w, corr = T) {
  # Goal: Create string to be later evaluated in group.by with dplyr
  # ------ INPUTS ------.
  # x: (vector string) variables from where to get correlation/covariance
  # w : (string) weight name
  # corr : (bool) T → get correlation, F → get covariance
  
  # Get strings of correlation for group_by
  str.res <- apply(as.data.frame(combn(x,2)), 2, function(x.i) {
    if (corr) {
      res <- paste0( "corr.", x.i[1], "_", x.i[2]," = weighted.corr(get('",x.i[1],"'),get('",x.i[2],"'),get('",w,"'))" )
      
    }else{
      res <- paste0( "cov.", x.i[1], "_", x.i[2]," = weighted.cov(get('",x.i[1],"'),get('",x.i[2],"'),get('",w,"'))" )
    }
    return(res)
  })
  
  
  return(paste(str.res, collapse = ","))
}
# Ex. ----
# eval(parse(text = 
#              paste0("df.t %>% group_by(CNTRY) %>% summarise(",
#                     weighted.corr.cov.n.gby(c("T3STAKE","T3TEAM","T3STUD"),"TCHWGT"),
#                     ")")
# ))
# ----
weighted.corr.cov.n.gby.PAR <- function(x, w, corr = T, ...) {
  # Goal: Create string to be later evaluated in group.by with dplyr
  # ------ INPUTS ------.
  # x: (vector string) variables from where to get correlation/covariance
  # w : (string) weight name
  # corr : (bool) T → get correlation, F → get covariance
  # ... (function) weighted.corr, weighted.cov
  
  # Get optional arguments
  arg <- list(...) 
  
  # Get strings of correlation for group_by
  str.res <- apply(as.data.frame(combn(x,2)), 2, function(x.i) {
    if (corr) {
      res <- paste0( "corr.", x.i[1], "_", x.i[2]," = weighted.corr(get('",x.i[1],"'),get('",x.i[2],"'),get('",w,"'))" )
      
    }else{
      res <- paste0( "cov.", x.i[1], "_", x.i[2]," = weighted.cov(get('",x.i[1],"'),get('",x.i[2],"'),get('",w,"'))" )
    }
    return(res)
  })
  
  
  return(paste(str.res, collapse = ","))
}
# Loop on weights for corr and cov ------------------------------------
pv.loop.corr.cov.on.weights <- function(data, svy, x, by.var, over, test = F, flag = F, rep_weights, corr = T, pv = F) {
  # Goal: List with all weighted frequencies of x by by.var
  # ------ INPUTS ------.
  # data : (dataframe) df to analize
  # w : (string) name of replicated weight var
  # x : (string) lvariable from which to get frequencies
  # by.var : (string vector) variables to break analysis by
  # over : (vector string) columns over which to do analysis
  # test : (bool) If TRUE will calculate the difference between over variables
  # rep_weights : (string vector) names of replicated weight vars
  # flag : (Bool) TRUE → Show NaN when there is not enough coverage
  # svy : List of possible projects to analyse PIAAC, PISA, TALISSCH and TALISTCH
  # corr : (bool) TRUE → get correlation, F → get covariance
  # pv : (Bool) TRUE → We are in plausible values and must parallelize
  
  if (!pv) {
    # NON PARALLEL ------------------------------------------------------------.
  
    ###################################################
    ################ Loop correlations ################ 
    res.l <- lapply(rep_weights, function(w.i) {
      res.str <- paste0("data %>% group_by(across(all_of(c(by.var,over)))) %>% summarise(",weighted.corr.cov.n.gby(x = x, w = w.i, corr = corr),")")
      
      res.df <- eval(parse(text = res.str)) %>% 
        ungroup()
      
      # Put together all categorical variables into by.var
      res.df <- res.df %>% 
        unite("by.var", all_of(c(by.var, over)), sep = "|")
      return(res.df)
    })
    ##################################################
  
    # PARALLEL ---------------------------------------------------------------.
  }else{ 
    res.l <- foreach(w.i = rep_weights,
                     .packages = c("dplyr","tidyr"),
                     .export = c("weighted.corr.cov.n.gby.PAR","weighted.corr",
                                 "weighted.cov")) %dopar% {
                                  # would remove NAs from over variables here
                                   res.str <- paste0("data.par %>% group_by(across(all_of(c(by.var,over)))) %>% summarise(",
                                                     weighted.corr.cov.n.gby.PAR(x = x, 
                                                                                 w = w.i, 
                                                                                 corr = corr, 
                                                                                 weighted.corr = weighted.corr,
                                                                                 weighted.cov = weighted.cov),")")
                                   
                                   res.df <- eval(parse(text = res.str)) %>% 
                                     ungroup()
                                   
                                   # Put together all categorical variables into by.var
                                   res.df <- res.df %>% 
                                     unite("by.var", all_of(c(by.var, over)), sep = "|")
                                   
                                   return(res.df) 
                                 }
  }
  # ---------------- FLAGS
  if (flag) {
    # Get n for flags and separate column
    n.df <- n.obs.x(df = data, by.var = c(by.var, over), x = x, svy = svy)
    
    res.l <- lapply(res.l, function(res.i){
      # Merge both dfs together n and freqs
      res.df <- left_join(res.i, n.df, by = c("by.var"="by.group"))
      
      #Assign NaN when not enough coverage for estimation
      res.df <- flags(data = res.df, svy = svy)
      
      return(res.df)
    })
  }
  # ---------------- .
  
  # ---------------- TEST
  if (test) {
    # Create extra rows with test differences
    res.l <- lapply(res.l, function(res.i){
      # Split by.var into c(by.var, over) by |
      res.df <- res.i %>% 
        separate(col = "by.var", into = all_of(c(by.var, over)), sep = "\\|") %>% 
        over.test(over = all_of(c(over))) %>% 
        rename("by.var"="by.group")
      
      return(res.df)
    })
  } 
  # ---------------- .
  
  # ---------------- FORMAT DF to have over on the columns
  res.l <- lapply(res.l, function(res.i){
    res.df <- res.i %>% 
      # Split by.var leaving only the variables in "by.var" united 
      separate(col = "by.var", into = all_of(c(by.var, over)), sep = "\\|") %>% 
      unite(col = "by.var", all_of(by.var), sep = "|")
    
    if (length(over) >= 1) {
      # Pivot to columns over
      if (ncol(select_if(res.df,is.numeric)) > 1) {
        # If more than one numeric column just pivot
        res.df <- res.df %>% 
          pivot_wider(names_from = all_of(c(over)),
                      # grab only numeric columns after by.var, over, and x
                      values_from = colnames(select_if(.,is.numeric)),
                      names_sep = "..")
      } else {
        # If only one numeric column
        
        # Original colname
        cor.col.name <- colnames(res.df)[length(res.df)]
        
        res.df <- res.df %>% 
          pivot_wider(names_from = all_of(c(over)),
                      # grab only numeric columns after by.var, over, and x
                      values_from = colnames(select_if(.,is.numeric)),
                      names_sep = "..")
        
        colnames(res.df) <- c("by.var", paste0(cor.col.name,"..",colnames(res.df)[-1]))
      }
      
    }
    
    
    return(res.df)
  })
  # ---------------- .
  
  
  return(res.l)
}

# FINAL CORRELATION COVARIANCE FUNCTION --------- 
pv.rrepest.corr.cov <- function(data, svy, x, by.var = NULL, over = NULL, test = F,
                             user_na=F, show_na=F, flag = F, fast = F, corr = T, pv = F,
                             ...) {
  # Goal: Dataframe with means and SE by Fay's BRR model
  # ------ INPUTS ------.
  # data : (dataframe) df to analyze
  # svy : List of possible projects to analyse PIAAC, PISA, TALISSCH and TALISTCH
  # by.var : (string vector) column in which we'll break down results
  # x : (string) variable from where to get frequencies
  # over : (vector string) columns over which to do analysis
  # test : (bool) If TRUE will calculate the difference between over variables
  # user_na : (Bool) TRUE → show nature of user defined missing values for by.var
  # show_na : (Bool) TRUE → include na in frequencies of x
  # flag : (Bool) TRUE → Show NaN when there is not enough cases (or schools)
  # fast : (bool) TRUE → Only do 6 replicated weights
  # corr : (bool) T → get correlation, F → get covariance
  # pv : (Bool) TRUE → We are in plausible values and must parallelize
  # ...
  # isced : (number) isced level to analyze
  
  # Get optional arguments
  extra.args <- list(...)
  
  # All variables in dataframe must be LOWERCASE
  over <- tolower(over)
  by.var <- tolower(by.var)
  x <- tolower(x)
  
  # # If there is an OVER variable remove NAs from OVER vars
  # if (length(over) > 0) {
  #   for (i in over) {
  #     df <- df %>% drop_na(i)
  #   }
  # }
  
  # Get weight names
  weight.names <- replicated_w_names(svy, ...)
  # If fast then less replicated weights
  if (fast) {
    weight.names <- weight.names[1:6]
  }
  
  # Loop over each variable
  brr.res <- pv.loop.corr.cov.on.weights(data = data,
                                      svy = svy,
                                      x = x, 
                                      by.var = by.var, 
                                      over = over,
                                      test = test,
                                      flag = flag,
                                      rep_weights = weight.names,
                                      corr = corr,
                                      pv = pv)
  
  # Reorder data to have b and se intertwined
  res <- pv.get.se.reorder(brr.res, svy = svy, pv = pv, ... = ...)
  
  
  
  return(res)
  
}


# rrepest.corr.cov(df.t, "TALISTCH", c("T3STAKE","T3TEAM"),"cntry",over = "tt3g01",fast = T,test=T)

coverage_column <- function(data, res, by = NULL, over = NULL, est, svy, user_na = FALSE, pct = NULL, ...){
  # Goal: Add a column of coverage_pct to the right of all se, not including those for tests, and show the coverage of the estimation
  # ------ INPUTS ------.
  # data : (dataframe) df from which to get replicated weights
  # res : (dataframe) df of results with b. and se. to average
  # by: (string vector) column in which we'll break down results
  # est : (est function) that takes arguments what = estimate, tgt = target, rgr = regressor
  # what : (string vector) accepts "mean", "mean pct","var","std", "quant", "iqr", "freq", "lm", "corr", "cov", "gen"
  # tgt : (string vector) variable from where to get frequencies or R script if selected "gen"
  # rgr : (string vector) independant variable for regression (1+)
  # over : (vector string) columns over which to do analysis
  # user_na : (Bool) TRUE: show nature of user defined missing values for by.var
  # pct: (numeric) NULL: shows column next to se. Numeric: Shows estimation (NA) [Coverage] if bellow the set coverage
  
  
  #Get na_to_zero from ...
  arguments <- list(...)
  
  # Same as initial Repest formating
  # Argument renaming to be adapted in v2 -----------------------------------
  what <- est$what
  tgt <- est$tgt
  rgr <- est$rgr
  
  # Plausible Values DIGITS--------------------------------------------------
  # If tgt or rgr has an @ then get digits of PVs
  if (any(grepl("@", c(tgt,rgr,by,over)))) {
    # Get all PV digits
    pv.digits <- digits.pvs(svy = svy, ...)
  } else {
    # if no @ the just iterate over 1 value
    pv.digits <- "@"
  }
  

  # Formatting depending on continuous or categorical tgt (assuming categorical target for all)
  df_covg <- format_data_repest(df = data,
                                   svy = svy,
                                   x = NULL, 
                                   # all arguments with an @ get converted into pvs
                                   by.over = get.pv.arguments(pv.digits, tolower(c(by,over,tgt))),
                                   user_na = user_na, 
                                   isced = is.there(arguments$isced),
                                   ...)
  
  #If no by provided
  if(is.null(by)) by <- "."

  # Get coverage vector
  # if (any(what %in% "freq")){
  #   tgt_coverage <- tgt
  # } else {
    # If lm, log or odds the coverage is from the regressors
    if(any(what %in% c("lm","log","odr"))){
      tgt_coverage <- rgr
    } else {
      tgt_coverage <- tgt
    }
  
    # If any element has @ in it replace it with a 1 in df_covg, PVs must have the same valid values = Coverage done on the 1st PV
    if(any(grepl(pattern = "@", x = tgt_coverage))){
      for(tgt_i in tgt_coverage){
        # Look for the 1st pv and replace the 1 with an @
        tgt2find <- sub(pattern = "@", replacement = "1",tgt_i)
        names(df_covg)[which(names(df_covg) == tgt2find)] <- tgt_i
      }
    }
    # Likewise on the over, do it on the 1st pv
    if(any(grepl(pattern = "@", x = over))){
      for(over_i in over){
        # Look for the 1st pv and replace the 1 with an @
        over2find <- sub(pattern = "@", replacement = "1",over_i)
        names(df_covg)[which(names(df_covg) == over2find)] <- over_i
      }
    }
  
    # If multiple  targets (correlation or covariance) we need all their coverage options
    res_cvg_l <- lapply(tgt_coverage, function(tgt_x){
      coverage_pct(df = df_covg, by = c(by, over) ,x = tgt_x, w = replicated_w_names(svy = svy, ...)[1]) %>% 
        separate(col = "by.group", sep = "\\|", into = c(by,over)) 
    } )
    
    # If there's an over
    if(!is.null(over)){
      res_cvg_l <- lapply(res_cvg_l, function(cvg_x){
        cvg_x %>% 
          unite(col = "over.cols", over, sep = "..") %>%
          # Make all column names have similar format to res
          pivot_wider(names_from = "over.cols", values_from = rev(names(.))[1], names_prefix = paste0(rev(names(.))[1],".."))
      })
    }
    
    # Give the same names as TGT
    names(res_cvg_l) <- tolower(tgt_coverage)
    
    # Join all dataframes from (possibly) multiple coverages into one
    res_cvg <- res_cvg_l %>% 
      reduce(inner_join, by = by)
    
    # If lm, log or odds add Intercept (and Rsqr) columns with 100 and targets for search in res_cvg
    if(any(what %in% c("lm","log","odr"))) {
      res_cvg[["cvge.intercept"]] <- 100
      tgt_coverage <- c(tgt_coverage, "intercept") # Must be lowercase for the search
      res_cvg_l[["intercept"]] <- res_cvg %>% select(!starts_with("cvge"),"cvge.intercept")
    }
    if(any(what %in% c("lm"))){
      res_cvg[["cvge.rsqr"]] <- 100
      tgt_coverage <- c(tgt_coverage, "rsqr") # Must be lowercase for the search
      res_cvg_l[["rsqr"]] <- res_cvg %>% select(!starts_with("cvge"),"cvge.rsqr")
    } 
  # } OLD: Frequencies not included in coverage
  
  # Check if names are similar for FREQUENCIES and add them to the res_cvg data
  if(any(what %in% c("freq"))){
    # Get the names of se after first dot
    names0res <- names(res)[grep("^se\\.",names(res))] %>% substring(4)
    # Get the names of cvge after first dot
    names0res_cvge <- names(res_cvg)[grep("^cvge\\.",names(res_cvg))] %>% substring(6)
    
    for(n_cvge_i in names0res_cvge){
      for(n_i in names0res){
        # Split both names and subtract 2nd component from n_i
        n_cvge_i_check <- paste(strsplit(n_cvge_i,"\\.{1,2}")[[1]],collapse = "..")
        n_i_check <- paste(strsplit(n_i,"\\.{1,2}")[[1]][-2],collapse = "..")
        if(n_cvge_i_check == n_i_check){
          # the name of 
          res_cvg[[paste0("cvge.",n_i)]] <- res_cvg[[paste0("cvge.",n_cvge_i)]]
        }
      }
    }
  }
  
  # We have the dataframe from where to start extracting coverage
  # !!!!!IMPORTANT!!!!!: Coverage must be done in Rrepest separately for either statistics, or frequencies, or corr/cov, or lm
  if(!any(what %in% c("gen"))){


    # -------------------------------------------------------------------------
    # Part 1: GETTING NAMES
    # Get all column names b, se, cvge in final Rrepest result --------
    
    # Only statistics and regressions will apply
    new_columns <- lapply(names(res),function(x){

        #Go over all names of the res df and if it finds se..
        if (startsWith(x,"se")){
          #Get the 3rd component in the column name (after the dots. and before the two dots..), the second if frequencies
          nth_component <- ifelse(any(what %in% c("freq")), 2 , 3)
          x_tgt <- strsplit(x,split = "\\.{1,2}")[[1]][nth_component]
          
          #Check which target variables are in this name component
          tgts_in_column <- tolower(tgt_coverage)[sapply(tolower(tgt_coverage), grepl,x_tgt, USE.NAMES = FALSE)]
          
          # For each of these names we need to grab cvge components
          sapply(tgts_in_column, function(tgt_i){
            
            # If has se and structure nn..(x-y)
            if (grepl(".*\\.{2}\\(.*\\-.*\\)", x)){
              # Seperate left and right side of test 
              names_x <- separate_test_name(x) %>% 
                sapply(function(y){
                  # and remove the 1st two components of the column name (b/se.what.)
                  ending <- paste(strsplit(y,"\\.{1,2}")[[1]][-c(1,2,3)],collapse = "..")
                  ending <- paste(c(tgt_i,ending),collapse = "..")
                  # return only the names that come from the coverage information database that are in the test part of the name
                  return(names(res_cvg_l[[tgt_i]])[endsWith(names(res_cvg_l[[tgt_i]]),ending)])
                },USE.NAMES = FALSE)
              
              
              # two coverage columns with information for the test
              new_x <- c(names_x)
            } else {
              # Grab the one from res_cvg, the one tha finishes the same way after the "what" in the name
              ending <- paste(strsplit(x,"\\.{1,2}")[[1]][-c(1,2,3)],collapse = "..")
              ending <- paste(c(tgt_i,ending),collapse = "..")
              new_x <- c(names(res_cvg_l[[tgt_i]])[endsWith(names(res_cvg_l[[tgt_i]]),ending)])
            }
            return(new_x)
          },USE.NAMES = FALSE) %>% c()
          
          
          
        } else {
          NULL
        }

      
    })
    names(new_columns) <- names(res)

    # Part 2: NEW COLUMNS in CVGE DB -----------------------------------------
    
    # Go over new_columns and check if they empty
    new_res_columns <- c()
    for (i in names(res)) {
      if (!is.null(new_columns[[i]])){
        
        
        # Removing 1st and 2nd component in all cases and 1st only for frequencies
        name_components2remove <- if(any(what %in% c("freq"))) 1 else c(1,2)
        # Get the ending of the column
        ending <- paste(strsplit(i,".",fixed = T)[[1]][-name_components2remove],collapse = ".")
        
        # if there are 2+ cvge columns to choose from
        if(length(new_columns[[i]]) > 1){
          # If the name of the cvge column to calculculate is already in the res_cvg don't calculate it
          if(! paste0("cvge.",ending) %in% names(res_cvg)){
            res_cvg <- res_cvg %>% 
              rowwise() %>%  
              mutate(Min = min(c_across(c(new_columns[[i]]))))
            
            #Rename Min column to cvg
            names(res_cvg)[names(res_cvg) == 'Min'] <- paste0("cvge.",ending)
          }
        }
        
        
        #Add the name and cvge column name to new_res_columns
        new_res_columns <- c(new_res_columns, c(i,paste0("cvge.",ending)))
        } else {
        # If no cvge column to add just get the name from res column
        new_res_columns <- c(new_res_columns,i)
        }
    }
    
     
    # Part 3: MERGE AND SELECT NAMES FROM PART 1 --------------------------------------
    res <- left_join(res,res_cvg, by = by) %>% 
      select(new_res_columns)

    # If a minimum coverage was provided
    if (!is.null(pct)){
  
      #go over cvge columns
      for(i in names(res)[startsWith(names(res),"cvge.")]){
        # Grab the ending of the names
        ending <- substr(i,6,nchar(i))
        for(j in names(res)[endsWith(names(res),ending) & !startsWith(names(res),"cvge.")]){
          # Assign a NaN to estimations under the minimum coverage
          res <- res %>% 
            mutate(!!j := ifelse(get(i) < pct, NaN, get(j)))
          
        }
      } 
      # Remove al cvge columns
      res <- res[!startsWith(names(res),"cvge.")]
    }
    
    
    
  }
  
  
  
    
  return(res)
}

#' Estimate list
#'
#' @description Obtains a list with the arguments for the est() function used within the Rrepest() function.
#' @param statistic (string vector) Statistics of interest that can include
#' mean ("mean"), variance ("var"), standard deviation ("std"), quantile ("quant"), inter-quantile range ("iqr"), 
#' frequency count ("freq"), correlation ("corr"), linear regression ("lm"), covariance ("cov") and 
#' any other statistics that are not pre-programmed into Rrepest but take a data frame and weights as parameters ("gen")
#' @param target (string vector) Variable(s) of interest of the estimation
#' @param regressor (string vector) Independent variable(s) to be included in a linear regression
#'
#' @return List containing the arguments for the est() function used within Rrepest() function
#' @export
#'
#' @examples
#' est(c("mean","quant",.5,"corr"),c("pv1math","pv1read","pv1scie"))
#' 

est <- function(statistic, target, regressor = NULL) {
  # statistic : (string vector) accepts "mean","var","std", "quant", "iqr", "freq", "lm", "corr", "cov", "gen"
  # target : (string vector) variable from where to get estimation
  # regressor : (string vector) independant variable for regression (1+)
  if ("gen" %in% statistic) {
    target <- deparse(substitute(target)) %>% 
      paste0(collapse = "")
    }
  return(list(what = statistic, tgt = target, rgr = regressor))
}

#' Weighted bivariate covariance
#' 
#' @description Computes the weighted covariance coefficient of two numeric vectors.
#' @param x (numeric vector) Variable of interest x for computing the covariance
#' @param y (numeric vector) Variable of interest y for computing the covariance
#' @param w (numeric vector) Vector with the weights
#' @param na.rm (bool) if TRUE: Excludes missing values before computing the covariance
#'
#' @return Scalar containing the covariance
#' @export
#'
#' @examples
#' data(df_talis18) 
#' 
#' weighted.cov(x = df_talis18$tt3g06b2, y = df_talis18$tt3g06c2, w = df_talis18$tchwgt)
weighted.cov <- function(x, y, w, na.rm = TRUE) {
  # Goal: Get weighted covariance
  # ------ INPUTS ------.
  # x: (vector) variable from where to get correlation
  # y: (vector) variable from where to get correlation
  # w : (vector) vector of weights
  
  # Use non NAs in both x and y
  use <- !is.na(x) & !is.na(y)
  x <- x[use]
  y <- y[use]
  w <- w[use]
  
  # Covariance
  w.mean <- weighted.mean(x, w, na.rm = na.rm)
  cov.w <- sum(w * (x - w.mean) * (y - w.mean), na.rm = na.rm)/sum(w, na.rm = na.rm)
  return(cov.w)
}

#' Format categorical variables as factor for Rrepest
#'
#' @param df (data frame) Data to analyse.
#' @param categ.vars (string vector) Categorical variables for analysis.
#' @param show_na (bool) Keeps NAs as categories to get frequency from.
#'
#' @return Data frame with categorical variables as factors for frequencies or over variables.
#' @export
#'
#' @examples
#' format_data_categ_vars(df = mtcars, categ.vars = "cyl") 
format_data_categ_vars <- function(df, categ.vars, show_na = F) {
  # Goal: Converts to numeric all variables for a continuous analysis (means, regression, etc.)
  # ------ INPUTS ------.
  # data : (dataframe) df to analyze
  # cont.vars : (string vector) continuous variables for analysis
  
  if (show_na) {
    df.res <- df %>% 
      mutate_at(categ.vars,to_factor,drop_unused_labels = TRUE ,user_na_to_na = TRUE) 
  }else{
    df.res <- df %>% 
      mutate_at(categ.vars,to_factor,drop_unused_labels = TRUE ,user_na_to_na = TRUE) %>% 
      drop_na(all_of(categ.vars))
  }
  
  return(df.res)
}

#Rrepest Isolated for one database (group)

rrepest_base <- function(data, svy, est, by = NULL, over = NULL,
                    test = FALSE, user_na= FALSE, show_na= FALSE, flag = FALSE, 
                    fast = FALSE, cores = NULL,...) {
  # Goal: Dataframe with statistics for TALIS
  # ------ INPUTS ------.
  ######### WHO ######### 
  # data : (dataframe) df to analyze
  # svy : List of possible projects to analyse TALISSCH and TALISTCH
  ######### WHAT ######### 
  # est : (est function) that takes arguments what = estimate, tgt = target, rgr = regressor
  # what : (string vector) accepts "mean","var","std", "quant", "iqr", "freq", "lm", "log", "corr", "cov"
  # tgt : (string vector) variable from where to get frequencies
  # rgr : (string vector) independant variable for regression (1+)
  ######### WHERE ######### 
  # by.var : (string vector) column in which we'll break down results
  # over : (vector string) columns over which to do analysis
  ######### HOW ######### 
  # test : (bool) If TRUE will calculate the difference between over variables
  # user_na : (Bool) TRUE → show nature of user defined missing values for by.var
  # show_na : (Bool) TRUE → include na in frequencies of x 
  # flag : (Bool) TRUE → Show NaN when there is not enough cases (or schools)
  # fast : (Bool) TRUE → Only do 6 replicated weights
  # ...
  # isced : (number) isced level to analyze
  
  
  # Argument renaming to be adapted in v2 -----------------------------------
  
  
  by.var <- by
  what <- est$what
  tgt <- est$tgt
  rgr <- est$rgr
  
  # Groups ------------------------------------------------------------------
  # if (!is.null(group)) {
  #   data <- do_group(data = data, grp.l = group)
  # }
  
  
  
  
  # Plausible Values DIGITS--------------------------------------------------
  # If tgt or rgr has an @ then get digits of PVs
  if (any(grepl("@", c(tgt,rgr,by.var,over)))) {
    # Get all PV digits
    pv.digits <- digits.pvs(svy = svy, ...)
    pv <- T #To know when to parallelize
  } else {
    # if no @ the just iterate over 1 value
    pv.digits <- "@"
    pv <- F #To know when to parallelize
  }
  
  # FORMATING ---------------------------------------------------------------.
  # Get optional arguments
  extra.args <- list(...)
  
  # If no by.var do it in all df
  if (is.null(by.var)) {
    data["tot.df"] <- "Total"
    by.var <- "tot.df"
  }
  
  # TEST: Remove NAs from last variable in over, to avoid yy..(xx-NA) --------.
  if (test){
    # If @ in over variable remove all NAs from all pv variables
    if(grepl(pattern = "@", x = over[length(over)])){
      last_pv_over <- sapply(pv.digits, function(pv_i){
        sub(pattern = "@", replacement = pv_i, x = over[length(over)])
      })
      # Remove all NA values from that over
      data <- data %>% 
        drop_na(all_of(last_pv_over))
    } else {
      data <- data %>% 
        drop_na(all_of(over[length(over)])) 
    }
  }
  
  
  #GENERAL option: If what is not "gen" convert to lowercase (might be functions in upper case)
  if (!("gen" %in% what))  {
    # All variables in dataframe must be LOWERCASE
    over <- tolower(over)
    by.var <- tolower(by.var)
    tgt <- tolower(tgt)
    rgr <- tolower(rgr)
    }
  
  # Formatting depending on continuous or categorical tgt
  if ("freq" %in% what) {
    # Data formatting
    df <- format_data_repest(df = data,
                             svy = svy,
                             x = NULL, 
                             # all arguments with an @ get converted into pvs
                             by.over = get.pv.arguments(pv.digits, c(by.var,over,tgt)),
                             user_na = user_na, 
                             isced = is.there(extra.args$isced),
                             ...)
    
    # # If there is an OVER variable remove NAs from OVER vars
    # if (length(get.pv.arguments(pv.digits,over)) > 0) {
    #   if (!any(show_na,user_na)) {
    #     for (i in get.pv.arguments(pv.digits,over)) {
    #       df <- df %>% drop_na(i)
    #     }
    #   }
    # }
    
    
    # # X is categorical in a frequency, must be formatted accordingly
    # df <- format_data_categ_vars(df,
    #                              get.pv.arguments(pv.digits, tgt),
    #                              show_na)
  } 
  else if (!("gen" %in% what)) {
    # Data formatting
    df <- format_data_repest(df = data,
                             svy = svy,
                             x = NULL, 
                             # all arguments with an @ get converted into pvs
                             by.over = get.pv.arguments(pv.digits, c(by.var,over)),
                             user_na = user_na, 
                             isced = is.there(extra.args$isced),
                             ...)
    
    # If there is an OVER variable remove NAs from OVER vars
    if (length(get.pv.arguments(pv.digits,over)) > 0) {
      for (i in get.pv.arguments(pv.digits,over)) {
        df %>% drop_na(i)
      }
    }
    
    # Format continuous vars
    df <- format_data_cont_vars(df, cont.vars = get.pv.arguments(pv.digits, c(tgt,rgr)))
  }
  
  
  # CLUSTER -----------------------------------------------------------------.
  # If pvs, set up cluster
  if (pv) {
    #Get repweights
    rep_weights <- replicated_w_names(svy = svy, ...)
    
    # Select necessary data and convert to to datable
    # GENERAL option: Assumes only essential data is preselected by user
    if (!("gen" %in% what)) {
      # Quantile table may include a what of the form "pv@math__pv@read" which needs to be dived into "pv@math" & "pv@read"
      data.par <- as.data.table(
        df[,get.pv.arguments(pv.digits,
                             c(unlist(strsplit(tgt,"__")), # Will assume "__" is a way to split the target tgt
                               rgr,rep_weights,by.var,over) 
        )])}
    else {data.par <- as.data.table(data)}
    
    # Set up cluster working nodes and what they have to know
    if(is.null(cores)){
      message("Rrepest will use all availables CPU cores. If not intended, set the argument 'cores' to the number of cores to use.")
      # If NULL, action the max of cores minus 1
      cl <- makeCluster(max(detectCores()-1,1))
    } else {
      # If no null, set the minimum between the set number and max-1
      cl <- makeCluster(min(detectCores()-1,cores))
    }
    
    clusterExport(cl, "data.par",envir = environment())
    doParallel::registerDoParallel(cl)
  }
   
  # PLAUSIBLE VALUES LOOP -------------------------------------------------
  pv.l <- lapply(pv.digits, function(pv.d.i) {
    if (pv) message(as.character(pv.d.i)) #counter of pvs
    # PARALLEL UNIVERSES ---------------------------------------------------.
    # replace @ with that digit in tgt and rgr
    tgt <- gsub("@", as.character(pv.d.i), tgt)
    rgr <- gsub("@", as.character(pv.d.i), rgr)
    # Also for over variables in the over and by.vars if categories created from pvs
    by.var <- gsub("@", as.character(pv.d.i), by.var)
    over <- gsub("@", as.character(pv.d.i), over)
    
    # REPLICATED WEIGHTS ---------------------------------------------------.
    res.l <- lapply(tgt, function(tgt.i){
      res.tgt.i <- list()
      #-------------- STATISTICS --------------.
      # If there is something else than freq
      if(any(c("mean","means","meanpct","meanspct","var","std", "sd", "quant", "iqr") %in% what )){
        # remove "log","lm" and "freq", "quintiletable"
        what.statistic <- what[-c(which(what == "quintiletable"), which(what == "quintiletable") + 1)]
        what.statistic <- what[! what %in% c("lm","log","freq","corr","cov","quintiletable")]
        
        # if flag show.n as "flag"
        if (flag) {show.n = "flag"} else {show.n = "n"}
        
        # Do statistics
        res.stats <- 
          pv.rrepest.statistics(data = df, svy = svy, statistic = what.statistic,
                                x = tgt.i, by.var = by.var,over = over, test = test,
                                flag = flag, user_na = user_na, fast = fast, pv = pv,
                                ...)
        
        # Rename to by.var first column to merge all results
        colnames(res.stats)[1] <- "by.var"
        # Append result of statistics to list of results
        res.tgt.i <- append(res.tgt.i, list(res.stats))
        
      }
      #---------------------------------------.
      
      #-------------- FREQUENCIES --------------.
      if ("freq" %in% what) {
        # Do frequencies
        res.freq <- 
          pv.rrepest.frequencies(data = df, svy = svy, x = tgt.i, by.var = by.var,
                                 over = over, test = test, user_na = user_na,
                                 show_na = show_na, flag = flag, fast = fast, pv = pv, 
                                 pvdigits = pv.digits, # Insert pv.digits to erase NAs from over variables 
                                 #pcluster = ifelse(pv,cl,c()), # Cluster of workers in case pv exists
                                 ...)
        
        # Append result of frequencies to list of results
        res.tgt.i <- append(res.tgt.i, list(res.freq))
        
      }
      
      #---------------------------------------.
      
      #-------------- LINEAR REGRESSION --------------.
      if ("lm" %in% what) {
        # Do regression
        res.lm <- 
          pv.rrepest.lm(data = df, svy = svy, x = rgr, y = tgt.i, by.var = by.var,
                        over = over, test = test, user_na = user_na, flag = flag,
                        fast = fast, pv = pv, ...)
        
        # Append result of regression to list of results
        res.tgt.i <- append(res.tgt.i, list(res.lm))
      }
      #---------------------------------------.
      
      #-------------- LOGISTIC REGRESSION --------------.
      if ("log" %in% what) {
        # Do regression
        res.logreg <- 
          pv.rrepest.logreg(data = df, svy = svy, x = rgr, y = tgt.i, by.var = by.var,
                        over = over, test = test, user_na = user_na, flag = flag,
                        fast = fast, pv = pv, ...)
        
        # Append result of regression to list of results
        res.tgt.i <- append(res.tgt.i, list(res.logreg))
      }
      #---------------------------------------.
      #-------------- ODDS RATIO DELTA METHOD Part 1: Names --------------.
      if ("odr" %in% what) {
        # Do regression
        res.odr <- 
          pv.rrepest.odr(data = df, svy = svy, x = rgr, y = tgt.i, by.var = by.var,
                            over = over, test = test, user_na = user_na, flag = flag,
                            fast = fast, pv = pv, ...)
        
        # Append result of regression to list of results
        res.tgt.i <- append(res.tgt.i, list(res.odr))
      }
      #---------------------------------------.
      #-------------- QUINTILE TABLE --------------.
      if ("quantiletable" %in% what) {
        # Get number of quantiles and if decimal the difference from last to first
        quant_n <- what[grep(pattern = "quantiletable",x = what)+1] %>% as.numeric()
        
        # Create variable for flags and coverage "var1__var2"
        if(grepl("__",tgt)){
          # If found separete element into "y for order" and "y for mean""
          y_vars <- strsplit(tgt,"__")[[1]]
          y_4order <- y_vars[1]
          y_4mean <- y_vars[2]
          # Get the name of y for columns
          y_name <- paste0(y_4order,"__",y_4mean)
          tgt.i <- y_name
          # y_name will be multiplication of both to get the proper ammount of NAs
          df[[y_name]] <- df[[y_4order]] * df[[y_4mean]]
        }
        
        # Do quantile table
        res.qtable <- 
          pv.rrepest.quantiletable(data = df, svy = svy, quant_n = quant_n, y = tgt.i, by.var = by.var,
                         over = over, test = test, user_na = user_na, flag = flag,
                         fast = fast, pv = pv, ...)
        
        # Append result of regression to list of results
        res.tgt.i <- append(res.tgt.i, list(res.qtable))
      }
      #---------------------------------------.
      
      #-------------- GENERAL OPTION --------------.
      if ("gen" %in% what) {
        # Do general calculation
        res.gen <- # Here the argument for data is data and not df as no data preparation is done in "gen"
          pv.rrepest.gen(data = data, svy = svy, y = tgt.i, by.var = by.var,
                        over = over, test = test, user_na = user_na, flag = flag,
                        fast = fast, pv = pv, ...)
        
        # Append result of regression to list of results
        res.tgt.i <- append(res.tgt.i, list(res.gen))
        
      }
      
      #---------------------------------------.
      
      # Merge all in list of results if there is a result
      if (length(res.tgt.i) > 0) {
        res.tgt.i <- reduce(res.tgt.i, full_join, by = "by.var")
      }
      
      return(res.tgt.i)
    })
    
    # If there is nothing on res.l initialize it
    res.l <- Filter(length, res.l)
    
    #-------------- CORRELATION AND COVARIANCE --------------.
    # Since corr will use all the variables in tgt we must do a separate loop
    if (any(c("corr","cov","cor") %in% what)) {
      # Leave what with only cor,corr,cov
      what.cov.cor <- what[what %in% c("corr","cov","cor")]
      # Loop over all left
      res.l.corr <- lapply(what.cov.cor, function(what.i){
        corr <- !(what.i == "cov")
        res.corr <- pv.rrepest.corr.cov(data = df, svy = svy, x = tgt, by.var = by.var,
                                        over = over, test = test, user_na=user_na, pv = pv,
                                        show_na=show_na, flag = F, fast = fast, corr = corr,
                                        ...) #Can't accept flags since many variables
        
        return(res.corr)
      })
      res.l <- append(res.l,res.l.corr)
    }
    #---------------------------------------.
    
    # return all results merged into one dataframe
    return(reduce(res.l, full_join, by = "by.var"))
    
  })
  
  # Stop the cluster if pvs
  if(pv) stopCluster(cl)
  
  ###if PIAAC, modify pv.l to account for by.var specific variance factors:
  
  if (svy=="PIAAC") {  
    pv.l<- add_PIAAC_variance_factors(pv.l=pv.l, pv=pv, df=df,by.var=by.var)
  }
  
  
  # If there are no PVs return first element of the list
  if (!pv) {
    result <- pv.l[[1]] %>% as_tibble()
  } else {
    
    # Check which inputs have an @
    pv.inputs <- c(tgt,rgr,by.var,over)[grepl("@", c(tgt,rgr,by.var,over))]
    
    
    #modify the names and make sure that the pv appears, along with the name of the pv vars in case there is a over variable.
    pv.inputs.collapsed<-  paste0(pv.inputs,collapse="") 
    
    for(pv.i in digits.pvs(svy = svy,...)) {
      pv.i.inputs_collapsed<-  pv.inputs.collapsed %>%  gsub(x = .,pattern="@",replacement=pv.i)
      names(pv.l[[pv.i]]) <- c(names(pv.l[[pv.i]])[1:length(by.var)],
                               paste0(
                                 names(pv.l[[pv.i]])[-c(1:length(by.var))],
                                 pv.i.inputs_collapsed))
    }
    
    
    # If there are PVs Calculate B and Imputation and Sampling Variance for SE
    result <- b.se.pv(res.l = pv.l, pv.inputs = pv.inputs, statistic = what) %>% 
      as_tibble() %>%   setNames(nm = str_remove(names(.),paste0(pv.inputs.collapsed,"$")))
    
  }
  
  #-------------- LOGISTIC REGRESSION DELTA METHOD Part 2: Estimates --------------.
  if("odr" %in% what){
    # Grab b. columns and strip the of the b.
    result_names <- names(result)
    estimate_names <- substring(result_names[grep("^b\\.",result_names)], first = 3)
    
    # Loop over estimate names and do for β: exp(b.odr), for SE: exp(b.odr) * se.odr
    for (est_i in estimate_names){
      result[[paste0("se.",est_i)]] <- exp(result[[paste0("b.",est_i)]]) * result[[paste0("se.",est_i)]]
      result[[paste0("b.",est_i)]] <- exp(result[[paste0("b.",est_i)]])
    }
  }
  #----------------------------.
  
  return(result)
}

#' Rrepest table of results for PISA 2018 showing age and gender
#'
#' A table of results from Rrepest of the mean age of students broken down by gender for PISA 2018.
#'
#' @format ## `rrepest_pisa_age_gender`
#' A data frame in tibble format with 3 rows and 7 columns:
#' \describe{
#'   \item{cnt}{Country ISO 3166-1 alpha-3 codes.}
#'   \item{b.mean.age..Female}{Mean age for female students.}
#'   \item{se.mean.age..Female}{Standar error of the mean age for female students.}
#'   \item{b.mean.age..Male}{Mean age for male students.}
#'   \item{se.mean.age..Male}{Standar error of the mean age for male students.}
#'   \item{b.mean.age..(Female-Male)}{Difference of the mean age from female to male students.}
#'   \item{se.mean.age..(Female-Male)}{Standard error of the difference of the mean age from female to male students.}
#' }
#' @source <https://www.oecd.org/en/data/datasets/pisa-2018-database.html#data>
"rrepest_pisa_age_gender"

#' Rrepest table of results for PISA 2018 showing the age and completed schooling level of students' mothers
#'
#' A table of results from Rrepest of the mean age of students broken down by 
#' completed schooling level of students' mothers for PISA 2018.
#'
#' @format ## `rrepest_pisa_age_isced`
#' A data frame in tibble format with 3 rows and 7 columns:
#' \describe{
#'   \item{cnt}{Country ISO 3166-1 alpha-3 codes.}
#'   \item{b.mean.age..ISCED level 1}{Mean age of students whose mothers completed ISCED 1.}
#'   \item{se.mean.age..ISCED level 1}{Standard error of the mean age of students whose mothers completed ISCED 1.}
#'   \item{b.mean.age..ISCED level 2}{Mean age of students whose mothers completed ISCED 2.}
#'   \item{se.mean.age..ISCED level 2}{Standard error of the mean age of students whose mothers completed ISCED 2.}
#'   \item{b.mean.age..ISCED level 3A}{Mean age of students whose mothers completed ISCED 3A.}
#'   \item{se.mean.age..ISCED level 3A}{Standard error of the mean age of students whose mothers completed ISCED 3A.}
#'   \item{b.mean.age..ISCED level 3B, 3C}{Mean age of students whose mothers completed ISCED 3B/3C.}
#'   \item{se.mean.age..ISCED level 3B, 3C}{Standard error of the mean age of students whose mothers completed ISCED 3B/3C.}
#'   \item{b.mean.age..She did not complete  ISCED level 1}{Mean age of students whose mothers did not complete ISCED 1.}
#'   \item{se.mean.age..She did not complete  ISCED level 1}{Standard error of the mean age of students whose mothers did not complete ISCED 1.}
#'   \item{b.mean.age..(ISCED level 1-She did not complete  ISCED level 1)}{Mean age difference between ISCED 1 completed vs. non-ISCED 1 completed mothers.}
#'   \item{se.mean.age..(ISCED level 1-She did not complete  ISCED level 1)}{Standard error of the mean age difference between ISCED 1 completed vs. non-ISCED 1 completed mothers.}
#'   ...
#' }
#' @source <https://www.oecd.org/en/data/datasets/pisa-2018-database.html#data>
"rrepest_pisa_age_isced"

#' Teaching and Learning International Survey (TALIS) 2018 noisy data subset
#'
#' A subset of noisy data from the Teaching and Learning International Survey (TALIS) 2018 cycle
#' for Mexico, Italy, and France.
#' @format ## `df_talis18`
#' A data frame in tibble format with 496 rows and 548 columns from which only some relevant variables are listed below. 
#' For a detailed description of all variables go to https://www.oecd.org/en/data/datasets/talis-2018-database.html#data
#' \describe{
#'   \item{idlang}{Country language}
#'   \item{cntry}{Country ISO 3166-1 alpha-3 codes.}
#'   \item{tt3g01}{Gender}
#'   \item{tt3g02}{Age}
#'   \item{tt3g03}{Highest level of formal education completed}
#'   ...
#' }
#' @source <https://www.oecd.org/en/data/datasets/talis-2018-database.html#data>
"df_talis18"

#' Program for International Student Assessment (PISA) 2018 noisy data subset
#'
#' A subset of noisy data from the Program for International Student Assessment (PISA) 2018 cycle
#' for Mexico, Italy, and France.
#' @format ## `df_pisa18`
#' A data frame in tibble format with 1269 rows and 1106 columns from which only some relevant variables are listed below. 
#' For a detailed description of all variables go to https://www.oecd.org/en/data/datasets/pisa-2018-database.html#data
#' \describe{
#'   \item{idlang}{Country language}
#'   \item{CNT}{Country ISO 3166-1 alpha-3 codes.}
#'   \item{st001d01t}{Student International Grade}
#'   \item{st004d01t}{Gender}
#'   ...
#' }
#' @source <https://www.oecd.org/en/data/datasets/pisa-2018-database.html#data>
"df_pisa18"

#' Rrepest table of results for TALIS 2018 showing a frequency for other areas of professional development
#' 
#' A table of results from Rrepest for professional development broken down by 
#' completed schooling level of students' mothers for the Teaching and 
#' Learning International Survey (TALIS) 2018 cycle
#' @format ## `talis18_tt3g23o_freq`
#' A data frame in tibble format with 3 rows and 7 columns: 
#' \describe{
#'   \item{cntry}{Country ISO 3166-1 alpha-3 codes.}
#'   \item{b.tt3g23o.No}{Percentage of teachers with No other areas of professional development}
#'   \item{se.tt3g23o.No}{Standard error of th percentage of teachers with No other areas of professional development}
#'   \item{cvge.tt3g23o.No}{Coverage of the percentage of teachers with No other areas of professional development}
#'   \item{b.tt3g23o.Yes}{Percentage of teachers with other areas of professional development}
#'   \item{se.tt3g23o.Yes}{Standard error of th percentage of teachers with other areas of professional development}
#'   \item{cvge.tt3g23o.Yes}{Coverage of the percentage of teachers with other areas of professional development}
#' }
#' @source <https://www.oecd.org/en/data/datasets/talis-2018-database.html#data>
"talis18_tt3g23o_freq"

#' Weighted bivariate correlation
#' 
#' @description Computes the weighted Pearson correlation coefficient of two numeric vectors.
#' @param x (numeric vector) Variable of interest x for computing the correlation
#' @param y (numeric vector) Variable of interest y for computing the correlation
#' @param w (numeric vector) Vector with the weights
#' @param na.rm (bool) if TRUE: Excludes missing values before computing the correlation
#'
#' @return Scalar containing the Pearson correlation coefficient
#' @export
#'
#' @examples
#' data(df_talis18) 
#' 
#' weighted.corr(x = df_talis18$tt3g06b2, y = df_talis18$tt3g06c2, w = df_talis18$tchwgt)
weighted.corr <- function(x, y, w, na.rm = TRUE) {
  # Goal: Get weighted correlation
  # ------ INPUTS ------.
  # x: (vector) variable from where to get correlation
  # y: (vector) variable from where to get correlation
  # w : (vector) vector of weights
  
  # Use non NAs in both x and y
  use <- !is.na(x) & !is.na(y)
  x <- x[use]
  y <- y[use]
  w <- w[use]
  
  # Calculate variance and covariance of x and y
  w.mean.x <- weighted.mean(x, w, na.rm = na.rm)
  w.mean.y <- weighted.mean(y, w, na.rm = na.rm)
  cov.w <- sum(w*(x-w.mean.x)*(y-w.mean.y), na.rm = na.rm)
  var.w.x <- sum(w*(x-w.mean.x)^2, na.rm = na.rm)
  var.w.y <- sum(w*(y-w.mean.y)^2, na.rm = na.rm)
  
  return(cov.w / sqrt(var.w.x * var.w.y))
}

utils::globalVariables(c("idpop", "small.sum'", "big.sum",".",
                         "idschool","school.n","cntschid","schoolid",
                         "schid","small.sum","w.i","data.par","by.group","df.qqq"))

#' Check if a number is prime
#' @description To check if a number n is prime, you only need to check for factors up to the square root of n. 
#' This is because if n has a factor greater than its square root, it must also have a smaller factor 
#' (since a factor is a number that divides n without leaving a remainder). 
#' This method significantly reduces the number of checks needed to determine if a number is prime.
#' 
#' @param n (numeric) Number to verify if it prime
#'
#' @return (bool) TRUE if the number is prime, FALSE if the number is not prime
#' @export
#'
#' @examples
#' is_prime(0)
#' is_prime(1)
#' is_prime(7)
#' is_prime(10)
#' is_prime(100011869)
is_prime <- function(n){
  # We know 1 or negatives are not a prime number
  if (n <= 1) return(FALSE)
  
  # If not integer
  if (n %% 1 !=0) return(FALSE)
  
  i <- 2
  # This will loop from 2 to int(sqrt(x))
  while (i*i <= n) {
    # Check if i divides x without leaving a remainder
    if (n %% i == 0){
      # This means that n has a factor in between 2 and sqrt(n)
      # So it is not a prime number
      return(FALSE)
    }
    i <-  i + 1
  }
  # If we did not find any factor in the above loop,
  # then n is a prime number
  return(TRUE)
}

#' Weighted inter-quantile range
#'
#' @description Computes the weighted inter-quantile range of a numeric vector.
#' @param x (numeric vector) Variable of interest for which to compute the inter-quantile range
#' @param w (numeric vector) Vector with the weights
#' @param rang (numeric vector) Two numbers between 0 and 1 indicating the desired inter-quantile range
#'
#' @return Scalar containing the inter-quantile range
#' @export
#'
#' @examples
#' weighted.iqr(x = mtcars$mpg, w = mtcars$wt,  rang = c(.5,.9))
#'   
weighted.iqr <- function(x, w = rep(1, length(x)), rang = c(0.25,0.75)) {
  # Goal: Get weighted interquantile range
  # ------ INPUTS ------.
  # x: (vector) variable from where to get quantiles
  # w : (vector) vector of weights
  # rang : (vector) two numbers indicating the range
  
  res <- weighted.quant(x, q=max(rang), w) - 
    weighted.quant(x, q=min(rang), w)
  
  return(res)
}

#' Mode
#' @description Calculate the arithmetic mode of a vector. If multiple elements 
#' have the same frequency, all of them will be displayed.
#' @param x (numeric vector) vector from which we'll obtain the mode
#' @param w (numeric vector) vector of weights. If not provided, it defaults to non-weighted mode.
#'
#' @return (numeric vector) one or multiple elements that will be the arithmetic mode
#' @export
#'
#' @examples
#' weighted.mode(c(1,2,3,4,5,4,3,4,5,3))
#' weighted.mode(c(NA,1,3,NA))
weighted.mode <- function(x, w = rep(1,length(x))) {
  df <- data.frame(x, w)
  res <- aggregate(w ~ x, df, sum)
  res$x[res$w==max(res$w)]
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/is_prime.R
\name{is_prime}
\alias{is_prime}
\title{Check if a number is prime}
\usage{
is_prime(n)
}
\arguments{
\item{n}{(numeric) Number to verify if it prime}
}
\value{
(bool) TRUE if the number is prime, FALSE if the number is not prime
}
\description{
To check if a number n is prime, you only need to check for factors up to the square root of n.
This is because if n has a factor greater than its square root, it must also have a smaller factor
(since a factor is a number that divides n without leaving a remainder).
This method significantly reduces the number of checks needed to determine if a number is prime.
}
\examples{
is_prime(0)
is_prime(1)
is_prime(7)
is_prime(10)
is_prime(100011869)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Mode.R
\name{weighted.mode}
\alias{weighted.mode}
\title{Mode}
\usage{
weighted.mode(x, w = rep(1, length(x)))
}
\arguments{
\item{x}{(numeric vector) vector from which we'll obtain the mode}

\item{w}{(numeric vector) vector of weights. If not provided, it defaults to non-weighted mode.}
}
\value{
(numeric vector) one or multiple elements that will be the arithmetic mode
}
\description{
Calculate the arithmetic mode of a vector. If multiple elements
have the same frequency, all of them will be displayed.
}
\examples{
weighted.mode(c(1,2,3,4,5,4,3,4,5,3))
weighted.mode(c(NA,1,3,NA))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/weighted_iqr.R
\name{weighted.iqr}
\alias{weighted.iqr}
\title{Weighted inter-quantile range}
\usage{
weighted.iqr(x, w = rep(1, length(x)), rang = c(0.25, 0.75))
}
\arguments{
\item{x}{(numeric vector) Variable of interest for which to compute the inter-quantile range}

\item{w}{(numeric vector) Vector with the weights}

\item{rang}{(numeric vector) Two numbers between 0 and 1 indicating the desired inter-quantile range}
}
\value{
Scalar containing the inter-quantile range
}
\description{
Computes the weighted inter-quantile range of a numeric vector.
}
\examples{
weighted.iqr(x = mtcars$mpg, w = mtcars$wt,  rang = c(.5,.9))
  
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/weighted_quantile.R
\name{weighted.quant}
\alias{weighted.quant}
\title{Weighted quantile}
\usage{
weighted.quant(x, w = rep(1, length(x)), q = 0.5)
}
\arguments{
\item{x}{(numeric vector) Variable of interest for which to compute the quantile}

\item{w}{(numeric vector) Vector with the weights}

\item{q}{(numeric vector) A number between 0 and less than 1 indicating the desired quantile}
}
\value{
Scalar containing the quantile
}
\description{
Computes weighted quantiles of a numeric vector.
}
\examples{
weighted.quant(x = mtcars$mpg, w = mtcars$wt,  q = seq(.1,.9,.1))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/indep_diff.R
\name{paired_indep_diff}
\alias{paired_indep_diff}
\title{Paired independent differences}
\usage{
paired_indep_diff(df, col1, col2)
}
\arguments{
\item{df}{(dataframe) Dataframe that contains the columns with b. and se. on their name to get the difference from}

\item{col1}{(numeric vector) Column name, not including "b.". Must have a column in dataframe also including "se.".}

\item{col2}{(numeric vector) Column name, not including "b.". Must have a column in dataframe also including "se.".}
}
\value{
Dataframe with extra columns containing the difference of the columns along with their standard error.
}
\description{
Get differences and standard errors from two columns starting with b. and se. assuming independence.
The second column will be subtracted from the first.
}
\examples{
paired_indep_diff(rrepest_pisa_age_gender,"mean.age..Male","mean.age..Female")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pv_rrepest_estimate_argument.R
\name{est}
\alias{est}
\title{Estimate list}
\usage{
est(statistic, target, regressor = NULL)
}
\arguments{
\item{statistic}{(string vector) Statistics of interest that can include
mean ("mean"), variance ("var"), standard deviation ("std"), quantile ("quant"), inter-quantile range ("iqr"),
frequency count ("freq"), correlation ("corr"), linear regression ("lm"), covariance ("cov") and
any other statistics that are not pre-programmed into Rrepest but take a data frame and weights as parameters ("gen")}

\item{target}{(string vector) Variable(s) of interest of the estimation}

\item{regressor}{(string vector) Independent variable(s) to be included in a linear regression}
}
\value{
List containing the arguments for the est() function used within Rrepest() function
}
\description{
Obtains a list with the arguments for the est() function used within the Rrepest() function.
}
\examples{
est(c("mean","quant",.5,"corr"),c("pv1math","pv1read","pv1scie"))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/repest_auxiliaries.R
\name{format_data_repest}
\alias{format_data_repest}
\title{Formatting target, by, and over variables for Rrepest.}
\usage{
format_data_repest(df, svy, x, by.over, user_na = F, ...)
}
\arguments{
\item{df}{(data frame) Data to analyze.}

\item{svy}{(string) Possible projects to analyse: PIAAC, PISA, TALISSCH, TALISTCH, etc.}

\item{x}{(string vector) Target variables.}

\item{by.over}{(string vector) Variables to break analysis by.}

\item{user_na}{(bool) TRUE → show nature of user defined missing values}

\item{...}{Optional arguments such as custom weights (cm.weights)}
}
\value{
Data frame with variables in numeric format for analysis.
}
\description{
Formatting target, by, and over variables for Rrepest.
}
\examples{
df1 <- format_data_repest(df_pisa18, "PISA", "pv1math", "cnt")
df2 <- format_data_repest(df_pisa18, "PISA", "pv1math", c("cnt","st004d01t"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R, R/oecd_data.R
\docType{data}
\name{df_pisa18}
\alias{df_pisa18}
\title{Program for International Student Assessment (PISA) 2018 noisy data subset}
\format{
\subsection{\code{df_pisa18}}{

A data frame in tibble format with 1269 rows and 1106 columns from which only some relevant variables are listed below.
For a detailed description of all variables go to https://www.oecd.org/en/data/datasets/pisa-2018-database.html#data
\describe{
\item{idlang}{Country language}
\item{CNT}{Country ISO 3166-1 alpha-3 codes.}
\item{st001d01t}{Student International Grade}
\item{st004d01t}{Gender}
...
}
}

A data frame with 1269 rows/observations and 1120 columns/variables.
}
\source{
\url{https://www.oecd.org/en/data/datasets/pisa-2018-database.html#data}
}
\usage{
df_pisa18

data(df_pisa18)
}
\description{
A subset of noisy data from the Program for International Student Assessment (PISA) 2018 cycle
for Mexico, Italy, and France.

This dataset is a subset of the PISA 2018 database produced by the OECD for the countries of France, Italy, and Mexico.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/format.data.categ.vars.R
\name{format_data_categ_vars}
\alias{format_data_categ_vars}
\title{Format categorical variables as factor for Rrepest}
\usage{
format_data_categ_vars(df, categ.vars, show_na = F)
}
\arguments{
\item{df}{(data frame) Data to analyse.}

\item{categ.vars}{(string vector) Categorical variables for analysis.}

\item{show_na}{(bool) Keeps NAs as categories to get frequency from.}
}
\value{
Data frame with categorical variables as factors for frequencies or over variables.
}
\description{
Format categorical variables as factor for Rrepest
}
\examples{
format_data_categ_vars(df = mtcars, categ.vars = "cyl") 
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/coverage_daggers.R
\name{coverage_daggers}
\alias{coverage_daggers}
\title{Dagger or double dagger according to coverage level}
\usage{
coverage_daggers(res_df, one_dagger = 75, two_dagger = 50)
}
\arguments{
\item{res_df}{(data frame) Rrepest output with columns for coverage (cvge)}

\item{one_dagger}{(numeric) Level at which the coverage is transformed into a dagger. 75 by default.}

\item{two_dagger}{(numeric) Level at which the coverage is transformed into a double dagger. 50 by default}
}
\value{
Dataframe with daggers or double daggers in coverage column
}
\description{
Transform coverage columns in Rrepest to dagger and double dagger according
to the coverage (cvge) column. Default levels are 75 (dagger) and 50 (double dagger).
If coverage is above both levels, no symbol is produced (empty). Used with the coverage o
option in Rrepest set to TRUE to produce columns with coverage percentages.
}
\examples{
coverage_daggers(talis18_tt3g23o_freq, one_dagger = 95, two_dagger = 90)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rrepest.R
\name{Rrepest}
\alias{Rrepest}
\title{Estimation using replicate weights}
\usage{
Rrepest(
  data,
  svy,
  est,
  by = NULL,
  over = NULL,
  test = FALSE,
  user_na = FALSE,
  show_na = FALSE,
  flag = FALSE,
  fast = FALSE,
  tabl = FALSE,
  average = NULL,
  total = NULL,
  coverage = FALSE,
  invert_tests = FALSE,
  save_arg = FALSE,
  cores = NULL,
  se_zero2na = FALSE,
  ...
)
}
\arguments{
\item{data}{(data frame) Data to analyse.}

\item{svy}{(string) Declares the survey settings. It must be equal to one of the following: ALL, IALS, ICCS, ICILS, IELS, PBTS, PIAAC, PIRLS, PISA, PISAOOS, PISA2015, SSES, SSES2023, SVY, TALISSCH, TALISTCH, TALISEC_LEADER, TALISEC_STAFF, TIMSS.}

\item{est}{(est function) Specifies the estimates of interest. It has three arguments: statistics type, target variable, and an (optional) regressor list in case of a linear regression.}

\item{by}{(string vector) Produces separate estimates by levels of the variable(s) specified.}

\item{over}{(string vector) Requests estimates to be obtained separately for each level of the categorical variable(s) specified.}

\item{test}{(bool) If TRUE: computes the difference between estimates obtained for the lowest and highest values of the 'over' variable(s) (see 'over' option above). Useful for testing differences between dependent samples (e.g. female-male).}

\item{user_na}{(bool) If TRUE: shows the nature of user-defined missing values.}

\item{show_na}{(bool) If TRUE: includes missing values (i.e. NAs) when estimating frequencies for the variable of interest.}

\item{flag}{(bool) If TRUE: replaces estimation results that are based on fewer observations than required for reporting with NaN.
When used with the PIAAC survey settings, checks if each estimation result is based on at least 30 observations.
When used with the PISA, PISAOOS, PISA2015 survey settings, checks if each estimation result is based on at least 30 observations and 5 schools.
When used with the TALISSCH survey settings, checks if each estimation result is based on at least 10 schools.
When used with the TALISTCH survey settings, checks if each estimation result is based on at least 30 observations and 10 schools.}

\item{fast}{(bool) If TRUE: computes estimates using only 6 replicated weights.}

\item{tabl}{(bool) If TRUE: creates customisable and transferable tables using the flextable R package.}

\item{average}{(grp function) Computes an arithmetic (or weighted) average. It has three arguments: name of the average, column/variable used for computing the average, and rows/observations included in the average.}

\item{total}{(grp function) Computes an average weighted by the estimated size of the target population covered.}

\item{coverage}{(bool/numeric) TRUE: shows a column next to se. Numeric: shows NaN if below the specified coverage threshold.}

\item{invert_tests}{(bool) Inverts test columns from Rrepest when test = TRUE, based on "b." and "se." in the column names, and by multiplying "b." by -1.}

\item{save_arg}{(bool) TRUE: returns a named list with the estimation data frame and all arguments used in Rrepest.}

\item{cores}{(numeric) NULL: recruits max-1 cores when doing PVs; otherwise, uses the specified number of cores for PVs.}

\item{se_zero2na}{(bool) TRUE: masks SE of 0 as NA for mean, meanpct, or freq equal to 1, 100, or 0.}

\item{...}{Other optional parameters include:
isced = Filters the data used for analysis by ISCED level (e.g. isced = 2),
n.pvs = Customizes the number of plausible values used in the estimation (e.g. n.pvs = 5),
cm.weights = Customizes the weights used in the estimation (e.g. cm.weights = c("finw",paste0("repw",1:22))),
var.factor = Customizes the variance factor used in the estimation (e.g. var.factor = 1/(0.5^2)),
z.score = qnorm(1-0.05/2)}
}
\value{
Data frame containing estimation "b." and standard error "se.".
}
\description{
Estimates statistics using replicate weights (Balanced Repeated Replication (BRR) weights,
Jackknife replicate weights,...), thus accounting for complex survey designs in the estimation
of sampling variances. It is designed specifically to be used with the data sets produced by
the Organization for Economic Cooperation and Development (OECD), some of which include the
Programme for the International Assessment of Adult Competencies (PIAAC), Programme for
International Student Assessment (PISA) and Teaching and Learning International Survey (TALIS)
data sets, but works for any educational large-scale assessment and survey that uses replicated
weights. It also allows for analyses with multiply imputed variables (plausible values);
where plausible values are used, average estimator across plausible values is reported and
the imputation error is added to the variance estimator.
}
\examples{
data(df_pisa18)

Rrepest(data = df_pisa18,
svy = "PISA2015",
est = est("mean","AGE"),
by = c("CNT"))



}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/weighted_multivariate_corr_cov.R
\name{weighted.corr.cov.n}
\alias{weighted.corr.cov.n}
\title{Multivariate correlation and covariance}
\usage{
weighted.corr.cov.n(
  data,
  x,
  w = rep(1, length(data[x[1]])),
  corr = TRUE,
  na.rm = TRUE
)
}
\arguments{
\item{data}{(data frame) Data to analyse}

\item{x}{(string vector) Variables of interest for which to compute the correlation/covariance}

\item{w}{(string) Name of the numeric variable representing the weights}

\item{corr}{(bool) if TRUE: Computes correlation; if FALSE: Computes covariance}

\item{na.rm}{(bool) if TRUE: Excludes missing values before computing the correlation/covariance.}
}
\value{
Data frame containing each pairwise bivariate correlation/covariance
}
\description{
Computes multivariate correlation and covariance for the variables of interest.
}
\examples{
data(df_talis18)

weighted.corr.cov.n(df_talis18,c("tt3g03","tt3g06a2","tt3g06b1"),"tchwgt")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R, R/oecd_data.R
\docType{data}
\name{df_talis18}
\alias{df_talis18}
\title{Teaching and Learning International Survey (TALIS) 2018 noisy data subset}
\format{
\subsection{\code{df_talis18}}{

A data frame in tibble format with 496 rows and 548 columns from which only some relevant variables are listed below.
For a detailed description of all variables go to https://www.oecd.org/en/data/datasets/talis-2018-database.html#data
\describe{
\item{idlang}{Country language}
\item{cntry}{Country ISO 3166-1 alpha-3 codes.}
\item{tt3g01}{Gender}
\item{tt3g02}{Age}
\item{tt3g03}{Highest level of formal education completed}
...
}
}

A data frame with 548 rows/observations and 496 columns/variables.
}
\source{
\url{https://www.oecd.org/en/data/datasets/talis-2018-database.html#data}
}
\usage{
df_talis18

data(df_talis18)
}
\description{
A subset of noisy data from the Teaching and Learning International Survey (TALIS) 2018 cycle
for Mexico, Italy, and France.

This dataset is a subset of the TALIS 2018 database produced by the OECD for the countries of France, Italy, and Mexico.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/coverage_pct.R
\name{coverage_pct}
\alias{coverage_pct}
\title{Coverage percentage  (1 - mean(is.na)) * 100}
\usage{
coverage_pct(df, by, x, w = NULL, limit = NULL)
}
\arguments{
\item{df}{(data frame) Data to analyse.}

\item{by}{(string vector) Variable(s) used to tabulate the variable of interest.}

\item{x}{(string) Variable of interest for which to compute the number of valid (i.e. non-missing) observations.}

\item{w}{(string) Vector of weights.}

\item{limit}{(numeric) Threshold below which the value will be TRUE.}
}
\value{
Data frame containing the number of valid (i.e. non-missing) observations for the variable of interest.
}
\description{
Computes the coverage percentage for the column/variable of interest.
}
\examples{
data(df_pisa18)
data(df_talis18) 

coverage_pct(df = df_pisa18, by = "cnt",x = "st003d03t")
coverage_pct(df = df_talis18, by = "cntry",x = "tt3g01", w = "TCHWGT")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/weighted_sdev.R
\name{weighted.std}
\alias{weighted.std}
\title{Weighted standard deviation}
\usage{
weighted.std(x, w, na.rm = TRUE)
}
\arguments{
\item{x}{(numeric vector) Variable of interest for which to compute the standard deviation.}

\item{w}{(numeric vector) Vector with the weights.}

\item{na.rm}{(bool) if TRUE: Excludes missing values before computing the standard deviation}
}
\value{
Scalar containing the standard deviation
}
\description{
Computes the weighted standard deviation of a numeric vector.
}
\examples{
data(df_talis18)

weighted.std(df_talis18$tt3g06d1, df_talis18$tt3g06e1)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/indep_diff.R
\name{indep_diff}
\alias{indep_diff}
\title{Independent Differences of columns}
\usage{
indep_diff(df, vec_series)
}
\arguments{
\item{df}{(dataframe) Dataframe that contains the columns with b. and se. on their name to get the difference from}

\item{vec_series}{(numeric vector) Column names to get difference from, not including "b." at the start. Must have a column in dataframe also including "se.".}
}
\value{
Dataframe with extra columns containing the difference of the columns along with their standard error.
}
\description{
Get in a dataframe all bivariate combinations of differences and standard errors from columns starting with b. and se. assuming independence.
For a time series the vector must be order from oldest to newest.
}
\examples{
indep_diff(rrepest_pisa_age_isced, paste0("mean.age..ISCED level ",c("1","3A","2")))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/grouped_frequencies.R,
%   R/pv_rrepest_frequencies.R
\name{grouped_sum_freqs}
\alias{grouped_sum_freqs}
\title{Grouped frequency counts}
\usage{
grouped_sum_freqs(data, small.level, big.level, w = NULL)

grouped_sum_freqs(data, small.level, big.level, w = NULL)
}
\arguments{
\item{data}{(data frame) Data to analyze.}

\item{small.level}{(string vector) all variables to get grouped sum.}

\item{big.level}{(string vector) Must be fully contained in variables from small.level}

\item{w}{(string) Numeric variable from which to get weights (if NULL then 1).}
}
\value{
Data frame containing the frequency counts

Data frame with frequencies from the grouped sum of small.level and big.level used for getting percentages.
}
\description{
Computes a data frame with frequency counts.
}
\examples{
grouped_sum_freqs(data = mtcars,small.level = c("cyl","am"),big.level = c("cyl"))

grouped_sum_freqs(data = mtcars,small.level = c("cyl","gear"),big.level = c("cyl"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{rrepest_pisa_age_isced}
\alias{rrepest_pisa_age_isced}
\title{Rrepest table of results for PISA 2018 showing the age and completed schooling level of students' mothers}
\format{
\subsection{\code{rrepest_pisa_age_isced}}{

A data frame in tibble format with 3 rows and 7 columns:
\describe{
\item{cnt}{Country ISO 3166-1 alpha-3 codes.}
\item{b.mean.age..ISCED level 1}{Mean age of students whose mothers completed ISCED 1.}
\item{se.mean.age..ISCED level 1}{Standard error of the mean age of students whose mothers completed ISCED 1.}
\item{b.mean.age..ISCED level 2}{Mean age of students whose mothers completed ISCED 2.}
\item{se.mean.age..ISCED level 2}{Standard error of the mean age of students whose mothers completed ISCED 2.}
\item{b.mean.age..ISCED level 3A}{Mean age of students whose mothers completed ISCED 3A.}
\item{se.mean.age..ISCED level 3A}{Standard error of the mean age of students whose mothers completed ISCED 3A.}
\item{b.mean.age..ISCED level 3B, 3C}{Mean age of students whose mothers completed ISCED 3B/3C.}
\item{se.mean.age..ISCED level 3B, 3C}{Standard error of the mean age of students whose mothers completed ISCED 3B/3C.}
\item{b.mean.age..She did not complete  ISCED level 1}{Mean age of students whose mothers did not complete ISCED 1.}
\item{se.mean.age..She did not complete  ISCED level 1}{Standard error of the mean age of students whose mothers did not complete ISCED 1.}
\item{b.mean.age..(ISCED level 1-She did not complete  ISCED level 1)}{Mean age difference between ISCED 1 completed vs. non-ISCED 1 completed mothers.}
\item{se.mean.age..(ISCED level 1-She did not complete  ISCED level 1)}{Standard error of the mean age difference between ISCED 1 completed vs. non-ISCED 1 completed mothers.}
...
}
}
}
\source{
\url{https://www.oecd.org/en/data/datasets/pisa-2018-database.html#data}
}
\usage{
rrepest_pisa_age_isced
}
\description{
A table of results from Rrepest of the mean age of students broken down by
completed schooling level of students' mothers for PISA 2018.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/weighted_covariance.R
\name{weighted.cov}
\alias{weighted.cov}
\title{Weighted bivariate covariance}
\usage{
weighted.cov(x, y, w, na.rm = TRUE)
}
\arguments{
\item{x}{(numeric vector) Variable of interest x for computing the covariance}

\item{y}{(numeric vector) Variable of interest y for computing the covariance}

\item{w}{(numeric vector) Vector with the weights}

\item{na.rm}{(bool) if TRUE: Excludes missing values before computing the covariance}
}
\value{
Scalar containing the covariance
}
\description{
Computes the weighted covariance coefficient of two numeric vectors.
}
\examples{
data(df_talis18) 

weighted.cov(x = df_talis18$tt3g06b2, y = df_talis18$tt3g06c2, w = df_talis18$tchwgt)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/average.R
\name{average_groups}
\alias{average_groups}
\title{Group Averages}
\usage{
average_groups(
  res,
  data = NULL,
  group,
  by = NULL,
  over = NULL,
  est = NULL,
  svy = NULL,
  user_na = FALSE,
  ...
)
}
\arguments{
\item{res}{(dataframe) Data frame of results, including estimates and standard errors to be averaged.}

\item{data}{(dataframe) Data frame containing replicated weights.}

\item{group}{(grp function) Function used to compute grouped averages; takes arguments group.name, column, and cases, and appends results at the end of the data frame.}

\item{by}{(string vector) Variable(s) used to break down the results.}

\item{over}{(vector string) Variable(s) over which the analysis is performed.}

\item{est}{(est function) Estimation function; takes arguments what = estimate, tgt = target, and rgr = regressor.}

\item{svy}{(string) Name of the survey/project to analyse (e.g. TALISSCH, TALISTCH).}

\item{user_na}{(bool) If TRUE, shows the nature of user-defined missing values for the grouping variable(s).}

\item{...}{Additional arguments such as na_to_zero: (Bool) TRUE → treats NA values as zero for simple average calculations.}
}
\value{
Dataframe with averages or weighted averages (totals) appended in the last rows for the selected cases.
}
\description{
Group Averages
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pv_rrepest_lin_reg.R
\name{format_data_cont_vars}
\alias{format_data_cont_vars}
\title{Format continuous variables as numeric for Rrepest}
\usage{
format_data_cont_vars(df, cont.vars)
}
\arguments{
\item{df}{(data frame) Data to be analysed.}

\item{cont.vars}{(string vector) continuous variables for analysis}
}
\value{
Data frame with cantinuous variables converted to numeric for a continuous analysis (means, regression, etc.)
}
\description{
Format continuous variables as numeric for Rrepest
}
\examples{
format_data_cont_vars(mtcars,"hp")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/weighted_variance.R
\name{weighted.var}
\alias{weighted.var}
\title{Weighted variance}
\usage{
weighted.var(x, w, na.rm = TRUE)
}
\arguments{
\item{x}{(numeric vector) Variable of interest for which to compute the variance}

\item{w}{(numeric vector) Vector with weights}

\item{na.rm}{(bool) if TRUE: Excludes missing values before computing the variance}
}
\value{
Scalar containing the variance
}
\description{
Computes the weighted variance of a numeric vector.
}
\examples{
data(df_talis18) 

weighted.var(df_talis18$tt3g06d1, df_talis18$tt3g06e1)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inv_test.R
\name{inv_test}
\alias{inv_test}
\title{inv_test}
\usage{
inv_test(data, name_index)
}
\arguments{
\item{data}{(dataframe) df to analyze}

\item{name_index}{(string/numeric) name or index for the estimate (b.) columns containing the data for the test in Rrepest)}
}
\value{
Dataframe cointaining inverted test column names for "b." and "se." according to Rrepest structure and column multiplied by (-1) for "b."
}
\description{
Invert test column from Rrepest test = TRUE by name on "b." and "se." in the column name and by sign (*-1) on "b."
}
\examples{
inv_test(rrepest_pisa_age_gender, name_index = 6) 
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{rrepest_pisa_age_gender}
\alias{rrepest_pisa_age_gender}
\title{Rrepest table of results for PISA 2018 showing age and gender}
\format{
\subsection{\code{rrepest_pisa_age_gender}}{

A data frame in tibble format with 3 rows and 7 columns:
\describe{
\item{cnt}{Country ISO 3166-1 alpha-3 codes.}
\item{b.mean.age..Female}{Mean age for female students.}
\item{se.mean.age..Female}{Standar error of the mean age for female students.}
\item{b.mean.age..Male}{Mean age for male students.}
\item{se.mean.age..Male}{Standar error of the mean age for male students.}
\item{b.mean.age..(Female-Male)}{Difference of the mean age from female to male students.}
\item{se.mean.age..(Female-Male)}{Standard error of the difference of the mean age from female to male students.}
}
}
}
\source{
\url{https://www.oecd.org/en/data/datasets/pisa-2018-database.html#data}
}
\usage{
rrepest_pisa_age_gender
}
\description{
A table of results from Rrepest of the mean age of students broken down by gender for PISA 2018.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pv_rrepest_groups_argument.R
\name{grp}
\alias{grp}
\title{Group list}
\usage{
grp(group.name, column, cases, full_weight = FALSE)
}
\arguments{
\item{group.name}{(string) Name of the group to be displayed}

\item{column}{(string) Column/variable of interest to be grouped}

\item{cases}{(string vector) Rows/values to be included in the group}

\item{full_weight}{(bool) TRUE: average of the group will be weighted average}
}
\value{
List containing the arguments for the grp() function used within Rrepest() function's average() and group() options.
}
\description{
Obtains a list with the arguments for the grp() function used within Rrepest() function's average() and group() options.
}
\examples{
append(grp("OECD Average","CNTRY",c("HUN","MEX")), grp("Europe","CNTRY",c("ITA","FRA")))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/n_obs_x.R
\name{n_obs_x}
\alias{n_obs_x}
\title{Number of valid (i.e. non-missing) observations for column/variable x}
\usage{
n_obs_x(df, by, x, svy = NULL)
}
\arguments{
\item{df}{(data frame) Data to analyse with lowercase column names.}

\item{by}{(string vector) Variable(s) used to tabulate the variable of interest.}

\item{x}{(string) Variable of interest for which to compute the number of valid (i.e. non-missing) observations.}

\item{svy}{(string) Survey setting, must be one of the following: ALL, IALS, ICCS, ICILS, IELS, PBTS, PIAAC, PIRLS, PISA, PISAOOS, PISA2015, SSES, SSES2023, SVY, TALISSCH, TALISTCH, TALISEC_LEADER, TALISEC_STAFF, TIMSS.}
}
\value{
Data frame containing the number of valid (i.e. non-missing) observations for the variable of interest.
}
\description{
Computes the number of valid (i.e. non-missing) observations for the column/variable of interest.
}
\examples{
n_obs_x(df = df_talis18, by = "cntry",x = "tt3g01", svy = "TALISTCH")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{talis18_tt3g23o_freq}
\alias{talis18_tt3g23o_freq}
\title{Rrepest table of results for TALIS 2018 showing a frequency for other areas of professional development}
\format{
\subsection{\code{talis18_tt3g23o_freq}}{

A data frame in tibble format with 3 rows and 7 columns:
\describe{
\item{cntry}{Country ISO 3166-1 alpha-3 codes.}
\item{b.tt3g23o.No}{Percentage of teachers with No other areas of professional development}
\item{se.tt3g23o.No}{Standard error of th percentage of teachers with No other areas of professional development}
\item{cvge.tt3g23o.No}{Coverage of the percentage of teachers with No other areas of professional development}
\item{b.tt3g23o.Yes}{Percentage of teachers with other areas of professional development}
\item{se.tt3g23o.Yes}{Standard error of th percentage of teachers with other areas of professional development}
\item{cvge.tt3g23o.Yes}{Coverage of the percentage of teachers with other areas of professional development}
}
}
}
\source{
\url{https://www.oecd.org/en/data/datasets/talis-2018-database.html#data}
}
\usage{
talis18_tt3g23o_freq
}
\description{
A table of results from Rrepest for professional development broken down by
completed schooling level of students' mothers for the Teaching and
Learning International Survey (TALIS) 2018 cycle
}
\keyword{datasets}
